/* dirent.h -- Win32 ported versions of opendir, readdir, and closedir.
 * (c) 2001 Columbia University. All rights reserved.
*/
#ifndef DIRENT_H_
#define DIRENT_H_
#ifdef __cplusplus
extern "C" {
#endif 

struct dirent {
  char* d_name; 
};

typedef struct  {
  HANDLE hwnd;
  WIN32_FIND_DATA data;
  struct dirent dent;
  int reserved;
} DIR;


/* opendir
 *  Given a directory name (without trailing slash) returns
 *  a handle to it, that can be used for subsequent calls to
 *  readdir and closedir.
 *
 * Return Value is null if dir is not found. 
 * Also, the internal contents of DIR is not for use by the caller. It
 * should be treated as opaque.
*/
DIR* opendir(const char* dirname);

/* readdir
 *  Iterate through dir, and return all entries.
 *
 * Return value is null if end of dir is reached. Also, the pointer
 * returned must be treated read only. Subsequent calls may change
 * the value of the pointer.
*/
struct dirent* readdir(DIR *dir);  

/* closedir
 *  cleanup resources allocated to dir, by a call to opendir.
 */
void closedir(DIR *dir);


#ifdef __cplusplus
};
#endif
#endif
