#ifndef _PTHREAD_H
#define _PTHREAD_H

#include "sys/types.h"
#include "sysdep.h"

#define pthread_t HANDLE
#define pthread_mutex_t HANDLE
#define pthread_cond_t HANDLE
#define pthread_condattr_t HANDLE

#define PTHREAD_MUTEX_INITIALIZER    0       /* = DEFAULTMUTEX */

#define PTHREAD_COND_INITIALIZER     0          /* = DEFAULTCV */

#define WEXITSTATUS(s) 0
#define WTERMSIG(s)    0
#define WIFSIGNALED(s) 0
#define WIFEXITED(s)   0
#define WCOREDUMP(s)   0

int kill(int pid, int flag);

/* 
* The definition is needed for pthread_create_detached.
*/
#define pthread_attr_t void

#ifdef _DEBUG
/* Include debug version of pthread_* */

#define pthread_create(th,attr,start_func,arg)              \
      pthread_create_d(th, attr, start_func, arg, #start_func, \
                       __FILE__, __LINE__ )
#define pthread_create_detached(th,attr,start_func,arg)              \
      pthread_create_detached_d(th, attr, start_func, arg, #start_func, \
                       __FILE__, __LINE__ )
#define pthread_join(p,n)         pthread_join_d(p, n, #p, __FILE__, __LINE__) 
#define pthread_exit(x)           pthread_exit_d(x)

#define pthread_cancel(x)         pthread_cancel_d(x)
#define pthread_equal(x,y)          pthread_equal_d(x,y)
#define pthread_mutex_init(m,a)   pthread_mutex_init_d(m, a, #m, __FILE__, __LINE__)
#define pthread_mutex_destroy(m)   pthread_mutex_destroy_d(m, #m, __FILE__, __LINE__) 
#define pthread_mutex_unlock(m)    pthread_mutex_unlock_d(m, #m, __FILE__,__LINE__)
#define pthread_mutex_lock(m)      pthread_mutex_lock_d(m, #m, __FILE__,__LINE__)

#define pthread_cond_init(c,a)     pthread_cond_init_d(c, a, #c, __FILE__, __LINE__)
#define pthread_cond_destroy(m)    pthread_cond_destroy_d(m, #m, __FILE__, __LINE__) 
#define pthread_cond_timedwait(c,m,t) pthread_cond_timedwait_d(c,m,t,#c,#m,__FILE__,__LINE__)
#define pthread_cond_wait(c,m)     pthread_cond_wait_d(c,m,#c,#m,__FILE__,__LINE__)
#define pthread_cond_signal(c)     pthread_cond_signal_d(c,#c,__FILE__,__LINE__)
#define pthread_cond_broadcast(c)  pthread_cond_broadcast_d(c,#c,__FILE__,__LINE__)

#else
/* Include release version of pthread_ */


#define pthread_create(th,attr,start_func,arg)              \
      pthread_create_r(th, attr, start_func, arg)
#define pthread_create_detached(th,attr,start_func,arg)     \
      pthread_create_detached_r(th, attr, start_func, arg)

#define pthread_cancel(x)         pthread_cancel_r(x)
#define pthread_equal(x,y)          pthread_equal_r(x,y)
#define pthread_join(p,n)         pthread_join_r(p, n) 
#define pthread_exit(x)           pthread_exit_r(x);

#define pthread_mutex_init(m,a)   pthread_mutex_init_r(m, a)
#define pthread_mutex_destroy(m)   pthread_mutex_destroy_r(m) 
#define pthread_mutex_unlock(m)    pthread_mutex_unlock_r(m)
#define pthread_mutex_lock(m)      pthread_mutex_lock_r(m)

#define pthread_cond_init(c,a)     pthread_cond_init_r(c, a)
#define pthread_cond_destroy(m)    pthread_cond_destroy_r(m) 
#define pthread_cond_timedwait(c,m,t) pthread_cond_timedwait_r(c,m,t)
#define pthread_cond_wait(c,m)     pthread_cond_wait_r(c,m)
#define pthread_cond_signal(c)     pthread_cond_signal_r(c)
#define pthread_cond_broadcast(c)  pthread_cond_broadcast_r(c)

#endif

/* Prototypes of pthread functions */
/* Debug */
int pthread_create_d(pthread_t *th, const void *attr, 
                     void * (*start_func)(void *), void *arg,
                     char *th_name, char* file, int line);
int pthread_create_detached_d(pthread_t *th, const void *attr, 
                     void * (*start_func)(void *), void *arg,
                     char *th_name, char* file, int line);
void pthread_cancel_d(pthread_t th);
int  pthread_equal_d(pthread_t t, pthread_t t1);
int pthread_exit_d(void * status);
int pthread_join_d(pthread_t th, void ** value_ptr,
                   char *th_name, char *file, int line) ;
int pthread_mutex_init_d(pthread_mutex_t *m, const void *attr,
                         char *m_name, char *file, int line) ;
int pthread_mutex_destroy_d(pthread_mutex_t *m,
                            char *m_name, char *file, int line) ;
int pthread_mutex_lock_d(pthread_mutex_t *m,
                         char *m_name, char *file, int line);
int pthread_mutex_unlock_d(pthread_mutex_t *m,
                           char *m_name, char *file, int line) ;
int pthread_cond_init_d(pthread_cond_t *c, const pthread_condattr_t *attr,
                        char *c_name, char *file, int line) ;
int pthread_cond_destroy_d(pthread_cond_t *c,
                           char *c_name, char *file, int line) ;
int pthread_cond_signal_d(pthread_cond_t *c,
                          char *c_name, char *file, int line) ;
int pthread_cond_broadcast_d(pthread_cond_t *cond,
                             char *c_name, char *file, int line) ;
int pthread_cond_timedwait_d(pthread_cond_t *cond,
          pthread_mutex_t *mutex,
          const struct timespec *abstime,
          char *c_name, char *m_name, char *file, int line) ;
int pthread_cond_wait_d(pthread_cond_t *cond,
          pthread_mutex_t *mutex,
          char *c_name, char *m_name, char *file, int line) ;


/* Release */
pthread_t pthread_self(void);
int pthread_create_r(pthread_t *th, const void *attr, 
                     void * (*start_func)(void *), void *arg);
int pthread_create_detached_r(pthread_t *th, const void *attr, 
                     void * (*start_func)(void *), void *arg);
void pthread_cancel_r(pthread_t th);
int  pthread_equal_r(pthread_t t, pthread_t t1);
int pthread_exit_r(void * status);
int pthread_join_r(pthread_t th, void ** value_ptr) ;
int pthread_mutex_init_r(pthread_mutex_t *m, const void *attr) ;
int pthread_mutex_destroy_r(pthread_mutex_t *m) ;
int pthread_mutex_lock_r(pthread_mutex_t *m);
int pthread_mutex_unlock_r(pthread_mutex_t *m) ;
int pthread_cond_init_r(pthread_cond_t *cond, const pthread_condattr_t *attr);
int pthread_cond_destroy_r(pthread_cond_t *cond) ;
int pthread_cond_signal_r(pthread_cond_t *cond) ;
int pthread_cond_broadcast_r(pthread_cond_t *cond) ;
int pthread_cond_timedwait_r(pthread_cond_t *cond,
          pthread_mutex_t *mutex,
          const struct timespec *abstime) ;
int pthread_cond_wait_r(pthread_cond_t *cond,
          pthread_mutex_t *mutex) ;


#endif
