#! /bin/sh
# dist.sh: Build a distribution file
# usage: dist.sh topsrcdir distname files...

# Copyright (c)1999-2000 by Columbia University; 
# All rights reserved 

# Explanation of how this works:
# 1. Clean up any old files from a previous run of dist on this DISTNAME
# 2. Create a directory named DISTNAME.
# 3. For every file specified in $@, create a subdirectory of DISTNAME for
#    the file to live in.
# 4. For every file specified in $@, copy it into the appropriate
#    subdirectory of the DISTNAME directory.  If the file in $@ is actually
#    a directory, copy the directory recursively.  For files, first look
#    for the file in the source tree (rooted under $TOPSRCDIR), then in
#    the object tree (rooted under .).  Directories are only copied from
#    $TOPSRCDIR.
# 5. Tar-gzip the directory.
# 6. Remove the directory.

if test $# -lt 3; then
    echo usage: $0 topsrcdir distname files... >&2
    exit 1
fi

TOPSRCDIR=$1
shift

DISTNAME=$1
shift

echo Building ${DISTNAME}.tar.gz...
rm -f ${DISTNAME}.tar ${DISTNAME}.tar.gz
rm -rf ${DISTNAME}
mkdir ${DISTNAME}

echo Creating directories...
dirs=`for f in $@; do dirname $f; done | sort | uniq`
for dir in $dirs; do
	echo $dir ;
	mkdir -p "${DISTNAME}/$dir" 2>/dev/null
done

echo Copying files...
for file in $@; do
    echo $file
    if test -d "${TOPSRCDIR}/$file"; then 
        cp -pr "${TOPSRCDIR}/$file" "${DISTNAME}/$file";
    else
        cp -p "${TOPSRCDIR}/$file" "${DISTNAME}/$file" 2>/dev/null \
            || cp -p "$file" "${DISTNAME}/$file";
    fi
done

echo Creating tar file...
tar cvf ${DISTNAME}.tar ${DISTNAME}
gzip -9 ${DISTNAME}.tar
rm -rf ${DISTNAME}
