#! /bin/sh
# use this part of the file to install Tcl because
# you can not install Tcl using a Tcl script.
# Use the commented blackslash \  line before every
# shell command.
# Logic to install Tclsh:
# Set the path to include std paths like /bin, /usr/bin and
# /usr/local/path
# If tclsh exists in any of these paths and is Tcl8.3
# then you don't have to install Tcl.
# otherwise install (gunzip+tar) the Tcl .tar.gz file
# for the appropriate platform. Set PATH and LD_LIBRARY_PATH
# environment variables to include this installed path.
# XXX Should we create softlinks at /usr/local/cinema-1.18
# or /usr/local/Tcl8.3 pointing to the installed directory?
#
# \
PATH=$PATH:/bin:/usr/bin:/usr/local/bin:/opt/tcl8.3/bin:/opt/apache/bin:/opt/mysql/bin; export PATH; \
LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/opt/tcl8.3/lib:/opt/apache/lib:/opt/mysql/lib; \
export LD_LIBRARY_PATH; \
installtcl=0; \
if [ -f /bin/tclsh -o -f /usr/bin/tclsh -o -f /usr/local/bin/tclsh -o -f /opt/tcl8.3/bin ]; \
then echo "Tcl is already installed"; \
tclversion=`echo "puts \\\$tcl_version" | tclsh`; \
echo "Tcl version is $tclversion"; \
if [ $tclversion != "8.3" ]; then installtcl=1; fi; \
else echo "Tcl8.3 is not installed in standard path /bin, /usr/bin, /opt/tcl8.3/bin, or /usr/local/bin"; \
installtcl=1; \
fi; \
if [ $installtcl -eq 1 ]; \
then os=`uname -s`; echo "OS is $os"; \
if [ -f support/tcl8.3-$os.tar.gz ]; \
then \
echo "Using support/tcl8.3-$os.tar.gz"; \
if [ -f tcl8.3 ]; \
then echo "Tcl 8.3 is found in tcl8.3/ directory"; \
else \
gunzip -c support/tcl8.3-$os.tar.gz | tar xvf - ; \
fi; \
PATH=`pwd`/tcl8.3/bin:$PATH; export PATH; \
LD_LIBRARY_PATH=`pwd`/tcl8.3/lib:$LD_LIBRARY_PATH; \
export LD_LIBRARY_PATH; \
echo "PATH=$PATH"; \
echo "LD_LIBRARY_PATH=$LD_LIBRARY_PATH"; \
else echo "No support file for Tcl8.3."; \
if [ `uname -s` = "SunOS" ]; \
then echo "Trying to install pkgadd tcl8.3..."; \
pkgadd tcl83; \
else \
if [ `uname -s` = "Linux" ]; \
then echo "Trying to install rpm -i tcl8.3..."; \
rpm -i tcl8.3; \
else \
echo "No Tcl found in your path. Please install it and re-run the script."; \
exit; \
fi; \
fi; \
cp /opt/tcl8.3/lib/*.so* /usr/lib; \
PATH=/opt/tcl8.3/bin:$PATH; export PATH; \
LD_LIBRARY_PATH=/opt/tcl8.3/lib:$LD_LIBRARY_PATH; \
export LD_LIBRARY_PATH; \
echo "PATH=$PATH"; \
echo "LD_LIBRARY_PATH=$LD_LIBRARY_PATH"; \
fi; \
fi

# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

# Above lines are more portable than hardcoding the tclsh path.

# Highlight the test
proc font_underline {str} {
  return "\[4m$str\[m"
}
proc font_bold {str} {
  return "\[1m$str\[m"
}
proc font_reverse {str} {
  return "\[7m$str\[m"
}

# Installation script for cinema system for
# binary distribution. It can also install other external
# dependencies e.g., mysqld and snmpd.

# Various directories in the distribution.
set applications [list \
   "sipd" "rtspd" "sip323" "sipconf" "sipum" \
   "sipua" "sipvxml" "mysql" "snmp" "web"]

# The note when installation is successful this is
# printed out to the user.
set reportfooter "  Please report any installation problem to 
    \"CINEMA Support\" <[font_bold "cinema-support@cs.columbia.edu"]>"

# set the cinema_url for web pages
set cinema_url ""

# Wait for the user to press the return key.
proc waitforuser {} {
    puts -nonewline "\nPress <return> to continue ..."
    flush stdout
    gets stdin dummy
}

# Get a valid directory or file path from the user.
# If the user enters an incorrect path that does not
# exits it prompts again. XXX should we make it
# retry for few times and then abort. It takes the default
# value if the user hits return without any path.
proc getpath {type def} {
    puts "$type"
    puts -nonewline "> "
    flush stdout
    while { [gets stdin dummy] > 0} {
        if { [file exists "$dummy"] == 1} {
            return $dummy
        } else {
            puts "This path does not exists. Enter another."
            puts -nonewline "> "
            flush stdout
        }
    }
    if { $def != "" } {
      puts "Assuming default value as\n $def"
    }
    return $def
}

# Get a string from stdin. It returns the default value
# if user hits return without any string.
proc getstring {type def} {
    puts "$type"
    puts -nonewline "> "
    flush stdout
    if { [gets stdin dummy] > 0} {
        return $dummy
    } else {
        puts "Assuming default value as $def"
        return $def    
    }
}

# Check if the given path of the module is correct or not.
# module is any external library or tools used by the
# system. e.g., mysqld. 
# This function tries to get the path and if it fails it
# prompts the user to install the particular module
# before running the script. It relies on Unix which command
# to locate the module's executable so it MUST be in your
# PATH env variable before you run the script.
# For windows version it prompts the user to give the path.
proc checkmodulepath {def {name ""} {url ""}} {
  global platform
  set thispath  $def
  set binary [file tail $def]
  if { $name == "" } {
      set name $binary
  }
  if { $url == "" } {
      set url "README"
  }
  if { $platform == "windows" } {
    set thispath [getpath "Enter the path for $name, e.g., /progra~1/$name/bin/$binary.exe." "$def"]
    if {![file exists $thispath]} {
      puts "$thispath does not exists. Please try again."
      exit 1
    }
  } else {
   if { [catch {set thispath [exec which $binary]}] == 0 && 
        [lindex [split $thispath " "] 0] != "no" } {
       puts "$name is detected at $thispath.
If this is incorrect terminate the script now."
   } else {
       puts "$name is not installed. Please install it (see $url),
add it in your PATH environment variable
and re-run this installation script."
       exit 1
   }
  }
  return $thispath
}

# Ask the user about whethe he has already installed the
# external tool or not. If not ask him whether he wants the
# script to install the tool or not on this machine.
# It returns "exists" if the tool exists,
# "no-install" if the tool does not exist but the
# user wants it to be not installed, and "install" in the
# other case.
proc ask_about_external_tool {tool comment} {
    global isroot
    global typical
    puts "$comment"

    set dummy "n"
    if { [catch {set thispath [exec which $tool]}] == 0 && 
    [lindex [split $thispath " "] 0] != "no" } {
        puts "$tool is found at $thispath"
        set dummy "y"
    }

    if { [file exists "/opt/cinema/$tool"] } {
        puts "$tool is found at /opt/cinema/$tool"
        set dummy "y"
    }

    if { [file exists "/usr/local/cinema/$tool"] } {
        puts "$tool is found at /opt/cinema/$tool"
        set dummy "y"
    }

    if { [file exists "$tool"] } {
        puts "$tool is found at local directory $tool"
        set dummy "y"
    } 

    set dummy [getstring "Have you already installed $tool (y/n).
[font_bold "Default is $dummy"]" $dummy]
    while { $dummy != "y" && $dummy != "n" } {
        puts "Please answer y or n"
        puts -nonewline "> "
        flush stdout
        gets stdin dummy
    }

    if { $dummy == "y" } {
        return "exists"
    } elseif { $dummy == "n" } {
        set dummy "y"
        if { $tool == "snmp" } {
            set dummy "n"
        }
        set dummy [getstring "Do you want me to install $tool on this machine (y/n).
[font_bold "Default is $dummy"]" $dummy]
        while { $dummy != "y" && $dummy != "n" } {
            puts "Please answer y or n"
            puts -nonewline "> "
            flush stdout
            gets stdin dummy
        }

        if { $dummy == "y" } {
            return "install"
        } else {
            return "no-install"
        }
    }
    return ""
}

# Test if the mysql is installed correctly.
# The current directory is in the mysql top install dir.
# this may return without doing anything if we do not
# want to test. It also starts mysql server.
proc test_mysql {dir} {
    catch {exec groupadd mysql}
    catch {exec useradd -g mysql mysql}
    
    catch {exec scripts/mysql_install_db}
    catch {exec chown -R root "."}
    catch {exec chown -R mysql "./data"}
    catch {exec chgrp -R mysql "."}
    catch {exec chown -R root "./bin"}
    
    if { [catch {exec bin/safe_mysqld --user=mysql &} err] != 0 } {
        puts "ERROR: Could not run safe_mysqld:\n$err"
    } else {
        puts "mysqld is running now..."
        waitforuser
    }
}

# Test if apache server is installed.
# Change the config file.
# Run the server.
proc test_apache {dir mydir} {
    global installdir
    global webhost
    global webport
    global webpath
    global isroot
    global hostname
    global adminemail
    global mysqluser
    global mysqlpasswd
    global mysqlhost
    global standard_path
    puts "Installing apache in $dir.
This may take few minutes. Please wait ..."
    if {[catch {exec ./install-bindist.sh "$mydir"} err]} {
        puts "ERROR: Could not install apache\n$err"
    }

    puts "Editing apache config file $dir/conf/httpd.conf"

    set webadmin "$adminemail"
    set webhost "$hostname"
    set webpath "$installdir/web"
    if { $isroot } {
        set webport "80"
        set webuser nobody
    } else {
        set webport "8080"
        set webuser daemon
    }

    set webport [getstring "
What port number do you want to use for the web server.
[font_bold "Default is $webport"]" $webport]
    set webuser [getstring "
Userid for running web server as. Root users should run it
as nobody while non-root users as daemon.
[font_bold "Default is $webuser"]" $webuser]
    set webadmin [getstring "
Email address of the web server administrator.
[font_bold "Default is $webadmin"]" $webadmin]
   
    set fr [open "conf/httpd.conf" r]
    set fw [open "conf/httpd.conf.cinema" w]
    if { $fr >= 0 && $fw >= 0} {
        set aliasfound 0
        while {[gets $fr line] >= 0} {
            set newline $line
            regsub -- "^Port (.*)" $line "Port $webport" newline

            set line $newline
            regsub -- "^User (.*)" $line "User $webuser" newline

            set line $newline
            regsub -- "^ServerAdmin (.*)" $line "ServerAdmin $webadmin" newline
            set line $newline
            regsub -- "DirectoryIndex index.html" $line "DirectoryIndex index.html index.cgi" newline

            set line $newline
            regsub -- "Options Indexes FollowSymLinks MultiViews" $line "Options Indexes FollowSymLinks ExecCGI MultiViews Includes" newline

            set line $newline
            regsub -- "#AddHandler cgi-script .cgi" $line "AddHandler cgi-script .cgi" newline

            set line $newline
            if {[regexp -- "Alias /cinema/" $newline]} {
                set aliasfound 1
                regsub -- "Alias /cinema/ .*" $line "Alias /cinema/ \"$webpath/\"" newline
            }

            puts $fw $newline
        }

        # Add Alias for the cgi directory.
        if { $aliasfound == 0 } {
            set al "
<IfModule mod_alias.c>
  # Alias for cinema installation.
  Alias /cinema/ \"$webpath/\"

  <Directory \"$webpath\">
    Options Indexes FollowSymLinks ExecCGI MultiViews Includes
    AllowOverride None
    Order allow,deny
    Allow from all
    DirectoryIndex index.html index.cgi
  </Directory>
</IfModule>"
            puts $fw $al
        } else {
            puts "Already alias for cinema exists"
        }
        
        # Put the CINEMA env in the web page. This will avoid the
        # need for the cinema_db.conf file. 
        if { $mysqlhost == $webhost && $standard_path == 1} {
           set surl "sql://$mysqluser:$mysqlpasswd@localhost/sip"
        } else {
           set surl "sql://$mysqluser:$mysqlpasswd@$mysqlhost/sip"
        }
        puts $fw "
# Define and pass the CINEMA env variable with the sql URL.
SetEnv CINEMA $surl
PassEnv CINEMA"

    } else {
        puts "ERROR: Could not open the httpd.conf file for reading 
or httpd.conf.cinema file for writing."
    }
    catch {close $fr}
    catch {close $fw}

    if {[catch {file copy -force -- conf/httpd.conf.cinema conf/httpd.conf} err]} {
        puts "Could not write the httpd config file...\n$err"
    } else {
        # Now try running httpd
        if { [catch {exec bin/apachectl start &} err] != 0 } {
            puts "ERROR: Could not run apachectl:\n$err"
        } else {
            set cinema_url "http://$webhost:$webport/cinema/"
            puts "httpd is running now..."
            puts "Cinema pages are installed at [font_bold $cinema_url]"
            puts "Note the terminating /. The URL will not work without that."
            waitforuser
        }
    }
}

# Test if SNMP is installed.
# Run the SNMP server if needed.
proc test_snmp {dir} {
  puts "Not testing if SNMP is installed correctly."
}

# Install the any external tool by unpacking it in the
# current directory. It can invoke install_mysql or install_apache
# internally.
# It sets the global variable indicating the mysql paths
# libs and bins. 
proc install_external {tool} {
    global installdir
    global platform
    global tcl_platform
    global gunzippath
    global tarpath
    global os
    global standard_path

    if { $platform == "unix" } {
        set ver $tcl_platform(osVersion)
        set os_small [string tolower $os]
        if { $os_small == "sunos" } {
            set os_small "solaris"
        }

        if { $standard_path == 0 } {
            set files [glob -type {f r} -nocomplain -- "$installdir/support/$tool*$os_small*.tar.gz"]
            if {[llength $files] <= 0} {
                set files [glob -type {f r} -nocomplain -- "$installdir/support/$tool*$os*.tar.gz"]
            }
            if {[llength $files] <= 0} {
                puts "ERROR: I don't have $tool binary for $os platform."
                exit 1
            }
            if {[llength $files] == 1} {
                set file [lindex $files 0]
            } else {
                set file [lindex $files 0]
                set p ""
                for {set k 0} {$k < [llength $files]} {incr k} {
                    set p "$p [expr $k + 1] [lindex $files $k]\n"
                }
                set file [getpath "
                Please select one of the files from the following. 
$p Specify the complete path of the file.
[font_bold "Default is $file"]" $file]
            }
        }
        # end of non stanrad path if

        if {$standard_path == 1 || [file exists "$file"]} {

            if { $standard_path == 1 } {
                set tooldir "/opt/$tool"
            } else {
              set tooldir "[file rootname [file tail $file]]"
              set tooldir "[file rootname $tooldir]"

              # Check if directory exists
              set ans "y"
              set temp1 [glob -type {d} -nocomplain -- "$tool*"]
              if {[file isdirectory $tooldir] || [llength $temp1] > 0} {
                set ans "n"
                set ans [getstring "
Directory for installation of $tool seems to exists ($tooldir). 
This might be due to unsuccessful installation attempt.
Do you want to overwrite this (y/n).
[font_bold "Default is $ans"]" $ans]
              }

              # gunzip and tar            
              if { $ans == "y" } {
                puts "Unpacking [file tail $file]. 
This may take few minutes. Please wait..."
                if {[catch {exec $gunzippath -c "$file" | $tarpath xf - } err]!= 0 } {
                    puts "ERROR: Could not unpack $file\n$err"
                } 
              } else {
                puts "Not unpacking [file tail $file]."
              }
            }
            # Successfully extracted. 
            
            if {![file isdirectory $tooldir]} {
                # directory does not exist. check for apache as special case.
                if { $standard_path == 1 } {
                    set dirs [glob -type {d} -nocomplain -- "/opt/$tool*"]
                } else {
                    set dirs [glob -type {d} -nocomplain -- "$tool*"]
                }
                if {[llength $dirs] <= 0} {
                    puts "ERROR: Could not find the directory $tooldir."
                    exit 1
                }
                if {[llength $dirs] == 1} {
                    set tooldir [lindex $dirs 0]
                } else {
                    set tooldir [lindex $dirs 0]
                    set p ""
                    for {set k 0} {$k < [llength $dir]} {incr k} {
                        set p "$p [expr $k + 1] [lindex $dir $k]\n"
                    }
                    set tooldir [getpath "
Please select one of the directories from the following. 
$p Specify the name instead of the number.
[font_bold "Default is $tooldir"]" $tooldir]
                }

                if {![file isdirectory $tooldir]} {
                    puts "ERROR: Could not find directory $tooldir"
                    exit 1
                }
            }
            
            if {[catch {cd "$tooldir"}] == 0} {
                if { $tool == "mysql" } {
                    test_mysql $tooldir
                } elseif {$tool == "apache" } {
                    if { $standard_path == 1 } {
                        test_apache $tooldir "/opt/apache"
                    } else {
                        test_apache $tooldir "$installdir/$tooldir"
                    }
                } elseif {$tool == "snmp" } {
                    test_snmp $tooldir
                } else {
                    puts "I know nothing about $tool."
                }
                
                # XXX do we need to set the path and LD_LIBRARY_PATH here.
                # Go back to the installtion directory.
                catch {cd "$installdir"}
            } else {
                puts "ERROR: Could not change dir to $tooldir"
                    exit 1
            }
        } else {
            # File does not exist
            puts "The specified file $file does not exist or is not readable"
            exit 1
        }
            
    } elseif { $platform == "windows" } {
        # Use winzip/unzip and install it here. XXX
        puts "This installation program can not install windows applications. Please install it yourself and re-run the installation script."
        exit 1
    } else {
        puts "ERROR: I don't have $tool binaries for $platform platform.
Please install it after downloading it
and re-run this installation script."
        exit 1
    }
}

####################### Main starts here #####################################
#
# *) Get user input on the following:
# - domain name (the script tries to extract the domain automatically
#   but prompts the user to change if needed)
# - MySQL url, etc if already installed; if not then ask the user whether
#   he wants to install it on this machine.
# - SNMP path if already installed. if not then ask the user whether he
#   wants to install it on this machine.
# - Apache server path if already present. If not then ask the user whether
#   he wants to install it on this machine.
# - Path to install cinema components.
# - Administrator's email (defaults to the one who executes the script)
# - Host names/ports on which various apps will be running, sipd, sipconf,
#   sip323, sipum, rtspd, mysqld, webserver (depending on what components are
#   to be installed or present in the distribution). This should be defaulted
#   with user changing this only in expert mode.
# - License string for various apps.
# - path for top directory of voicemails.
# - Path for sendmail

# *) Install (i.e., move files) for the cinema components and external
# libraries or tools. Including the web scripts and md5string, fbsql etc
# files to web directory and audio files for voicemail prompts into
# voicemail directory.

# *) Generate config files as needed. httpd.conf, cinema_db.conf,
# run-monitor, gateways.sample, dialplan.sample, and alive.cgi. 
# (I have written another CGI script that will
# allow me to stop and start the cinema applications - sipd,
# sipconf,...etc, from web using unix rsh; but has may have security problem
# hence not good. However it can be used for common installation of
# "everything in one box".).
# Also generate resolv.conf for windows applications.

# *)Set the LD_LIBRARY_PATH (or other unix sys have other env var for
# this) to include mysql and tcl path.

# *) Create and alter the mysql database table user, for host
# privileges. Create or alter the database sip. Populate the
# global config tables for cinema, sipd and vmail.
# Populate the database for
# initial admin user. Populate database with licenses.
# Create a dummy conference for test if sipconf is installed.
# All this is already done by the createdb script. This script
# just needs to invoke that.

# *) Start mysqld server.(if needed)

# *) Run the cinema apps with -h option to see if installed correctly.
# Use http get to see if the HTTP pages where installed correctly.

# *) Start the apps using run-monitor...(??)

# *) display a note to send bug report to cinema-support@
#

puts "-----------------------------------------------------------"
puts "[font_reverse "CINEMA, (c) 2001, Columbia University"], installation script."
puts "  If the installation gets terminated abnormally, please
  uncompress the binary distribution again and then retry.

  $reportfooter
"
 
# Check if the command is invoked as ./install.tcl.
# Do not allow invoking it from any other place. This
# makes installation easy. And we readily know the
# installation directory.
if { $argv0 != "./install.tcl" } {
    puts "Change your current directory to the directory where
  install.tcl is present and invoke the script as 
    $ ./install.tcl           -- on Unix
    $ tclsh83 ./install.tcl    -- on Windows"
    exit 1;
}

# Prompt for prompt
puts "
The installation script tries to find the most appropriate
config options for your set up. However you will be prompted
to modify these values. [font_bold "For most installation we recommend that
you choose the default value by just pressing the <return> key
when prompted."]"

# wait for user to read this.
waitforuser

# Prompt the user for various types of installation.
# 1. Installation by root. Installs all in the root directory.
# 2. Typical installation, installs all in the local directory.
# 3. Expert installation. Installs the components needed on this machine.

# Try to guess if user is a root or not.
set isroot 1
if {[info exists tcl_platform(platform)] && 
 $tcl_platform(platform) == "windows" && 
 [info exists tcl_platform(user)] && 
 $tcl_platform(user) != "Administrator"} {
    set isroot 0
} elseif {[info exists tcl_platform(platform)] && 
 $tcl_platform(platform) == "unix" && 
 [info exists tcl_platform(user)] && 
 $tcl_platform(user) != "root"} {
    set isroot 0
}

if { $isroot } { set op 1 } else { set op 2 }

set op  [getstring "
CINEMA allows different modes of installation. 
1 [font_bold "Typical installation by root"] or administrator will install 
  all the files in the standard paths on this machine.
  For this mode you should have permissions to write system files.
2 [font_bold "Typical installation by a non-root"] user will install all
  the files in the local paths inside this directory on
  this machine.
3 [font_bold "Expert installation by root"] prompts you to specify the path 
  for installation of various components and it also
  allows you to install different components (sipd, sipum, etc)
  on different machines. However, you should be familiar with
  the installation documentation to choose this option.
  For this mode you should have permissions to write system files.
4 [font_bold "Expert installation similar to (3) by non-root user"].
Recommended mode is [font_bold "$op"]" $op]
if { $op == "1" } { 
    set isroot 1
    set typical 1
} elseif { $op == "2" } {
    set isroot 0
    set typical 1
} elseif { $op == "3" } { 
    set isroot 1
    set typical 0
} elseif { $op == "4" } { 
    set isroot 0
    set typical 0
} else {
    puts "You chose an invalid option $op. Choose from 1,2,3 or 4. 
Try again..."
    exit 1
}

# Get the platform and OS.
if {[info exists tcl_platform(platform)]} {
    set platform $tcl_platform(platform)
    puts "Platform: $platform"
} else {
    set platform ""
    puts "Could not identify the platform (unix/windows)."
    set platform [getstring "
Enter the platform you are using, unix or windows" ""]
    if { $platform != "unix" && $platform != "windows" } {
      puts "Could not identify the platform (unix/windows)."
      exit 1
    }
}
if {[info exists tcl_platform(os)]} {
    set os $tcl_platform(os)
    puts "Operating System: $os"
} else {
    set os ""
    puts "Could not identify the Operating System as 
one of (SunOS, FreeBSD, Linux, Windows, Tru64)"
    set os [getstring "
Enter the Operating System you are using, it should be
one of SunOS, FreeBSD, Linux, Windows or Tru64" ""]
    if { $platform != "SunOS" && $platform != "FreeBSD"
&& $platform != "Linux" && $platform != "Windows" && $platform != "Tru64"} {
      puts "Could not identify the Operating System."
      exit 1
    }
}

# Get the current directory and set it as install directory.
set installdir [pwd]

# Initialize present[] array if the directory or component is present
foreach app $applications {
    if {[file isdirectory "$installdir/$app"]} {
        set present($app) 1 
    } else {
        set present($app) 0
    }
}
# MySQL is needed by sipd, sipum, and sipconf.
if { $present(sipd) || $present(sipum) || $present(sipconf) } {
    set present(mysql)  1
    set present(web)    1
}

# Print the list of available applications.
puts -nonewline "Available applications: "
set c 0
foreach app $applications {
    if {[file isdirectory "$installdir/$app"]} {
        puts -nonewline "$app,"
        incr c
    }
}
puts ""

# Standard path var indicates if apache and mysql are in the
# standard path or not.
set standard_path 0
if { $c == 0 } { 
    # No available application. Check for package.
    if { $isroot == 1 } {
        if { $os == "SunOS" || $os == "Linux" } {
            # Change the current dir to "/opt/cinema"
            catch {
                cd "/opt/cinema"
                set installdir [pwd]
            }

            set standard_path 1
        } elseif { $os == "FreeBSD" } {
            # Change the current dir to "/usr/local/cinema"
            catch {
                cd "/usr/local/cinema"
                set installdir [pwd]
            }

            set standard_path 1
        } else {
            puts "You do not have any application available for
installation. You should not use package or rpm installation for
$os platform. They are available only for SunOS, Linux and FreeBSD."
        }
    } else {
        puts "You do not have any available application.
If you are using the package or rpm installation
you must run this script as root."
    }
}

if { $standard_path == 1 } {
  # Initialize present[] array if the directory or component is present
  # for standard path installation.
  foreach app $applications {
      if {[file isdirectory "$installdir/$app"]} {
          set present($app) 1
      } else {
          set present($app) 0
      }
  }
  # MySQL is needed by sipd, sipum, and sipconf.
  if { $present(sipd) || $present(sipum) || $present(sipconf) } {
      set present(mysql)  1
      set present(web)    1
  }
  
  # Print the list of available applications.
  puts -nonewline "Available applications: "
  set c 0
  foreach app $applications {
      if {[file isdirectory "$installdir/$app"]} {
          puts -nonewline "$app,"
          incr c
      }
  }
  puts ""
}

# Print the installation directory for reference.
puts "Installation directory is $installdir"

# Internal tools needed for web...
if { $platform == "windows" } {
    # For windows installation take the Release version.
    set addsipuser "tools/addsipuser/Release/addsipuser.exe"
    set md5string  "tools/md5string/Release/md5string.exe"
    set random32   "tools/random32/Release/random32.exe"
    set fbsql      "tools/fbsql/Release/fbsql.dll"
} else {
    set addsipuser "tools/addsipuser/addsipuser"
    set md5string  "tools/md5string/md5string"
    set random32   "tools/random32/random32"
    set fbsql      "tools/fbsql/libfbsql.so"
}

# Get path for common utilities like gunzip, and tar
if { $platform == "unix" } {
  set gunzippath [checkmodulepath "/usr/local/bin/gunzip" "gunzip" ""]
  set tarpath [checkmodulepath "/usr/local/bin/tar" "tar" ""]
} elseif { $platform == "windows" } {
  set unzippath [checkmodulepath "c:\\windows" "unzip" ""]
}

# Try getting the local host name.
set hostname "localhost"
puts "Trying to find local host name..."
catch {set hostname [exec uname -n]}
if { $hostname == "localhost" } {
  catch {set hostname "[hostname]"}
}
puts "Host name is $hostname"

# We assume that the $hostname is of the form host.domain
# where domain is used for default value of domain for the
# installation.
set dummylist [split $hostname .]
set dummylist [lrange $dummylist 1 [llength $dummylist]]
set domain    [join $dummylist .]

# Guess the web server host name by prepending www to domain.
set dummylist [linsert $dummylist 0 www]

# Set the default web server address.
set webhost [join $dummylist .]
set webhostalias $webhost
set webport "80"

# Prompt the user to confirm the domain name for the installation.
if {$domain == "" } {
  # Try to get from /etc/resolv.conf
  catch {
      set domain [lindex [split [exec grep domain /etc/resolv.conf] " "] 1]
  }
}
if {$domain == "" } {
  set domain "wonderland.com"
}

set domain [getstring "
Enter the domain of your organization.
[font_bold "default is $domain"]" $domain]
puts ""

set adminemail "webmaster@$domain"

# Administrator's email (defaults to the one who executes the script)
# Email domain defaults to domain.
# LOGNAME is the login name of the user.
# These fields are used to populate the ADMIN emails
# whereever needed.
if { $present(web) || $present(mysql) || $present(rtspd) } {
  set emaildomain $domain
  set emaildomain [getstring "
Enter the email domain of your organization.
Usually this is the same as your domain. All registered
users should have email address as \*@emaildomain in 
local mode of operation.
[font_bold "default is $emaildomain"]" $emaildomain]
  puts ""

  set LOGNAME "sipmaster"
  catch {set LOGNAME [exec whoami]}
  set LOGNAME [getstring "
Enter the unix login id of the administrator who will
manage the CINEMA installation for your domain.
[font_bold "Default is $LOGNAME"]" $LOGNAME]

  set adminemail "$LOGNAME@$emaildomain"
  set adminemail [getstring "
Enter the email address of the administrator who will
manage the CINEMA installation for your domain.
[font_bold "Default is $adminemail"]" $adminemail]
}

# Ask the user if he has already installed mysql or not. And if not
# whether he would like to install it on this machine.

set mysqluser "root"
set mysqlhost "$hostname"
set mysqlpasswd "NULL"

if {$present(mysql) == 1} {
  set ans [ask_about_external_tool "mysql" \
"MySQL is needed for your application. 
You must install it before proceeding." ]

  if { $ans == "install" } {
      install_external "mysql"
  } elseif { $ans == "exists" } {
      # Get the password and host name for mysql.
      puts "Since you have already installed mysql 
make sure that the server mysqld is running and 
you can connect to it from this machine."

      set mysqluser [getstring "
Enter the user name used for accessing the MySQL database.
[font_bold "Default is $mysqluser"]" $mysqluser]
      puts ""
      
      set mysqlhost [getstring "
Enter the host name on which MySQL server is running.
[font_bold "Default is $mysqlhost"]" $mysqlhost]
      puts ""
      
      set mysqlpasswd [getstring "
Enter the user password for accessing mysql for user $mysqluser.
[font_bold "Default is $mysqlpasswd"]" $mysqlpasswd]
      puts ""      

  }
}

# Ask for webserver installation.
set webpath ""
if {$present(web) == 1} {
  set ans [ask_about_external_tool "http" \
"HTTP server (e.g., Apache) that supports CGI scripts
is needed for your application. This installation does not
work with Windows IIS, so if you are using windows machine
then you should install Apache Web server. 
Also, if you have web server running somewhere but if [font_bold "you want
to install another server (preferred) dedicated for use with
CINEMA, you should answer 'n' for the next question"] and
ask the script to install it later." ]

  if { $ans == "install" } {
      install_external "apache"
  } elseif { $ans == "exists" } {
      # Get the password and host name for mysql.
      puts "Since you have already installed http server 
make sure that the server httpd is running and 
you can connect to it from this machine."

      set webhost [getstring "
Enter the host name on which Http server is running,
e.g., $webhost
By default web servers use port 80. If your webserver
uses some other port then you must specify that here, e.g.,
if your webserver uses port 7124 then 
specify as $webhost:7124

[font_bold "Default is $webhost"]" $webhost]
      puts ""
      set hostport [split $webhost :] 
      if { [llength $hostport] == 1 } {
          lappend hostport "80"
      }
      set webhost [lindex $hostport 0]
      set webport [lindex $hostport 1]

      #Check for alias to www.
      set webhostalias $webhost
      if { [lindex [split $webhost .] 0] == "www" } {
          set str "unknown"
          if {[catch {set str [exec nslookup $webhost | grep Name]}]==0} {
              set strl [split $str \ ]
              set str [lindex $strl [expr [llength $strl] - 1]]
          }
          set webhostalias [getstring "
Usually the web host names are DNS aliases to actual host
names. Your host name $webhostalias is progrably an alias
to $str. Please confirm with your system administrator and
enter the actual host name.
[font_bold "Default is $webhostalias"]" $webhostalias]
      } 
  }
}

if { $webpath == "" } {
# Get the webpath only when not already set.
set webpath  "$installdir/web"
set webpath [getpath "
Enter the absolute directory path where you would like to install
the web files. This must be accessible from your HTTP server.
Also make sure you have write permissions in this directory.
It is preferable to keep the default value.
[font_bold "Default is $webpath"]" $webpath]
puts ""
}

# Ask for SNMP installation.
if {$present(sipd) == 1} {
  set ans [ask_about_external_tool "snmp" \
"SNMP server is optional for sipd. Your application will run
without SNMP. But if you want to do network management using
SIP MIB for the sipd server, please install SNMP server, snmpd.
It must be installed on the same host as sipd." ]

  if { $ans == "install" } {
      install_external "snmp"
  } elseif { $ans == "exists" } {
      # Get the password and host name for mysql.
      puts "Since you have already installed snmpd server 
make sure that the server snmpd is running and
is on the same host as sipd when you want to use
SNMP."

  }
}

# Check for tclsh. This must not fail, otherwise it wouldn't have run
# the script so far !
if { $present(web) == 1 } {
  set tclpath [checkmodulepath "/usr/bin/tclsh" "Tcl" "http://dev.scriptics.com"]
}

# Check for sendmail path
if {$present(sipum) == 1} {
  set sendmailpath "/usr/lib/sendmail"
  set sendmailpath [getpath "
Enter the absolute path for the sendmail tool on Unix.
[font_bold "Default is $sendmailpath"]" $sendmailpath]
} else {
  set sendmailpath ""
}

# Add the sipd directory to the LD_LIBRARY_PATH, for the mysql shared lib
# (and the libraries it depends on). XXX why do we need this in sipd/
if { $present(sipd) == 1 } {
  if {[info exists env(LD_LIBRARY_PATH)]} {
    set env(LD_LIBRARY_PATH) "$env(LD_LIBRARY_PATH):$installdir/sipd"
  } else {
    set env(LD_LIBRARY_PATH) "$installdir/sipd"
  }
  # For root installation in standard_path mode we should move all
  #the shared libraries to the /usr/lib path.
  if { $isroot == 1 && $standard_path == 1 } {
    if {[catch {set files [glob -- "$installdir/sipd/*.so*"]} msg] == 0} {
      foreach filen $files {
        if { [file exists "/usr/lib/[file tail $filen]"] } {
          puts "Shared library [file tail $filen] exists"
        } else {
          if {[catch {file copy -- "$filen" /usr/lib} err]} {
            puts "ERROR: Could not copy [file tail $filen]: $err"
          }
        }
      }
    }
  }
}

if { $present(sipd) == 1 } {
  set sipdhost $hostname
  set sipdhost [getstring "
Enter the hostname on which the SIP server (sipd) will be running.
[font_bold "Default is $sipdhost"]" $sipdhost]
  puts ""
} else {
    set sipdhost ""
    set sipdport ""
}

if { $present(sipum) == 1 } {
  set sipumhost $hostname
  set sipumhost [getstring "
Enter the hostname on which the SIP unified messaging (sipum) will be running.
[font_bold "Default is $sipumhost"]" $sipumhost]

  if { $sipumhost == $sipdhost } {
      set sipumport "5070"
      set sipumport [getstring "
You can not run more than one SIP application on the same host at the
same port. 
Enter the port number on which the sipum will be
running. 
[font_bold "Default is $sipumport"]" $sipumport]
  } else {
      set sipumport "5060"
  }
  puts ""
} else {
    set sipumhost ""
    set sipumport ""
}

if { $present(rtspd) == 1 } {
  set rtspdhost "$hostname"
  set rtspdhost [getstring "
Enter the hostname on which 
the RTSP media server (rtspd) will be running.
[font_bold "Default is $rtspdhost"]" $rtspdhost]
  puts ""
  set rtspdport "554"
  if {[info exists tcl_platform(user)] &&
   $tcl_platform(user) != "root" } {
       set rtspdport "8554"
  }
  
  set rtspdport [getstring "
By default rtspd server uses port 554. This
is specified by RTSP specification. But this
port number is not available to non-root users.
If you plan to run rtspd without super user privileges
it is strongly advised that you use port number 8554.

[font_bold "Default is $rtspdport"]" $rtspdport]
  puts ""
} else {
    set rtspdhost ""
    set rtspdport ""
}

if { $present(sipconf) == 1 } {
  set sipconfhost $hostname
  set sipconfhost [getstring "
Enter the hostname on which the SIP conferencing server (sipconf) will be running.
[font_bold "Default is $sipconfhost"]" $sipconfhost]
  puts ""

  if { $sipconfhost == $sipdhost || $sipconfhost == $sipumhost} {
      set sipconfport "5072"
      set sipconfport [getstring "
You can not run more than one SIP application on the same host at the
same port. 
Enter the port number on which the sipconf will be
running. 
[font_bold "Default is $sipconfport"]" $sipconfport]
  } else {
      set sipconfport "5060"
  }
  puts ""
} else {
    set sipconfhost ""
    set sipconfport ""
}

if { $present(sip323) == 1 } {
  set sip323host $hostname
  set sip323host [getstring "
Enter the hostname on which the SIP-H.323 converter (siph323) will be running.
[font_bold "Default is $sip323host"]" $sip323host]
  puts ""
  if { $sip323host == $sipdhost || $sip323host == $sipumhost || $sip323host == $sipconfhost } {
      set sip323port "5074"
      set sip323port [getstring "
You can not run more than one SIP application on the same host at the
same port. 
Enter the port number on which the sip323 will be
running. 
[font_bold "Default is $sip323port"]" $sip323port]
  } else {
      set sip323port "5060"
  }
  puts ""
} else {
    set sip323host ""
    set sip323port ""
}

# - path for top directory of voicemails.
set vmailuserdir ""
if { $present(sipum) } {
    catch {mkdir vmusers}
    set vmailuserdir "$installdir/vmusers"
    set vmailuserdir [getpath "
Enter the directory path where you would like to store all
voicemails for the registered users.
[font_bold "Default is $vmailuserdir"]" $vmailuserdir]
}

# License string for various apps. XXX we do not need license string
# for sipua or rtspd
set sipd_license    ""
set sipconf_license ""
set sipum_license   ""
set sip323_license  ""

if { $present(sipd) } {
  set sipd_license [getstring "
Please provide the license string for sipd. 
License string is of the form \"domain:date:encodedtext\"" $sipd_license]
}
if { $present(sipconf) } {
  set sipconf_license [getstring "
Please provide the license string for sipconf. 
License string is of the form \"domain:date:encodedtext\"" $sipconf_license]
}
if { $present(sipum) } {
  set sipum_license [getstring "
Please provide the license string for sipum. 
License string is of the form \"domain:date:encodedtext\"" $sipum_license]
}
if { $present(sip323) } {
  set sip323_license [getstring "
Please provide the license string for sip323. 
License string is of the form \"domain:date:encodedtext\"" $sip323_license]
}

#------------------- End of user input ---------------------------

# We do not move files for this installation. Everything is under this
# toplevel installation directory/
# Install (i.e., move files) for the cinema components and external
# libraries or tools. Including the web scripts and md5string, fbsql etc
# files to web directory and audio files for voicemail prompts into
# voicemail directory.

if { $present(web) == 1 } {
  # Copy the webfiles to the correct destination if the user
  # wants a different web directory.
  if { $webpath != "$installdir/web" } {

    if {[catch {file mkdir $webpath} msg] != 0} {
      puts "ERROR: Could not create $webpath: $msg"
    }
    puts "Copying web files ... please wait."

    if {[catch {set files [glob -- "$installdir/web/*"]} msg] == 0} {
      foreach filen $files {
        if {[catch {file copy -force -- "$filen" "$webpath/[file tail $filen]"} err]} {
          puts "ERROR: Could not copy [file tail $filen]: $err"
        }
      }
    }
  }

  # Set the tclpath correctly in the first line of the web cgi files.
  # This is needed only for windows version.
  if { $platform == "windows" } {
   set newtclpath $tclpath
   catch {regsub -all / $tclpath \\/ newtclpath}

   if {[catch {set files [glob -- "$webpath/*"]} msg] == 0} {
     foreach filen $files {
      if {[file isfile $filen] && [file extension $filen] == ".cgi" || [file extension $filen] == ".tcl"} {                
            if { [catch {file copy -force -- "$filen" ".tmp"}] } {
                puts "ERROR: Could not move [file tail $filen]"
            } elseif {[catch {
              set fp1 [open ".tmp" "r"]
              set fp2 [open "$filen" "w"]

              # Give the path in quotes to avoid cases like "Program Files"
              puts $fp2 "#! \"$tclpath\""
              
              fcopy $fp1 $fp2
              close $fp1
              close $fp2
            } msg] != 0 } {
                puts "ERROR: Could not edit [file tail $filen]: $msg"
            }
        }
    } 
    # For all files
   } else {
    puts "ERROR: Could not list the web directory $webpath/*: $msg"
   }
   # If webpath can be listed.
  }
  # windows installation

  if {[catch {file copy -force "$installdir/$md5string" "$webpath"} msg] != 0} {
    puts "ERROR: Could not copy [file tail $md5string]: $msg"
  }
  if {[catch {file copy -force "$installdir/$random32" "$webpath"} msg] != 0} {
    puts "ERROR: Could not copy [file tail $random32]: $msg"
  }
  if {[catch {file copy -force "$installdir/$fbsql" "$webpath"} msg] != 0} {
    puts "ERROR: Could not copy [file tail $fbsql]: $msg"
  }

  if {$platform == "windows"} {
    puts "Make sure that you have enough permissions for $webpath"
  } else {
    if {[catch {exec chmod -R 755 "$webpath"} msg] != 0} {
      puts "ERROR: Could not change permissions for $webpath: $msg"
    }
    # Should we make sure that the parent/ancester dirs of webpath are
    # given correct permissions. XXX
  }
}

# Set voicemail path if needed.
if { $present(sipum) == 1 } {
  if {![file isdirectory "$vmailuserdir/audio"]} {
    if {[catch {file mkdir "$vmailuserdir/audio"} err] !=0 } {
      puts "ERROR: Could not created $vmailuserdir/audio: $err"
    }
  }

  if {[catch {file copy -force "$installdir/sipum/audio/welcome.au" "$vmailuserdir/audio/welcome.au"} err] != 0 } {
    puts "ERROR: Could not copy welcome.au: $err"
  }
  
  if { $isroot == 1 } {
      if {[catch {exec chgrp -R nobody "$vmailuserdir"} msg] != 0} {
          puts "ERROR: Could not change group for $vmailuserdir: $msg"
      }
      if {[catch {exec chmod -R 770 "$vmailuserdir"} msg] != 0} {
          puts "ERROR: Could not change permissions for $vmailuserdir: $msg"
      }
  } 
}

# Generate config files as needed. httpd.conf, cinema_db.conf,
# run-monitor, gateways.sample, dialplan.sample, and alive.cgi. 
# and sip323.license, sipua.license.
# (I have written another CGI script that will
# allow me to stop and start the cinema applications - sipd,
# sipconf,...etc, from web using unix rsh; but has may have security problem
# hence not good. However it can be used for common installation of
# "everything in one box".).
# Also generate resolv.conf for windows applications.

if { $standard_path == 1 } {
  set sqlurl "sql://$mysqluser:$mysqlpasswd@localhost/sip"
} else {
  set sqlurl "sql://$mysqluser:$mysqlpasswd@$mysqlhost/sip"
}
set env(CINEMA) "$sqlurl"

# XXX update the httpd.conf file.
# add an alias to /cinema as usual, then add a handler to .cgi files
# also add setenv and passenv as follows
# sprintf(buf, "sql://%s:%s@%s/sip", db_params.db_user, 
#    db_params.db_pwd, 
#    db_params.db_server);
#  fprintf(fp, "\nSetEnv CINEMA %s\n", buf);
#  fprintf(fp, "PassEnv CINEMA\n");

# Create cinema_db.conf
set cinema_db_conf "$installdir/cinema_db.conf"
if { $present(mysql) } {
    puts "Creating cinema_db.conf"
    if { [catch {exec echo "$sqlurl" > "/etc/cinema_db.conf"}] } {
        catch {exec echo "$sqlurl" > "$installdir/cinema_db.conf"} 
        set cinema_db_conf "$installdir/cinema_db.conf"
    } else {
        set cinema_db_conf "/etc/cinema_db.conf"
    }
    if { $present(web) } {
        puts "Creating $webpath/cinema_db.conf"
        catch {exec echo "$sqlurl" > "$webpath/cinema_db.conf"} 
    }
    # XXX we should change the permission of cinema_db.conf
    # to allow only nobody, root and admin.
    # also if CINEMA env var was added in httpd.conf then
    # do not create this file.
}

# Create run-monitor
set adminemailesc $adminemail
regsub "@" $adminemail "\\@" adminemailesc

if {[catch {set fp [open "$installdir/run-monitor" "w"]} msg ] == 0} {
     puts $fp "#!/usr/bin/env perl -w
# This is the CINEMA monitor program.  Keep the cinema 
# modules running and send an email to the administrator if or when
# it exits unexpectedly.

# Configuration options:

(\$#ARGV >= 0) or die (\"Usage: run-monitor (sipd | sipum | sipconf | sip323 | rtspd | snmpd)\\n\");

# Name of the software.
\$name = \$ARGV\[0\];

# Number of times a particular application is allowed to crash.
# the run-monitor stops after this count.
\$retry = 10;  


# cmd is the command line to execute.
# dir is the directory in which to execute.
# dest is the email of the person to send mail to.
# log is the log file name.

\$dest    = \"$adminemailesc\";
\$dir  = \"$installdir/\$name\";

if ( \$name eq \"sipd\" ) {
    \$cmd = \"./sipd -X -d -f \\\"$cinema_db_conf\\\"\";
    \$allowedhost = \"$sipdhost\";
\$log = \"sipd_errlog\";
}
elsif ( \$name eq \"snmpd\" ) {
    \$cmd = \"./snmpd -a -f -L -r -p 5061 -x 127.0.0.1:5062\";
    \$allowedhost = \"$sipdhost\";
    \$log = \"/dev/null\";
}
elsif ( \$name eq \"sipum\" ) {
    \$cmd = \"./sipum -d -X -n -p $sipumport -f \\\"$cinema_db_conf\\\" -m \\\"$sendmailpath\\\"\";
    \$allowedhost = \"$sipumhost\";
\$log = \"sipum_errlog\";
}
elsif ( \$name eq \"rtspd\" ) {
    \$cmd = \"./rtspd -d -X -p $rtspdport -s \\\"$installdir/rtspd\\\" -r \\\"$vmailuserdir\\\"\";
    \$allowedhost = \"$rtspdhost\";
\$log = \"rtspd_errlog\";
}
elsif ( \$name eq \"sip323\" ) {
    \$cmd = \"./sip323 -v -r -a -k -p $sip323port\"; 
    \$allowedhost = \"$sip323host\";
\$log = \"sip323_errlog\";
}
elsif ( \$name eq \"sipconf\" ) {
    \$cmd = \"./sipconf -d -X -m 20 -p $sipconfport -f \\\"$cinema_db_conf\\\"\";
    \$allowedhost = \"$sipconfhost\";
\$log = \"sipconf_errlog\";
}
else {
    die (\"\$name is not defined\\n\");
}

# The owner of this script.  This will be the \"From: \" address of the
# outgoing mail.
\$owner = \"CINEMA monitor bot <$adminemailesc>\";

# The program used to send the e-mail.  Takes the destination address on the
# command line, and a raw e-mail message on stdin.
# Solaris: has /usr/lib/sendmail, BSD: has /usr/sbin/sendmail
\$sendmail = \"$sendmailpath -i\";

\$hostname = `hostname`;
chomp \$hostname;

if ( \$hostname ne \$allowedhost ) {
    die (\"\$name is allowed on \$allowedhost, and not on \$hostname\\n\");
}

sub report_problem {
    my (\$problem) = @_;

    open (MAIL, \"|\$sendmail \$dest\") or die \"Couldn't run sendmail: \$!\";
    print MAIL <<EOF;
To: \$dest
From: \$owner
Subject: CINEMA monitor: \$name failure on \$hostname

EOF
    print MAIL \$problem;
    close MAIL or die \"Couldn't close sendmail: \$!\";
}


# Changing directory
chdir(\$dir);

# Spawn sipd forever, but exit if it fails to spawn.
while (\$retry > 0) {
    open(LOG, \">>\$log\$retry\") or die (\"Couldn't open \$log\$retry for append: \$!\");
    print LOG \"\\n\\nStarting \$cmd on \$hostname at \" . `date` . \"\\n\";
    close(LOG) or die (\"Couldn't close \$log\$retry: \$!\");
    \$val = system(\"\$cmd >>\$log\$retry 2>&1\");
    \$ret = \$val >> 8;
    \$signal = \$val & 0x7f;
    \$cored = \$val & 0x80;
    if (\$ret == 0xff) {
        # Failed to start.  Unfortunately we don't seem to get a \$! here.
        report_problem(\"Couldn't start \$cmd from \$dir.  Monitor exiting.\\n\");
        exit(1);
    }
    if (\$signal != 0) {
        report_problem(\"\$cmd died on signal \$signal\" .
                       (\$cored ? \" (core dumped)\" : \"\") .
                       \".  Re-spawning.\");
    }
    else {
        report_problem(\"\$cmd exited with status \$ret.  Re-spawning.\");
    }
    \$retry = \$retry - 1;
}

report_problem(\"\$cmd exiting.\");
"
catch {close $fp}
  puts "Created run-monitor script"
} else {
  puts "ERROR: Could not open run-monitor for writing"
}
#finished run-monitor.

# Create alive.cgi and the script that allows start/stop from web.
# XXX do we need this for 1.19 release.

# Create gateways.sample. XXX

# Create dialplan.sample. XXX

# Create sip323.license and sipua.license XXX
if {$present(sip323) && $sip323_license != ""} {
    catch {
      puts "Rewriting the license file for sip323."
      if {[file exists "$installdir/sip323/sip323.license"]} {
        exec echo "$sip323_license" >> "$installdir/sip323/sip323.license"
      } else {
        exec echo "$sip323_license" > "$installdir/sip323/sip323.license"
      }
      puts "Created sip323.license file."
    }
}

# *) Create and alter the mysql database table user, for host
# privileges. Create or alter the database sip. Populate the
# global config tables for cinema, sipd and vmail.
# Populate the database for
# initial admin user. Populate database with licenses.
# Create a dummy conference for test if sipconf is installed.
# All this is already done by the createdb script. This script
# just needs to invoke that.

if { $present(mysql) && $present(web) } {
  if {[catch {

    # Load the external tool for mysql-tcl
    puts "Trying to load fbsql.so..."

    if { $platform == "windows" } {
      # Load windows version also if platform is windows.
      load "$webpath/fbsql.dll"
    } else {
      load "$webpath/libfbsql.so"
    }

    puts "fbsql is loaded..."

    lappend auto_path "$webpath"
    package require forms

   #Open the database to change the privileges.
    if {[catch {
      if { "$hostname" == "$mysqlhost" } {
        puts "Opening the database sql://$mysqluser:$mysqlpasswd@localhost/mysql"
        set status [opendb "sql://$mysqluser:$mysqlpasswd@localhost/mysql"]
      } else {
        puts "Opening the database sql://$mysqluser:$mysqlpasswd@$mysqlhost/mysql"
        set status [opendb "sql://$mysqluser:$mysqlpasswd@$mysqlhost/mysql"]
      }
    }]} {
      puts "ERROR: Can not open database sql://$mysqluser:$mysqlpasswd@$mysqlhost/mysql"
      exit 1
    } elseif {$status != 0} {
      puts "ERROR: Can not open database mysql correctly sql://$mysqluser:$mysqlpasswd@$mysqlhost/mysql"
      exit 1
    } else {
      if { $mysqlpasswd != "NULL" } {
        if {[catch {sql "UPDATE user SET Password=PASSWORD('$mysqlpasswd') WHERE user='$mysqluser'"} msg]} {
          puts "ERROR: Could not change password for user $mysqluser: $msg"
        }
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$hostname IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
          puts "ERROR: Could not change privilege for $hostname: $msg"
        }
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$mysqlhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
          puts "ERROR: Could not change privilege for $mysqlhost: $msg"
        }
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$webhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
            puts "ERROR: Could not change privilege for $webhost: $msg"
        }
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$webhostalias IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
            puts "ERROR: Could not change privilege for $webhostalias: $msg"
        }
        if { $present(sipd) == 1 } {
            if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipdhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
                puts "ERROR: Could not change privilege for $sipdhost: $msg"
            }
        }
        if { $present(sipum) == 1 } {
            if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipumhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
                puts "ERROR: Could not change privilege for $sipumhost: $msg"
            }
        }
        if { $present(sipconf) == 1 } {
            if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipconfhost IDENTIFIED BY '$mysqlpasswd' WITH GRANT OPTION"} msg]} {
                puts "ERROR: Could not change privilege for $sipconfhost: $msg"
            }
        }
      } else {
        puts "Changing permissions for $hostname"
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$hostname  WITH GRANT OPTION"} msg]} {
            puts "ERROR: Could not change privilege for $hostname: $msg"
        }
        puts "Changing permissions for $mysqlhost"
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$mysqlhost  WITH GRANT OPTION"} msg]} {
            puts "ERROR: Could not change privilege for $mysqlhost: $msg"
        }
        puts "Changing permissions for $webhost"
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$webhost  WITH GRANT OPTION"} msg]} {
            puts "ERROR: Could not change privilege for $webhost: $msg"
        }
        puts "Changing permissions for $webhostalias"
        if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$webhostalias  WITH GRANT OPTION"} msg]} {
            puts "ERROR: Could not change privilege for $webhostalias: $msg"
        }
        if { $present(sipd) == 1 } {
            puts "Changing permissions for $sipdhost"
            if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipdhost  WITH GRANT OPTION"} msg]} {
                puts "ERROR: Could not change privilege for $sipdhost: $msg"
            }
        }
        if { $present(sipum) == 1 } {
            puts "Changing permissions for $sipumhost"
            if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipumhost  WITH GRANT OPTION"} msg]} {
                puts "ERROR: Could not change privilege for $sipumhost: $msg"
            }
        }
        if { $present(sipconf) == 1 } {
            puts "Changing permissions for $sipconfhost"
            if {[catch {sql "GRANT ALL PRIVILEGES ON *.* TO $mysqluser@$sipconfhost  WITH GRANT OPTION"} msg]} {
                puts "ERROR: Could not change privilege for $sipconfhost: $msg"
            }
        }
      }

      puts "Flushing privleges"
      if {[catch {sql "FLUSH PRIVILEGES"} msg]} {
          puts "ERROR: Could not flush privileges: $msg"
      }
    }

    # Mysql setting of host privileges is done.

    # Call the script to do the creation of database.
    if {[catch {sql disconnect} err]} {
      puts "sql disconnect failed, $err"
    }
    puts "Calling createdb"
    source "$installdir/scripts/createdb"
    puts "createdb is complete"

    # Insert license string in sipd
    if { $present(sipd) && $sipd_license != ""} {
        puts "Putting sipd license"
        set sl [split "$sipd_license" :]
        if {[catch {sql "INSERT INTO sipd_license values('[lindex $sl 0]','[lindex $sl 1]','[lindex $sl 2]')"} msg]} {
            puts "ERROR: Could not update sipd license information: $msg"
        }
    }

    # Insert license string in sipum
    if { $present(sipum) && $sipd_license != ""} {
        puts "Putting sipum license"
        set sl [split "$sipum_license" :]
        if {[catch {sql "INSERT INTO sipum_license values('[lindex $sl 0]','[lindex $sl 1]','[lindex $sl 2]')"} msg]} {
            puts "ERROR: Could not update sipum license information: $msg"
        }
    }

    #insert license string in sipconf
    if { $present(sipconf) && $sipd_license != ""} {
        puts "Putting sipconf license"
        set sl [split "$sipconf_license" :]
        if {[catch {sql "INSERT INTO sipconf_license values('[lindex $sl 0]','[lindex $sl 1]','[lindex $sl 2]')"} msg]} {
            puts "ERROR: Could not update sipconf license information: $msg"
        }
    }

    # Create a cinema table entry for this installation (domain)
    puts "Creating cinema config"
    if {[catch {sql "INSERT INTO cinema set server_name='$domain',mode='local',realm='$domain',administrator='$adminemail',http_server_name='$webhost',icon='http://www.cs.columbia.edu/~hgs/clipart/Logos/Columbia/IRT_logo50.gif',banner='CINEMA - Columbia InterNet Extensible Multimedia Architecture'"} msg]} {
        puts "ERROR: Could not create cinema table entry: $msg"
    }
    
    # UPdate a vmail config entry
    if { $present(sipum) } {
        puts "Creating vmail config"
        if {[catch {sql "INSERT INTO vmail set server_name='$domain',vmailuserdir='$vmailuserdir',vmailhost='$sipumhost',vmailport='$sipumport',rtsphost='rtspdhost',rtspport='rtspdport',randomdigits='6',vmailquota_kb='2000'"} msg]} {
            puts "ERROR: Could not update vmail table: $msg"
        }
    }

    # Update sipd_config
    if { $present(sipd) } {
        puts "Creating sipd config"
        set regpat "($domain)"
        if {[catch {sql "INSERT INTO sipd_config SET server_name='$domain',domain='$regpat',server_root='.',pid_file='logs/sipd.pid',realm='$domain',default_reg='proxy'"} msg]} {
            puts "ERROR: Could not update sipd config table: $msg"
        }
    }

    # Calculate password as same as userid if password was NULL.
    # User should change there password.
    set adminpasswd $mysqlpasswd
    if { $adminpasswd == "NULL" } {
        set adminpasswd $mysqluser
        if { $adminpasswd == "NULL" || $adminpasswd == "" } {
            set adminpasswd "none"
        }
    }
    
    # Initialize hash value
    puts "Calculating hash value"
    set hash_val "ebbc0ff9a121dbb6789bbe5f82174fa0"
    if [catch {set hash_val [exec $md5string "default@$domain:$domain:$adminpasswd"]} err] {
        puts "ERROR: Could not calculate hash value for default user"
    }
    
    # initialize groups
    if { $present(sipum) } {
        set groups "cgi voicemail"
        
        # Construct rtsp url for welcome message and the email template for voice messages.
        set rtspurl "rtsp://SERVER/audio/welcome.au"
        set email_template "To: \[email \$to\]
Reply-To: \[email \$from\]
Subject: voice mail -- \$subject
Priority: \$priority

Dear \$name,

Voice mail from \[dataurl \$from \$from] has arrived. 
You can play the message by using QuickTime using   
url \$rtspurl or
by going to the web page at \$httpurl.

- \$administrator

--
This mail was automatically generated by the vmail system."

    } else {
        set groups "cgi"
        set rtspurl ""
        set email_template ""
    }
    set lastmod [clock seconds]
    # Update put with default user. Don't use addsipuser tool
    if {[catch {sql "INSERT INTO put values('default@$domain','$hash_val','$domain','$groups','never','MD5','REGISTER INVITE any',NULL,'$rtspurl','$rtspurl','$email_template', '15','200','$lastmod','guest','default@$domain')"} msg]} {
        puts "ERROR: Could not create default@$domain user profile: $msg"
    }
    
    # Initialize hash value
    set hash_val "ebbc0ff9a121dbb6789bbe5f82174fa0"
    if [catch {set hash_val [exec $md5string "$LOGNAME@$domain:$domain:$adminpasswd"]} err] {
        puts "ERROR: Could not calculate hash value for user $LOGNAME@$domain"
    }
    # initialize groups
    if { $present(sipum) } {
        set groups "cgi voicemail admin"
    } else {
        set groups "cgi admin"
    }
    # Update put with this/admin user.
    set lastmod [clock seconds]
    if {[catch {sql "INSERT INTO put values('$LOGNAME@$domain','$hash_val','$domain','$groups','never','MD5','REGISTER INVITE any',NULL,'$rtspurl','$rtspurl','$email_template', '15','200','$lastmod','guest','$LOGNAME@$domain')"} msg]} {
        puts "ERROR: Could not create $LOGNAME@$domain user profile: $msg"
    } else {
        # Adding this user as the administrator.
        puts "*IMPORTANT*: [font_bold "Adding administrator $LOGNAME@$domain with password $adminpasswd"]
Please note this userid and password for logging in to the CINEMA web page.
You should change your password as early as possible."
        waitforuser
    }
    
    # Set the default for groups
    if { $present(sipum) } {
        if {[catch {sql "ALTER TABLE put ALTER COLUMN sip_groups SET DEFAULT 'cgi voicemail'"} msg]} {
            puts "ERROR: Could not set default for sip_groups: $msg"
        }
    } else {
        if {[catch {sql "ALTER TABLE put ALTER COLUMN sip_groups SET DEFAULT 'cgi'"} msg]} {
            puts "ERROR: Could not set default for sip_groups: $msg"
        }
    }

  # Done all the initialization of databse part
  } err]} {
    puts "Could not initialize the database.\n$err"
    puts "ERROR: Your installation is incomplete"
  } else {
    puts "Database is successfully initialized."
  }
}
# Both web and mysql are present.


# Check with ldd or with -h option to see if installed correctly.
puts "Checking available shared libraries..."
foreach app {sipd rtspd sipconf sipua sipum sip323} {
    if { [file exists "$installdir/$app/$app"] } {
        puts "$app:"
        catch {puts [exec ldd "$installdir/$app/$app" | grep found]}
    }
}
puts "If there was any missing library please make sure to install it."
waitforuser

# Use http get to see if the HTTP pages where installed correctly.
# XXX do this?

if {$isroot == 1} {
  # Update the rc scripts.
  set ans "n"
  set ans [getstring "Do you want me to install RC scripts in your system for
automatically starting mysql server and cinema components. 
This is risky and may make your system to not function properly.
Answer y or n. Default is n" $ans]

  if { $ans == "y" } {
    catch {
        file copy "$installdir/scripts/ctl-cinema" /etc/rc3.d/S70cinemactl
        file copy "$installdir/scripts/mysqlctl"   /etc/rc3.d/S60mysqlctl
    }
    puts "You should put apachectl as /etc/rc3.d/S50apachectl
to automatically start apache web server on this host."
  }

}

# Display note on how to run the application.
puts "You can run following command(s) to see if things works or not:
 \$ cd $installdir"
if { $present(sipd) == 1} {
  puts " \$ sipd/sipd -h"
}

if { $present(rtspd) == 1} {
  puts " \$ rtspd/rtspd -h"

}

if { $present(sip323) == 1} {
  puts " \$ sip323/sip323 -h"

}

if { $present(sipum) == 1} {
  puts " \$ sipum/sipum -h"

}

if { $present(sipconf) == 1} {
  puts " \$ sipconf/sipconf -h"
} 

puts "
[font_bold "Use the run-monitor script provided to use to
start various servers. This script will send you an
email notification when the server dies and it will 
automatically try to restart the server"].
For example to start sipd:
 \$ ./run-monitor sipd &
"
waitforuser

# Change the permission of all the executables.
foreach app {sipd rtspd sipconf sipua sipum sip323} {
    catch {file attributes "$installdir/$app/$app" -permissions 0755}
}
foreach tool {canonicalize addsipuser random32 md5string sippasswd} {
    catch {file attributes "$installdir/tools/$tool/$tool" -permissions 0755}
}
catch {file attributes "$installdir/run-monitor" -permissions 0755}

# Prompt the user to start the application using run-monitor.
# if the app was installed and supposed to be run on this machine.
foreach app {sipd rtspd sipconf sipua sipum sip323} {
    if { [file exists "$installdir/$app/$app"] && [info exists [subst ${app}host]] && "$hostname" == "[subst $[subst ${app}host]]" } {
        set ans "y"
        set ans [getstring "
Do you want me to run $app on this machine now...
[font_bold "Default is $ans"]" $ans]
        
        if {$ans == "y"} {
          if {[catch {
              exec ./run-monitor $app &
          } err]} {
              puts "ERROR: $err"
          } else {
              puts "$app started"
          }
        }
    }
}

# display a note to send bug report to cinema-support@

puts "
Your installation is complete. 
To add new user or to change your profile visit appropriate
URL corresponding to $webpath.
"

if { $cinema_url != "" } {
  puts "[font_bold "Visit $cinema_url for the user interface"].
[font_bold "Use userid as $LOGNAME@$domain and password as $adminpasswd"] for 
logging in as administrator."
}

puts "
Once again, $reportfooter"

###################### End of FILE ###################
