#ifndef DSTRING_H
#define DSTRING_H

#include <time.h>
#include <stdarg.h>

typedef struct {
  char *string;
  int length;
  int spaceAvl;
} DString;

#ifdef __GNUC__
/* This function uses printf-style formats, so tell GNUC to do printf
 * format checking. */
#define ATTRIBUTE_PRINTF(fmt, args) \
   __attribute__ ((format (printf, fmt, args)))
#else
#define ATTRIBUTE_PRINTF(fmt, args)
#endif

/* Create new, empty DString. */
extern void  DStringInit(DString *d);

/* Append 'string' of length 'length' to 'd'. */
extern char *DStringAppend(DString *d, const char *string, int length);

/* Append 'string' to 'd', while quoting characters in 'quote'. */
char *DStringAppendQuoted(DString *d, const char *string, const char *quote);

/* Append 'string' to 'd', while hex %-escaping non-alphanumeric characters
 * not in 'unescaped'. */
char *DStringAppendEscaped(DString *d, const char *string,
                           const char *unescaped);

/* Append DString 'src' to DString 'dst'. */
extern char *DStringAppendD(DString *dst, const DString *src);

/* sprintf() into DString 'd' using format 'format'. */
extern char *DStringPrintf(DString *d, const char *format, ...)
     ATTRIBUTE_PRINTF(2, 3);

/* vsprintf() into DString 'd' using format 'format' and args 'args' */
extern char *DStringVPrintf(DString *d, const char *format, va_list args)
     ATTRIBUTE_PRINTF(2, 0);

/* Convert time 'tm' to text using 'format', with destination 'd'. */
extern char *DStringStrftime(DString *d, const char *format,
                             const struct tm *tm); 

/* Convert time 't' to text using 'format', with destination 'd'. If
'localtime' is 0, GMT is used. */
extern char *DStringCftime(DString *d, const char *format, time_t t,
                           int localtime);

/* Return char * value of DString. */
extern char *DStringValue(DString *d);

/* Remove leading and trailing white space from DString. */
extern char *DStringTrim(DString *d);

/* Set length of DString to 0, but don't free memory. */
extern void  DStringTrunc(DString *d);

/* Zero the storage of a dstring, then truncate it. */
extern void  DStringZero(DString *d);

/* Free storage for DString. */
extern void  DStringFree(DString *d);

/* Return length of DString 'd'. */
extern int   DStringLength(const DString *d);

#undef ATTRIBUTE_PRINTF
#endif
