/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : msgflow.h
#
#  Module       : Message Flow Display for Program Trace.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 27 Nov 1999
#
#  Description  : A generic message flow drawing routine. This is
#                 used to get program trace information.
#                
#  Notes        : Error checking for duplicate msgclass,valid filename
#                 etc is not done.
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/


#ifndef _MSGFLOW_H_
#define _MSGFLOW_H_

#ifdef __cplusplus
extern "C" 
{
#endif

  /*
   * It is recommended that all calls to DrawMessageFlow() should be
   * within DRAW_MESSAGE_FLOW() block so that it can be easity switched
   * off later 
   */
  #define DRAW_MESSAGE_FLOW(st)  st;

  /* 
   * Change the log file for message flow.
   * default is stdout.
   * You can call it with NULL to switch off message flow display.
   *  e.g. MessageFlowFile(NULL);
   * stdout,stderr or any other file pointer can be used for FILE *.
   */
  int MessageFlowFile(FILE *fp); 

  /*
   * Draw the message flow for the message (msg) of class (msgclass)
   * from source module (src) to destination module (dest);
   * It calls fflush() after drawing the msgflow.
   */
  int DrawMessageFlow(char *msgclass,char *src,char *dest,char *msg);

  /*
   * Register a message class.
   * Example
   *   RegisterMessageClass("NET",3,"T1","T2","GW");
   */
  int RegisterMsgClass(char *name, int count, ... /* List of modules */);

  /*
   * TBD: Provide functions to set mesage type (HTML/TEXT) width (80columns)
   * etc
   */

#ifdef __cplusplus
}
#endif

#endif /* MSGFLOW_H */

/************************** END OF FILE ********************************
*/
