/*
 * pattern.c - match (possibly negated) regex or glob patterns.
 */

/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef PATTERN_H_
#define PATTERN_H_

#include <regex.h>

/* Pattern types. */
typedef enum {pattern_unknown, pattern_glob, pattern_regex} pattern_type_t;

/*
 * An entry from the dialplan file.
 * Since we always walk the dialplans in order, a singly-linked list is a
 * good-enough data structure.
 */
typedef struct {
  pattern_type_t type;
  regex_t re;
  int negate;

  char *pat_str;
  char *re_str;

  int suffix_offset;
  int match_count;
} pattern_t;

extern int pattern_compile(pattern_t *comp, const char *orig_pattern,
                           char *error, size_t error_len);

extern int pattern_match(const char *string, const pattern_t *comp,
                         char *errorstr, size_t errorstr_len);

extern int pattern_interpolate(const char *string, const pattern_t *comp,
                               const char *interpolate_str, char **deststr,
                               char *errorstr, size_t errorstr_len);

extern void pattern_free(pattern_t *comp);


#endif /* PATTERN_H_ */
