/* raise-rlimits.c - Raise resource limits for a server to the
 * maximum allowed by the system. */

/* Copyright 1998 - 2001 Columbia University.  All rights reserved. */

#include "raise-rlimits.h"

#ifdef WIN32
/* Dummy function on win32 */
void raise_rlimits(void)
{

}
#else /* !WIN32 */

#include <stdlib.h>
#include <string.h>

#include "error.h"

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>


/*
 * raise_rlimits()
 * Raise resource limits to the maximum allowed by the system.
 */
void raise_rlimits(void)
{
  struct rlimit rbuf;

  /* Set the open file limit. Works for solaris */
  memset(&rbuf, 0, sizeof(rbuf));
  if (getrlimit(RLIMIT_NOFILE, &rbuf) != 0) {
    error("raise_rlimits", "Could not get the RLIMIT_NOFILE limit\n");
  }
  else {
    rbuf.rlim_cur = rbuf.rlim_max;

    if (setrlimit(RLIMIT_NOFILE, &rbuf) != 0) {
      error("raise_rlimits", "Could not set the RLIMIT_NOFILE limit to %lu\n",
            (long)rbuf.rlim_cur);
    }
  }
  
  /* Set the core file rlimit. Works for solaris */
  memset(&rbuf, 0, sizeof(rbuf));
  if (getrlimit(RLIMIT_CORE, &rbuf) != 0) {
    error("raise_rlimits", "Could not get the RLIMIT_CORE limit\n");
  }
  else {
    rbuf.rlim_cur = rbuf.rlim_max;

    if (rbuf.rlim_cur > 10000000L) {
      rbuf.rlim_cur = 10000000L;
    }

    if (setrlimit(RLIMIT_CORE, &rbuf) != 0) {
      error("main", "Could not set the RLIMIT_CORE limit to %lu\n",
            (long)rbuf.rlim_cur);
    }
  }
} /* raise_rlimits */


#endif /* !WIN32 */

