/* 
 * shortform.c -- convert between short-form and long-form message
 * headers.
 *
 * Copyright 1998-1999 by Columbia University; all rights reserved
 */

#include <string.h>
#include "sysdep.h"

#include "shortform.h"

/*
 * get_shortform()
 * Get the short form of a header, given its long form.
 *
 * Right now this just walks an array.  Sometime in the future, if this
 * turns out to be a bottleneck, we may want to change this to be a hash
 * table.
 */
const char *get_shortform(const char *longform)
{
  int i;

  for (i = 0; Proto_Shortform_Map[i].longform != NULL; i++) {
    if (strcasecmp(longform, Proto_Shortform_Map[i].longform) == 0) {
      return Proto_Shortform_Map[i].shortform;
    }
  }

  return NULL;
} /* get_shortform */


/*
 * get_longform()
 * Get the long form of a header, given its short form.
 */
const char *get_longform(const char *longform)
{
  int i;

  for (i = 0; Proto_Shortform_Map[i].shortform != NULL; i++) {
    if (strcasecmp(longform, Proto_Shortform_Map[i].shortform) == 0) {
      return Proto_Shortform_Map[i].longform;
    }
  }

  return NULL;
} /* get_longform */


/*
 * get_otherform()
 * Get the other form of this header (short or long), depending on
 * which one this is.
 * This assumes that all short-forms
 */
const char *get_otherform(const char *header)
{
  if (header[0] != '\0' && header[1] == '\0') {
    /* Short form */
    return get_longform(header);
  }
  else {
    return get_shortform(header);
  }
}
