/* cpp_pthread_mutex.h -- object-based pthread mutex locking for C++.
 * pthread_mutex_obj calls pthread_mutex_(init|destroy) as a constructor
 * and destructor.  pthread_mutex_stack_obj, when allocated on the stack,
 * automatically locks when instantiated.  If you instantiate it on the stack,
 * you'll automatically unlock when leaving the enclosing block by any means,
 * including a thrown exception.)
 */

/* Copyright 2001 Columbia University.  All rights reserved. */

#include "stack_lock.h"

#include <stdexcept>
#include <cstring>
#include <cerrno>
#include <new>
using namespace std;

/*
 * pthread_mutex_obj::throw_from_errno(int)
 * Given an errno, throw an appropriate exception.
 * Only for those errno values which are returned by pthread functions.
 * Never returns normally.
 */
void pthread_mutex_obj::throw_from_errno(int err) {
  switch (err) {
  case ENOMEM:
    throw bad_alloc();

  case EINVAL:
    throw invalid_argument(strerror(err));

  case EPERM:
  case EAGAIN:
  case EBUSY:
  case EDEADLK:
  default:
    throw runtime_error(strerror(err));
  }
} /* pthread_mutex_obj::throw_from_errno(int) */
