/* Test program for gsearch */
/* Copyright 2001 Columbia University.  All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <regex.h>

#include "gmatch.h"
#include "glob2regex.h"

/*
 * usage()
 * Print usage and exit.
 */
static void usage(char *argv0)
{
  fprintf(stderr, "Usage: %s [-r] [-n count] string pattern\n", argv0);
  exit(2);
} /* usage */


/*
 * do_gmatch_match()
 * Match using Gmatch().
 */
void do_gmatch_match(char *string, char *pattern, int count)
{
  int i;
  int res;

  for (i = 0; i < count - 1; i++) {
    res = Gmatch(string, pattern);
    if (res < 0) break;
  }

  res = Gmatch(string, pattern);

  if (res < 0) {
    printf("Error: %s\n", gmatch_strerror(res));
    exit(1);
  }
  else if (res) {
    printf("Matched!\n");
  }
  else {
    printf("Didn't match!\n");
  }
} /* do_gmatch_match */


/*
 * do_regex_match()
 * Match using glob2regex()/regcomp()/regexec().
 */
void do_regex_match(char *string, char *pattern, int count)
{
  int i;
  int res;
  char *re_pat;
  regex_t re;
  char regbuf[1024];

  if ((res = glob2regex(pattern, &re_pat, 0, NULL)) != 0) {
    glob2regex_regerror(res, NULL, regbuf, sizeof(regbuf));
    printf("Error: %s\n", regbuf);
    exit(1);
  }    

  if ((res = regcomp(&re, re_pat, REG_EXTENDED)) != 0) {
    glob2regex_regerror(res, NULL, regbuf, sizeof(regbuf));
    printf("Error: %s\n", regbuf);
    exit(1);
  }    
  
  for (i = 0; i < count - 1; i++) {
    res = regexec(&re, string, 0, NULL, 0);
    if (res != 0 && res != REG_NOMATCH)
      break;
  }

  res = regexec(&re, string, 0, NULL, 0);

  if (res == 0) {
    printf("Matched!\n");
  }
  else if (res == REG_NOMATCH) {
    printf("Didn't match!\n");
  }
  else {
    glob2regex_regerror(res, NULL, regbuf, sizeof(regbuf));
    printf("Error: %s\n", regbuf);
    exit(1);
  }

  regfree(&re);
  free(re_pat);
} /* do_gmatch_match */


/*
 * main(). 
 * Parse args, then exec appropriate matcher.
 */
int main(int argc, char *argv[])
{
  int match_with_regex = 0;
  int count = 1;
  int c;
  char *string;
  char *pattern;

  while ((c = getopt(argc, argv, "rn:")) != EOF) {
    extern char *optarg;
    extern int optind;

    switch(c) {
    case 'r':
      match_with_regex = 1;
      break;
    case 'n':
      count = atoi(optarg);
      break;
    case '?':
    default:
      usage(argv[0]);
    }
  }

  if (argc < optind + 2) usage(argv[0]);

  if (count <= 0) exit(1);

  string = argv[optind];
  pattern = argv[optind+1];

  if (match_with_regex) {
    do_regex_match(string, pattern, count);
  }
  else {
    do_gmatch_match(string, pattern, count);
  }

  exit(0);
}
