#include "uri.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

static const uri_nv_t sip_default_params[] = {
  {"user", "ip"},
  {"transport", "udp"},
  {"ttl", "1"},
  {"method", "INVITE"},
  {NULL, NULL}
};

static const uri_defaults_t uri_defaults[] = {
  {"sip", 5060, sip_default_params},
  {"rtsp", 554, NULL},
  {"rtspu", 554, NULL},
  {"http", 80, NULL},
  {NULL, 0, NULL}
};

#define STRING_OR_NULL(str) ((str) != NULL ? (str) : "(null)")
  
static void print_broken_down_nv(uri_nv_t *nv)
{
  if (nv == NULL) {
    printf("  (none)\n");
    return;
  }

  while (nv != NULL && nv->name != NULL && nv->value != NULL) {
    printf("  \"%s\": \"%s\"\n", nv->name, nv->value);
    nv++;
  }
}


static void print_broken_down_uri(uri_t *u)
{
  printf("scheme: %s\n", STRING_OR_NULL(u->scheme));
  printf("user: %s\n", STRING_OR_NULL(u->user));
  printf("password: %s\n", STRING_OR_NULL(u->password));
  printf("host: %s\n", STRING_OR_NULL(u->host));
  printf("port: %d\n", u->port);
  printf("path: %s\n", STRING_OR_NULL(u->path));
  printf("param: %s\n", STRING_OR_NULL(u->param));
  printf("search: %s\n", STRING_OR_NULL(u->search));
  printf("parameters:\n");
  print_broken_down_nv(u->parameters);
  printf("headers:\n");
  print_broken_down_nv(u->headers);
}

int main(int argc, char *argv[])
{
  uri_t u, uc, v, vc; 

  if (argc <= 1) {
    fprintf(stderr, "Usage: %s uri [uri]\n", argv[0]);
    exit(2);
  }

  u = URI_Parse(argv[1]);
  URI_Print(u);

  print_broken_down_uri(&u);

  uc = URI_Copy(u);

  URI_Canonicalize(&uc, uri_defaults);
  printf("Canonicalized :\t");
  URI_Print(uc);

  if (argc > 2) {
    v = URI_Parse(argv[2]);
    URI_Print(v);

    print_broken_down_uri(&u);

    vc = URI_Copy(v);
    URI_Canonicalize(&vc, uri_defaults);
    printf("Canonicalized :\t");
    URI_Print(vc);

    if (URI_Equal_With_Defaults(u, v, uri_defaults)) {
      printf("Equal.\n");
    }
    else {
      printf("Not equal.\n");
    }
    URI_Free(&v);
    URI_Free(&vc);
  }

  URI_Free(&u);
  URI_Free(&uc);

  return 0;
}
