/* timespec.c -- functions for manipulating timeval and timespec structures. */
/* Copyright 1999, 2001 Columbia University. All rights reserved*/

#include "sysdep.h"
#include <time.h>
#include <sys/time.h>
#include "timespec.h"
#define NSEC 1000000000
#define USEC 1000000

/*
 * timespec_delay()
 * Add a delay (in seconds) to a starting time.
 */
struct timespec timespec_delay(struct timeval now, double delay)
{
  struct timespec next;

  next.tv_sec = now.tv_sec;
  next.tv_nsec = now.tv_usec * 1000 + (delay - (long)delay) * NSEC;
  if (next.tv_nsec > NSEC) {
    next.tv_sec++;
    next.tv_nsec -= NSEC;
  }
  next.tv_sec += (long)delay;
  return next;
} /* timespec_delay */


/*
 * timeval_until()
 * Return a timeval value representing the amount of time between the 
 * current time and the given value.
 */
struct timeval timeval_until(struct timeval timeout)
{
  struct timeval now, remaining;

  gettimeofday(&now, NULL);

  if (timeout.tv_sec < now.tv_sec ||
      (timeout.tv_sec == now.tv_sec && timeout.tv_usec <= now.tv_usec)) {
    remaining.tv_sec = 0;
    remaining.tv_usec = 0;
    return remaining;
  }

  while (timeout.tv_usec < now.tv_usec) {
    timeout.tv_usec += USEC;
    timeout.tv_sec -= 1;
  }
  remaining.tv_usec = timeout.tv_usec - now.tv_usec;
  remaining.tv_sec = timeout.tv_sec - now.tv_sec;

  return remaining;
} /* timeval_until */


/*
 * timespec_compare()
 * Compare two timespecs for equality.  Return <0, 0, >0 depending on
 * whether the first one is less, they are equal, or the first one is greater.
 * XXX: this probably has problems if timespecs are more than 2^31 apart.
 */
int timespec_compare(struct timespec a, struct timespec b)
{
  if (a.tv_sec != b.tv_sec) {
    return (a.tv_sec - b.tv_sec);
  }
  return a.tv_nsec - b.tv_nsec;
} /* timespec_compare */


/*
 * timespec_min()
 * Return the lesser (earlier) of two given timespec values.
 */
struct timespec timespec_min(struct timespec a, struct timespec b)
{
  if (timespec_compare(a, b) < 0) {
    return a;
  }
  return b;
} /* timespec_min */
