#ifndef URI_H
#define URI_H
#include "dstring.h" /* for DString arg in URI_SPrint() */

/* Last one has zero pointers. */
typedef struct uri_nv_t {
  char *name;
  char *value;
} uri_nv_t;

/* All URI components have % encodings removed */
typedef struct {
  char *scheme;   /* URI scheme: http, rtsp, ... */
  char *user;     /* user */
  char *password; /* password */
  char *host;     /* host name */
  int port;       /* port number */
  int v6host;     /* =1 if host contains a IPv6 addr str */
  char *path;     /* path name */
  char *param;
  char *search;   /* search path (after ?) */
  uri_nv_t *parameters;
  uri_nv_t *headers;
} uri_t;

/* Defaults for a URI scheme, used for URI_Equal_With_Defaults and
 * URI_Canonicalize */
typedef struct {
  const char *scheme;             /* Which scheme these defaults apply to */
  int default_port;               /* Default port number for this scheme */
  const uri_nv_t *param_defaults; /* Default parameter values */
} uri_defaults_t;

extern uri_t URI_Parse(const char *url);
extern int   URI_Equal(const uri_t u1, const uri_t u2);
extern int   URI_Equal_With_Defaults(const uri_t u1, const uri_t u2,
                                     const uri_defaults_t *defaults);
extern int   URI_NV_Equal(const uri_nv_t *nv1, const uri_nv_t *nv2,
                          const uri_nv_t *defaults);
extern void  URI_Canonicalize(uri_t *up, const uri_defaults_t *defaults);
extern uri_t URI_Copy(const uri_t u);
extern void  URI_Free(uri_t *u);
extern void  URI_Print(uri_t u);
extern char *URI_AsString(uri_t u);
extern void  URI_SPrint(const uri_t *u, DString *u_dstr);
extern const char *URI_Parameter(uri_t u, const char *pname);
extern void  URI_SetParameter(uri_t *u, const char *pname, 
                              const char *pvalue);
#endif
