/* -*- C++ -*- */
/* cinema_dummy_table.h -- A dummy database implementation, intended as a
 * sample from which real database implementations can be written. */
   
/* Copyright 2001 Columbia University.  All rights reserved. */

#ifndef CINEMA_DUMMY_TABLE_H_
#define CINEMA_DUMMY_TABLE_H_

#include "cinema_table.h"

/* A dummy CINEMA table. */
class cinema_dummy_table : virtual public cinema_table
{
private:
  uri_t table_uri;

public:
  cinema_dummy_table();
  cinema_dummy_table(std::string uri);
  cinema_dummy_table(uri_t& uri);

  virtual ~cinema_dummy_table();

  virtual char* get_key(const uri_t *uri) const;
  virtual uri_t get_uri(const char *key) const;


  virtual alias_table_entry *get_aliases_by_alias(const char* alias);

  virtual put_table_entry *get_put_by_user(const char* user);


  virtual contact_table_entry** get_contacts_by_user(const char *user,
                                                     int *count = NULL);

  virtual void insert_contact(const contact_table_entry *entry);

  virtual void delete_contacts_by_user(const char *user);

  virtual void delete_contact_by_contact(const char *user,
                                         const char *contact);

  virtual void delete_expired_contacts_by_user(const char *user, 
                                               time_t expire_time);

  virtual sub_table_entry* get_sub_by_fromto(const char* from,
					     const char* to);
  
  virtual int num_subscriptions(const char *to);
  
  virtual sub_table_entry **get_all_subscriptions(const char* to, int *count);
  

  virtual void insert_sub(const sub_table_entry *entry); 
  
  virtual void delete_sub_by_fromto(const char* from, const char *to);
  
  TABLE_SCHEME("dummy")
};

#endif /* CINEMA_DUMMY_TABLE_H_ */
