#include "dbaccess.h"
#include "cinema_table.h"

#include <iostream>
#include <ctime>
#include <cstdlib>

using namespace std;

extern "C" int debug_flag;

int main(int argc, char* argv[])
{
  cinema_table *table = NULL;
  char *user = NULL;

  debug_flag = 1;

  if (argc < 3) {
    cerr << "Usage: " << argv[0] << " URL key" << endl;
    exit(2);
  }

  try {
    table = cinema_table::create(argv[1]);
  }
  catch (cinema_error& e) {
    cerr << "Could not create CINEMA table: " << e.what() << endl;
    exit(1);
  }

  try {
    alias_table_entry *entry;

    entry = table->get_aliases_by_alias(argv[2]);
  
    char lastmodbuf[256];
    
    strftime(lastmodbuf, sizeof(lastmodbuf), "%a, %b %d %Y %T GMT",
             gmtime(&entry->last_modified));
    
    cout << entry->alias << ": " << entry->primary_user
         << " (" << entry->last_modified << " = " << lastmodbuf << ")" << endl;

    user = strdup(entry->primary_user);

    free_alias_table_entry(entry);
  }
  catch (cinema_notfound& e) {
    cout << "Not an alias: " << argv[2] << endl;
    user = strdup(argv[2]);
  }
  catch (cinema_error& e) {
    cerr << "Could not get alias table entry: " << e.what() << endl;
    exit(1);
  }

  try {
    put_table_entry *entry;

    entry = table->get_put_by_user(user);
  
    cout << entry->user << ": found" << endl;

    free_put_table_entry(entry);
  }
  catch (cinema_notfound& e) {
    cerr << "Not in database: " << user << endl;
    exit(1);
  }
  catch (cinema_error& e) {
    cerr << "Could not get primary user table entry: " << e.what() << endl;
    exit(1);
  }

  try {
    contact_table_entry **contacts;
    int i, count;

    contacts = table->get_contacts_by_user(user, &count);
    
    for (i = 0; i < count; i++) {
      cout << contacts[i]->user << "[" << i << "] = ";
      cout << contacts[i]->contact << endl;
    }

    free_contact_table_entries(contacts);
  }
  catch (cinema_notfound& e) {
    cerr << "Not registered: " << user << endl;
  }
  catch (cinema_error& e) {
    cerr << "Could not get contact table entries: " << e.what() << endl;
    exit(1);
  }

  free(user);
  delete table;

  exit(0);
}
