/* ads.cpp  - Common data structures and templates for list, arrays etc.
 * (c) 2001-2001 Columbia University. All rights reserved.
*/
#include <iostream.h>  
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "ads.h"

extern "C" {
#include "sysdep.h"
}

/* 
 * Uncomment the printf x statement to enable
 * debugging of GenericList
 */
#define TRACE(x)  // printf x

GenericList::GenericList() 
{ 
  TRACE(( "GenericList is constructed\n"));

  allow_delete = true; 
  is_array     = false; 
  head         = NULL; 
  size         = 0;
}

GenericList::GenericList(bool allow_del) 
{ 
  TRACE(( "GenericList is constructed\n"));

  allow_delete = allow_del; 
  is_array     = false; 
  head         = NULL; 
  size         = 0;
}

GenericList::GenericList(const GenericList & arg)
{
  TRACE(("GenericList copy constructor\n"));
  head = NULL;
  size = 0;
  allow_delete = arg.allow_delete;
  is_array = arg.is_array;
  for ( int i=arg.GetSize()-1; i>=0; i--) {
    void  *data = AllocateElement(arg.GetAt(i));
    if ( data == NULL ) {
      TRACE(("data is NULL !!!\n"));
    }
    Insert(data);
  }
  return;
}

GenericList & GenericList::operator=(const GenericList & arg)
{
  TRACE(("GenericList copy =\n"));
  if ( this == &arg ) {
    /* No need to assign the same thing; a = a; */
    return *this;
  }

  RemoveAll();

  allow_delete = arg.allow_delete;
  is_array = arg.is_array;
  for ( int i=arg.GetSize()-1; i>=0; i--) {
    void  *data = AllocateElement(arg.GetAt(i));
    if ( data == NULL ) {
      TRACE(("data is NULL !!!\n"));
    }
    Insert(data);
  }
  return *this;
}

GenericList::~GenericList()
{
  TRACE(("GenericList is destroyed\n"));
  RemoveAll();
}

void * GenericList::GetAt(int index) const
{

  TRACE(("GenericList::GetAt(%d) ", index));

  if ( index < 0 ) {
    TRACE(("Returning NULL because index is %d\n", index));
    return NULL;
  }

  GenericList::Element * elem = head;

  while (index > 0) {
    if ( elem != NULL) {
      elem = elem->next;
    }
    index --;
  }
  if ( elem ) {
    TRACE(("Returning valid data\n"));
    return elem->data;
  }
  else {
    TRACE(("Returning NULL\n"));
    return NULL;
  }
}

int  GenericList::Append(void *data)
{
  TRACE(("GenericList::Append(%p) \n", data));

  GenericList::Element *elem = head;
  while ((elem!=NULL) && (elem->next!= NULL)) {
    elem = elem->next;
  }
  GenericList::Element *newelem = new GenericList::Element(data);
  newelem->next = NULL;
  if ( elem != NULL ) {
    elem->next = newelem;
  }
  else {
    head = newelem;
  }
  size ++;

  TRACE(("new size is %d\n", size));

  return (size - 1);
}
 
void GenericList::Insert(void *data)
{
  GenericList::Element *newelem = new GenericList::Element(data);
  newelem->next = head;
  head = newelem;
  size ++;
  TRACE(("GenericList::Insert(%p) new size is %d\n", data, size));
}

void  GenericList::RemoveAt(int index)
{
  TRACE(("GenericList::RemoveAt(%d) \n", index));

  if ( (head == NULL) || 
       (index < 0) ||
       (index >= size)) {
    /* Either list is empty or
     * the index is out of range */
    TRACE(("failed head==%p, index=%d, size=%d\n", head, index, size));
    return;
  }

  GenericList::Element *elem = NULL;
  if ( index == 0 ) {
    elem = head;
    head = head->next;
    elem->next = NULL;
  }
  else {
    elem = head;
    while ( --index > 0) {
      if ( elem ) {
        elem = elem->next;
      }
    } 
    if ( elem && elem->next ) {
      GenericList::Element *e = elem->next;
      elem->next = e->next;
      elem = e;
      elem->next = NULL;
    }
    else {
      elem = NULL; /* Nothing to delete */
    }
  }

  if ( elem ) {
    if ( allow_delete ) {
      DeleteElement(elem->data);
    }
    delete elem;
    size --;
    TRACE(("removed, new size=%d\n", size));
  }
  else {
    TRACE(("not removed, not found\n"));
  }
}

void GenericList::RemoveAll()
{
  TRACE(("GenericList::RemoveAll()\n"));

  if ( (head == NULL) || (size <=0 ) ) {
    /* list is empty */
    TRACE(("failed head==%p, size=%d\n", head, size));
    return;
  }

  GenericList::Element *elem = head;
  head = NULL;
  size = 0;

  while( elem != NULL ) {
    GenericList::Element *next_elem = elem->next;
    if ( allow_delete ) {
      DeleteElement(elem->data);
    }
    delete elem;
    elem = next_elem;
  }
}

ostream & operator<<(ostream & o, const GenericList & list)
{
  o << "size(" << list.size << "):";
  o << (list.allow_delete ? "+:" : "x:");
  GenericList::Element *elem = list.head;
  for ( int i=0; i<list.size; i++, elem = elem->next) {
    o << "->";
    if ( elem->data == NULL ) {
      o << "{NULL," << elem->next << "}";
    }
    else {
      o << "{" << elem->data << "," << elem->next << "}";
    }
  }
  return o;
}

GenericHeaderValue::GenericHeaderValue()
{
  header = NULL;
  value = NULL;
}

GenericHeaderValue::GenericHeaderValue(const char *attrib, const char *v)
{
  header = ((attrib==NULL) ? NULL : strdup(attrib));
  value = ((v==NULL) ? NULL : strdup(v));
}

GenericHeaderValue::GenericHeaderValue(const char *attrib, char separator)
{
  header = NULL; value = NULL;
  char sep[2] = { separator, '\0'} ;
  if ( attrib == NULL ) {
    return;
  }
  char *ptr = strdup(attrib);
  char *last = NULL;
  header = strtok_r(ptr, sep,&last);
  value = strtok_r(NULL, "\0", &last);
  header = (header != NULL) ? strdup(header) : NULL;
  value = (value != NULL) ? strdup(value) : NULL;
  free(ptr);
}

GenericHeaderValue::GenericHeaderValue(
                                       const
                                       GenericHeaderValue & g)
{
  header = ((g.header==NULL) ? NULL : strdup(g.header));
  value = ((g.value==NULL) ? NULL : strdup(g.value));
  
  //SetType(g.GetType());
  //SetValue(g.GetValue());
  return;
}

GenericHeaderValue & GenericHeaderValue::operator=(
                                                   const
                                       GenericHeaderValue & g)
{
  if ( this == &g ) {
    return *this;
  }

  SetType(g.GetType());
  SetValue(g.GetValue());
  return *this;
}

GenericHeaderValue::~GenericHeaderValue()
{
  if ( header ) { 
    free(header); 
  }
  if (value) {
    free(value);
  }
}

GenericHeaderValue::operator const char *() const
{
  return value;
}

void GenericHeaderValue::SetType(const char *h)
{
  if ( header ) {
    free(header);
  }
  header = ((h==NULL) ? NULL : strdup(h));
}

void GenericHeaderValue::SetValue(const char *v)
{
  if ( value ) {
    free(value);
  }
  value = ((v==NULL) ? NULL : strdup(v));
}

const char *GenericHeaderValue::GetType() const
{
  return header;
}

const char *GenericHeaderValue::GetValue() const
{
  return value;
}

GenericHeaderValue & GenericHeaderValue::operator=(const char *v)
{
  SetValue(v);
  return *this;
}

bool GenericHeaderValue::operator==(const GenericHeaderValue hv) const
{

  if ( header == NULL ) {
    /* Headers are null */
    return hv.header == NULL;
  }
  if ((hv.header == NULL) || 
      strcmp(hv.header,header)!=0) {
    /* Headers do not match */
    return false;
  }
  if ( value == NULL ) {
    return hv.value == NULL;
  }
  if ( (hv.value == NULL ) ||
       strcmp(hv.value,value)!=0) {
    /* values do not match */
    return false;
  }
  return true;
}

GenericHeaders::GenericHeaders(const char s)
{
  separator = s;
}

GenericHeaders::GenericHeaders(const GenericHeaders & h)
{
  headers   = h.headers;
  separator = h.separator;
}

GenericHeaders & GenericHeaders::operator=(const GenericHeaders & h)
{
  if ( this == &h ) {
    return *this;
  }

  headers   = h.headers;
  separator = h.separator;
  return *this;
}

GenericHeaders::~GenericHeaders()
{
}

int GenericHeaders::GetSize() const
{
  return headers.GetSize();
}

GenericHeaderValue & GenericHeaders::operator[](int index)
{
  if ( index < 0 ) {
    headers.Insert(GenericHeaderValue());
    index = 0;
  }
  else if (index >= GetSize()) {
    index = GetSize();
    headers.Append(GenericHeaderValue());
  }

  GenericHeaderValue *hv = (GenericHeaderValue *)headers.GetAt(index);
  return *hv;
}

const GenericHeaderValue GenericHeaders::operator[](int index) const
{
  return headers[index];
}

GenericHeaderValue & GenericHeaders::operator[](const char *attrib)
{
  static GenericHeaderValue empty((const char *)NULL, (const char *)NULL);

  int size = GetSize();
  for (int i=0; i<size; i++) {
    GenericHeaderValue *hv = (GenericHeaderValue *) headers.GetAt(i);
    const char *ptr = hv->GetType();
    if ( ptr && !strcasecmp(ptr, attrib)) {
      return *hv;
    }
  }
  return empty; /* XXX this is not correct */
}

const GenericHeaderValue GenericHeaders::operator[](const char *attrib) const
{
  GenericHeaderValue empty((const char *)NULL, (const char *)NULL);

  int size = GetSize();
  for (int i=0; i<size; i++) {
    const char *ptr = headers[i].GetType();
    if ( ptr && !strcasecmp(ptr, attrib)) {
      return headers[i];
    }
  }
  return empty;
}

GenericHeaders & GenericHeaders::operator+= (const GenericHeaderValue g)
{
  headers.Append(g);
  return *this;
}

GenericHeaders & GenericHeaders::operator+= (const char *headervalue)
{
  if (headervalue != NULL) {
    headers.Append(GenericHeaderValue(headervalue,separator));
  }
  return *this;
}

GenericHeaders & GenericHeaders::operator-= (const char *header)
{
  int size = GetSize();
  for (int i=size-1; i>=0; i--) {
    const char *ptr = headers[i].GetType();
    if ( ptr && !strcasecmp(ptr, header)) {
      headers.RemoveAt(i);
    }
  }
  return *this;
}

GenericHeaders & GenericHeaders::operator-= (const GenericHeaderValue g)
{
  int size = GetSize();
  for (int i=size-1; i>=0; i--) {
    if ((headers[i] == g)) {
      headers.RemoveAt(i);
    }
  }
  return *this;
}

bool GenericHeaders::operator & (const char *header) const
{
  int size = GetSize();
  for (int i=0; i<size; i++) {
    const char *ptr = headers[i].GetType();
    if ( ptr && !strcasecmp(ptr, header)) {
      return true;
    }
  }
  return false;
}

bool GenericHeaders::operator & (const GenericHeaderValue g) const
{
  int size = GetSize();
  for (int i=0; i<size; i++) {
    if ((headers[i] == g)) {
      return true;
    }
  }
  return false;
}

