/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : sipreg.h
#
#  Module       : SIP registration related information.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 23 Sept 2000
#
#  Description  : This file defines the C sipreg_t structure.
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/
 
/***********************************************************************
******************* INCLUDES *******************************************
*/

#ifndef SIP_REG_H
#define SIP_REG_H  1

#include <stdio.h>    /* FILE */
#include "ads.h"
#include "iptelapi.h"
#include "sipapi.h"

#ifdef __cplusplus
extern "C" {
#endif

#include "time.h"
#include "sys/time.h"

#include "request-sip.h"
#include "message.h"
#include "headers.h"

#ifdef __cplusplus
}
#endif

/*
 * Call state for sip reg 
 */
typedef enum sipregstate_t {
  SIPREG_IDLE,               /* SipReg is idle */
  SIPREG_OUTGOING_1,         /* sent REGISTER waiting for response */
  SIPREG_ACTIVE,             /* SipReg is active */
  SIPREG_OUTGOING_2,         /* Re-SENT REGISTER */
  SIPREG_OUTGOING_3,         /* Refreshing the REGISTER */
  SIPREG_TERMINATING,        /* SipReg is being terminated. */
  /* add new states here */

  SIPREG_STATE_COUNT
} sipregstate_t;

//forward declaration.
struct sipreg_t;

/**
 * sipreg_t structure stores all information related to SipReg state.
 */
typedef struct sipreg_t {
  sipregstate_t   state;         /* SipReg state */

  /* SIP related */
  char             *callid;       /* call id string */
  char             *self;         /* self address */
  char             *remote;       /* remote address */
  int              lastcseq_tx;   /* last sequence number used for og req. */

  uri_t            proxy;       /* address of the outbound proxy, port 
                                   * if zeron then use 5060. */
  sip_contact_t    contact;     /* The contact info */
  timespec         expireson;   /* To try refresh */

  /* FSM related */
  message_t        *mqueue;       /* message queue */
  pthread_t        tid;           /* thread id */
  pthread_mutex_t  mutex;         /* for lock */
  pthread_cond_t   cond;          /* for signaling */
  int              done;          /* 0 means SipReg thread is active */

  parser_t         p;             /* generic headers, parser for first request*/
  body_t           body;          /* last body received or sent */
  uri_t            remote_uri;    /* remote address where all, requests should go */

  parser_t         p_irs;         /* generic parser, headers for incoming res */
  headers_t        h_orq;         /* generic headers for outgoing requests */

  /* C++ object */
  SIPEndpoint     *regobj;        /* C++ registration object */

  int              type;          /* SOCK_STREAM, SOCK_DGRAM */

  int              incoming_reinvite; /* 1 if pending incoming
                                         reinvite */
  int              unauth_count;  /* Number of times unauth received
                                   */
  char             *userid;       /* Authorization info */
  char             *password;     /* Authorization info */

  /* next element in the list */
  struct sipreg_t * next;
} sipreg_t;

/* Functions are defined in sipreg.c */

/* SipRegCreation, deletion, and thread, FSM handling */
sipreg_t *SipRegCreate(const char *id);
void SipRegAdd(sipreg_t *c);
void SipRegFree(sipreg_t *r);
void SipRegPrint(FILE *fp);
sipreg_t *SipRegSearch(const char *callid);
void *SipRegThread(void *cid);
int  SipRegHandleRequest(sipreg_t *c, message_t *m);
int  SipRegHandleResponse(sipreg_t *c, message_t *m);
void SipRegComplaint(sipreg_t *c);
int  SipRegCleanup(sipreg_t *c);
void SipRegDone(sipreg_t *c);
void SipRegSetStatus(sipreg_t *c, int status, const char *reason);

int CallCreatePolicyThread(request_t *r, uri_t uri, uri_t proxy);


#endif
