/***********************************************************************
 **  authenticate.h
 **  
 **  Used for authenticate the SIP client
 **
 **  Functions:
 **    int authenticateBasic(char *userID, char *passwd, char *pwdFilename);
 **
 **  Xiaotao Wu, Henning Schulzrinne
 **  Last modified: 07/22/98
 ***********************************************************************/

#ifndef __AUTHENTICATE_H
#define __AUTHENTICATE_H

#include "http.h"
#include "dbaccess.h"

typedef enum {
  auth_success = 0,
  auth_no_user = -1,
  auth_wrong_password = -2,
  auth_db_error = -3,
  auth_nonce_stale = -4,
  auth_wrong_method = -5,
  auth_wrong_algorithm = -6,
  auth_wrong_realm = -7,
  auth_malformed_auth = -8,
  auth_internal_error = -9
} auth_status;

auth_status authenticate(http_authentication_method_t authMethod,
                         const put_table_entry *user,
                         http_authorization_t authorization);

int build_basic_auth_challenge(http_authenticate_t *authen,
                               const put_table_entry *user);

int build_digest_auth_challenge(http_authenticate_t *authen,
                                const char *opaque,
                                int wasStale,
                                const char *privateKey,
                                const put_table_entry *user);

typedef struct nonce {
  char* nonceData;
  struct nonce *next;
  struct nonce *prev;
  time_t evtTime;
} Nonce_t;

typedef struct {
  Nonce_t *first;
  Nonce_t *last;
  pthread_mutex_t nonces_lock;
} NonceHead_t;

extern NonceHead_t gCallIdCseqDateNonceHead;

extern int NonceFoundElseAddTimeOrdered(NonceHead_t *theNonceHead,
                                        char *theData,
                                        time_t theTimeOccurred,
                                        time_t theClockSkew);

extern NonceHead_t gNonceHead;

extern int AddNonce(NonceHead_t *theNonceHead,
                    char *theData,
                    time_t theLifeTime);

extern int NonceFound(NonceHead_t *theNonceHead,
                      char *theData,
                      int theRemoveFlag);

extern void FreeNonceList(Nonce_t *theFirstNonce);

#endif
