/*
 * canonicalizer_interface.h
 *  Interface exported by the canonicalizer library (libcanon.so).
 *
 *  Ideally this should be inside tools/canonicalize. But, to avoid
 *  too many changes to Makefile fragments, we keep this inside 
 *  libsip, and make canonicalizer depend on this file. 
 *
 * Copyright 2001 Columbia University.  All rights reserved.
 */

#ifndef CANONICALIZER_INTERFACE_H_
#define CANONICALIZER_INTERFACE_H_
#include "sysdep.h"
#include "uri.h"
#include "dbaccess.h"

struct canon_nameaddrs {
  char *name;
  uri_t addr;
};

typedef struct {
  enum {canon_error, canon_unique, canon_multiple, canon_remote} status;
  union {
    uri_t *unique;
    struct {
      int nameaddr_count;
      struct canon_nameaddrs *nameaddrs;
    } multiple;
    char *error;
  } u;
} canon_result;

/*
 * InitCanonicalize.
 *
 * Call during init, with appropriate command line options. 
 *
 * Returns 0 on success, non-zero on error. Prints error to stderr. 
 *
 * (The DSO_EXTERN macro will expand to extern in unix systems and
 * __declspec(dllexport) or __declspec(dllimport) in Win32 systems).
 */
DSO_EXTERN int InitCanonicalize(int argc, char *argv[]);

/*
 * InitCanonicalize2
 *
 * Same as InitCanonicalize, but now has an optional third parameter,
 * If the third parameter namely, optional_database is non-null, canon will
 * reuse the same connection for its use (useful for all cases, most
 * useful for fastsql). If it is null, it will then look into the
 * command line for a -T param.
 */
DSO_EXTERN int InitCanonicalize2(int argc, char *argv[], 
                                 cinema_table *optional_database);

/*
 * canonicalize_uri()
 * Canonicalize input_uri, and return canonicalized URI,
 * 
 * Returns 0 on success, 1 on error.
 *
 * Both the parameters must be non-null, out_result->status will be set. 
 */
DSO_EXTERN int canonicalize_uri(uri_t* input_uri, canon_result* out_result);

/*
 * CloseCanonicalize.
 *
 * Cleanup resources, and finish operation.
 */
DSO_EXTERN int CloseCanonicalize();

#endif /* canonicalizer_interface.h */



