#ifndef CONFIG_H_
#define CONFIG_H_

#include <sys/types.h>
#include <regex.h>
#include "request-sip.h"
#include "dstring.h"
#include "authenticate.h"
#include "dbaccess.h"

extern struct config_t {
  char *ServerRoot;      /* The root of the server, as specified in the
                            config file */

  /* IP addresses, hostnames */
  char *PrimaryIP;       /* Null to use INADDR_ANY, for binding */
  char *PrimaryIP6;      /* Null to use inaddr_any6, for binding */
  char *HostName6;       /* IPv6 host name, if != from gethostname() */
  char *ServerName;      /* host name of server */
  char *ProxyName;       /* host name of proxy added in Via/IPv4 headers */
  char *ProxyName6;      /* Via/IPv6 headers. If using
                          * numeric representation, must be enclosed by [ ]*/

  int  NumericVia;       /* Is ProxyName numeric? */

  /* general configuration */
  char *Canonicalize;    /* The location of the canonicalize program */
  char *ErrorLog;        /* The location of the error log */
  char *LogCondition;    /* The conditions under which requests are logged */
  char *LogFormat;       /* Format of request logs. */
  char *PidFile;         /* location of file for PID */
  char *ScriptBase;      /* root directory of script repository */
  regex_t *domain;       /* domain for registrations (regular expression) */
  sip_action_t foreign;  /* how to handle foreign domains */
  sip_action_t defaultreg; /* what a registration without an action means */

  /* authentication */
  http_authentication_method_t AuthMethod;
                         /* the authentication method */
  char *PrivateKey;      /* the private key for digest authentication */
  char *Realm;           /* the realm for authentications */
  
  int HelpResolve;       /* boolean, whether to resolve DNS name for client */

  gid_t gid;             /* effective group */
  uid_t uid;             /* effective user */
  int port;              /* the server port number */

  /* soft-state parameters */
  int expires;           /* default expiration time for registrations */
  int maxexpires;        /* max expiration time for SIP registrations */
  int timeout;           /* proxy timeout */
  int cgi_timeout;       /* Max. time a CGI script may run */

  /* proxy, registrar */
  int DisplayAmbiguous;  /* Whether all matches should be displayed in
                            case of ambiguous response */
  int ProxyRecursion;    /* Whether proxies should recurse on 3xx responses */
  int ThirdPartyReg;     /* Whether third-parties can perform registrations */
  int RegRespContacts;   /* Whether to send contact list in Registration
                            responses */
  time_t NonceLifeTime;  /* The life time of nonces */
  time_t ClockSkew;      /* Allowable clock skew between server and client */

  char *UserConfig;      /* pseudo-URL to user configuration information. 
                            Either an SQL database or a flat text file */
  char *GatewayMap;      /* File mapping tel URLs to gateways. */
  char *EnumRoot;        /* The name of enum root: like trial.e164.com */

  cinema_table *database; /* The system user, registration, etc., database */
  int  TryStateless;      /* Whether should try for stateless proxy */  
  int  UseThreadPool;     /* Whether to enable the pool? */
  int  ThreadPoolSize;    /* The number of threads in the thread-pool */
  int  RecordRoute;       /* Insert RecordRoute header? */

  char *AgentxSocketPath; /* The AF_UNIX socket to use for talking to snmpd */
  int  StartSnmp;         /* Start libsipsnmp or not? */
  
  char *Reserved;         /* Reserved */

} config;

#endif /* CONFIG_H_ */
