/*
 * message.h: manipulate messages and message queues.
 * Copyright 1998-1999 by Columbia University; all rights reserved 
 */

#ifndef MESSAGE_H_
#define MESSAGE_H_

typedef enum {
  message_real_message, message_pseudo_message,
  message_cancel_branch, message_shutdown_client,
  message_socket_error, message_api_message
} message_type_t;

typedef enum {
  nobody_handled, branch_handled, policy_handled, finished_handling
} handled_by_t;

/* Forward type declaration */
typedef struct message_t message_t;

/* List of pending messages; used for communication with policy
 * (e.g. proxying) */
struct message_t {
  message_type_t type;      /* The type of message this is */
  handled_by_t handled;     /* Who handled this request most recently */
  parser_t p;               /* The parsed message headers */
  body_t body;              /* The message body */
  struct sockaddr_storage source; /* The address this came from */
  branch_t *branch;         /* The branch this message arrived on (if it's
                               a response) */
  void *user_policy_info;   /* Info the user policy can insert.  Should be a
                               single malloc'd block, or NULL */
  void *transaction_policy_info; /* Info the user policy can insert.  Should
                                    be a single malloc'd block, or NULL */

  message_t *next;          /* The next message in the queue */
};


/* Manipulate messages */

/* the sin parameter must be non null */
message_t *MessageCreate(message_type_t type, parser_t *p,
                         body_t *body, const struct sockaddr_storage* sin, 
                         branch_t *b, handled_by_t handled);
message_t *MessageCreatePseudoResponse(branch_t *b, handled_by_t handled,
                                       int status);
message_t *MessageCreateTrivial(message_type_t type, handled_by_t handled);
message_t *MessageDuplicate(message_t *m);
void MessageDelete(message_t *m);
message_t *GetRequestAsMessage(struct request_t *r);

/* handle message queues */

/* source must be non-null */
void MessageEnqueue(message_t **list, message_type_t type, parser_t *p,
                    body_t *body, const struct sockaddr_storage *source, 
                    branch_t *b, handled_by_t handled);
void MessageEnqueuePseudoResponse(message_t **list, branch_t *b,
                                  handled_by_t handled, int status);
void MessageEnqueuePseudoRequest(message_t **list, const char *method,
                                  handled_by_t handled,
                                  body_t *body);
void MessageTrivialEnqueue(message_t **list, message_type_t type,
                           handled_by_t handled);
void MessageEnqueueApiRequest(message_t **list, const char *method,
                              handled_by_t handled);
void MessageEnqueueApiResponse(message_t **list, int status, 
                               const char *reason,
                               handled_by_t handled);

message_t *GetNextMessage(struct message_t **mlist);
void MessageReallyEnqueue(message_t **list, message_t *new1);


#endif /* MESSAGE_H_ */
