#include "audioBuffer.h"


audioBuffer::audioBuffer()
{
  Initialize();
}



audioBuffer::~audioBuffer()
{
  pthread_mutex_destroy(&mutex);	  
}



void  audioBuffer::Initialize()
{
  
  r_ptr = 0; 
  w_ptr = 3200; 
  ni = 0;
  ni_previous_1 = 0;
  ni_previous_2 = 0;
  di = 0; 
  di_previous_1 = 0;   
  vi = 0;
  vi_previous_1 = 0;
  initialoffset = 0;
  pi = 0; pj = 0;
  PJ = 0;
  ti = 0; tj = 0;
  Just_Started = 1;
  last_seq = 0;
  index =0;
  late_count = 0;
  var = 0;
  
  algorithm = ALG_EXP_AVG;
  mode = NORMAL;
  pthread_mutex_init(&mutex, NULL);
  
}



int  audioBuffer::CompareIndex()
{
  if ( r_ptr < w_ptr ) 
    return  1;
  else if ( r_ptr > w_ptr ) 
    return  1;
  else
    return  0;
}



void   audioBuffer::IncrementIndex( int * ptr)
{
  *ptr = ( *ptr + 1 ) % MAX_PLAYOUT_BUFFER_SIZE;
}



void   audioBuffer::Copy_To_Cirular_Buf(unsigned short *packet_buf, int len)
{
  int i;
  while (index != w_ptr){
    circular_buf[w_ptr]=0;
    IncrementIndex(&w_ptr);
	 }
  //printf("Copy_To_Cirular_Buf\n");
  //printf("r_ptr = %d, W_ptr = %d, Index = %d\n",r_ptr, w_ptr, index);
  //w_ptr = index;
  
  for (i=0; i<len; i++) {
    circular_buf[w_ptr] = packet_buf[i];
    if ( !CompareIndex() ) {
      circular_buf[r_ptr] = 0; /* make it empty and increment */
      IncrementIndex(&r_ptr);
    }
    IncrementIndex(&w_ptr);
  }
  //printf("r_ptr = %d, W_ptr = %d, Index = %d\n",r_ptr, w_ptr, index);
}





void   audioBuffer::Copy_From_Circular_Buf(unsigned short *packet_buf, int len)
{
  int i;
  //printf("Copy_From_Circular_Buf\n");
  //printf("r_ptr = %d, W_ptr = %d, Index = %d\n",r_ptr, w_ptr, index);
  
  for (i=0; i<len; i++) {
    if (CompareIndex()) { 
      packet_buf[i] = circular_buf[r_ptr];
      circular_buf[r_ptr] = 0;
      IncrementIndex(&r_ptr);
    }
    else {           
      packet_buf[i] = 0; /* silence */
      IncrementIndex(&r_ptr);
      IncrementIndex(&w_ptr);
    }
    
  }
  //printf("r_ptr = %d, W_ptr = %d, Index = %d\n",r_ptr, w_ptr, index);
  
  
}



void   audioBuffer::Store_Packet(rtp_hdr_t * rtp, 
                                 char *payload,
                                 int payload_len,
                                 long receiver_timestamp)
{
  tj = rtp->ts;
  current_sequence = rtp->seq;
  Marked = (( rtp->mpt & 0x80) ? 1:0);
  ai= receiver_timestamp;
  
  pthread_mutex_lock(&mutex);
  
  Get_Index();
  Copy_To_Cirular_Buf((unsigned short *)payload,
    payload_len/2);
  
  //printf("r_ptr = %d, W_ptr = %d, Index = %d\n",r_ptr, w_ptr, index);
  //printf("pj= %lu, pi = %lu, di = %f, vi = %f\n", pj, pi, di, vi);
  
  pthread_mutex_unlock(&mutex);
  
  
}



void  audioBuffer::Retrieve_Packet(unsigned short *buffer, int len)
{
  pthread_mutex_lock(&mutex);
  
  Copy_From_Circular_Buf(buffer, len);
  
  pthread_mutex_unlock(&mutex);
  
  
}


void   audioBuffer::Get_Index()
{
  
  Check_Sequence();  
  Update_Delay_Mean_Variance();
  Set_Playout_Time();
  Map_Playout_Time_To_Index();
  if ( Just_Started ) Just_Started = 0;
  
}


void   audioBuffer::Check_Sequence()
{
  if ( Just_Started ) {
    last_seq = current_sequence;
  }
  else {
    last_seq ++;
    if ( current_sequence < last_seq ) {
      late_count++;
      //EK528 have to drop packet if too late
      // Compare with r_ptr
      last_seq = current_sequence;
    }
  }
  
}



void   audioBuffer::Update_Delay_Mean_Variance()     
{
  ni = (double) (ai - tj); /* network delay */
  if ( Just_Started ) {
    ti = tj;
    di = ni;
    vi = 0;
    ni_previous_1 = ni;
    ni_previous_2 = ni;
    di_previous_1 = di;
    vi_previous_1 = vi;
    
  }
  else {
    switch (algorithm)
    {
    case ALG_EXP_AVG:
      di = ALPHA * di + ( 1-ALPHA) * ni;
      vi = ALPHA * vi * ( 1-ALPHA) * fabs( (double) di - (double) ni);
      break;
    case ALG_FAST_EXP_AVG:
      if (ni < di)
        di = ALPHA * di + ( 1-ALPHA) * ni;
      else 
        di = BETA * di + ( 1-BETA) * ni;
      vi = ALPHA * vi * ( 1-ALPHA) * fabs( (double) di - (double) ni);
      break;
    case ALG_MIN_DELAYS:
      di = Minimum  (ni,di);
      vi = ALPHA * vi * ( 1-ALPHA) * fabs( (double) di - (double) ni);
      break;
    case ALG_SPK_DELAY:
      if (mode == NORMAL){
        if ( fabs(ni- ni_previous_1) > fabs (vi) * 2 + 800 ){
          var = 0; /*detected beginning of spike */
          mode = IMPULSE;
        }
      }
      else {
        var = var/2 +fabs((2 * ni - ni_previous_1 - ni_previous_2)/8);
        if (var <= 63){
          mode = NORMAL;
          ni_previous_2 = ni_previous_1;
          ni_previous_1 = ni;
          break;
        }
      }
      if (mode == NORMAL) 
        di = 0.125 * ni + 0.875 * di_previous_1;  
      else
        di = di_previous_1 +  ni - ni_previous_1;
      vi = 0.125 * fabs (ni - di) + 0.875 * vi_previous_1;
      ni_previous_2 = ni_previous_1;
      ni_previous_1 = ni;
      di_previous_1 = di;  
      vi_previous_1 = vi;  
      break;
    default:
      break;
    }
  }
}



void   audioBuffer::Set_Playout_Time()
{
  if ( Just_Started ) {
    ti = tj;
    pi = (long)( ti + di + MU * vi);
    pj = pi;
    PJ = pi;
    
  }
  else {
    if ( Marked == 1 ) { /* First packet in talk spurt */
      ti = tj;
      pi = (long)( ti + di + MU * vi);
      if ( pi < PJ + 321) pi = PJ + 321;
      pj = pi;
    } 
    else { /* Subsequent packets in talk spurt */
      pj = pi + tj - ti; 
    }    
  }
  if (PJ <= pj) PJ = pj;
}




void   audioBuffer::Map_Playout_Time_To_Index()
{       
  if ( Just_Started ) {
    initialoffset = -pj;
    index = INITIALOFFSET;
  }
  else {
    index = ( (unsigned) (  pj + initialoffset + INITIALOFFSET))
      % MAX_PLAYOUT_BUFFER_SIZE;
  }
}      

double  audioBuffer::Minimum(double a, double b)
{
  if (a < b ) return a;
  else return b;
}






