////audioBuffer.h

#ifndef AUDIO_BUFFER
#define AUDIO_BUFFER

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <pthread.h>

#include "rtp_api.h"
#include "rtp_highlevel.h"

#define MAX_PLAYOUT_BUFFER_SIZE 16000   /* 1 sec of 16bits 16khz */
#define ALPHA                   0.95    /* weighting factor */
#define BETA                    0.75    /* used in ALG_FAST_EXP_AVG */
#define MU                      4       /* safety margin */

#define INITIALOFFSET   3200            /* delays Audio start by 200ms */

#define ALG_EXP_AVG             1       /* Exponential Weighted Average */
#define ALG_FAST_EXP_AVG        2       /* Faste Exponential Weighted Average */
#define ALG_MIN_DELAYS          3       /* Minimum Delay Average */
#define ALG_SPK_DELAY           4       /* Spike Delay Average */

#define NORMAL 	                1       /* Normal delay mode */
#define IMPULSE	                2       /* Spike/Impulse Delay mode */


class audioBuffer {
  
public:
  /* Default Constructor */
  audioBuffer(); 
  
  /* Default Destructor */
  ~audioBuffer();
  
  /* Store_Packet: Stores into the circular playout buffer */
  void Store_Packet(rtp_hdr_t * rtp, 
			 char *payload,
       int payload_len,
       long local_timestamp);
  
  /* Retrieve_Packet: Retrieves from the circular playout buffer */
  void Retrieve_Packet(unsigned short *buffer, int len);
  
  //private:
  
  /* Initializes the class varialbles */
  void Initialize();
  
  /* Compares the read and write indexes */
  int CompareIndex();
  
  /* Increments the index */ 
  void IncrementIndex( int * );
  
  /* Copy_To_Cirular_Buf copies from the packet */
  void Copy_To_Cirular_Buf(unsigned short *packet_buf, int len);
  
  /* Copy_From_Circular_Buf copies to packet */
  void Copy_From_Circular_Buf(unsigned short *packet_buf, int len);
  
  void Get_Index(); 
  void Check_Sequence();  
  void Update_Delay_Mean_Variance();
  void Set_Playout_Time();
  void Map_Playout_Time_To_Index(); 
  
  double Minimum(double a, double b);
  
  unsigned short circular_buf[MAX_PLAYOUT_BUFFER_SIZE];
  int  r_ptr;                                    /* read pointer */
  int  w_ptr;                                    /* write pointer */
  double  ni;                                    /* Delay = ti - local timestamp */
  double ni_previous_1;
  double ni_previous_2;
  double di;                                     /* Average delay */
  double di_previous_1;
  double vi;                                     /* Delay Variance */
  double vi_previous_1;
  long  pi, pj;                                  /* Playout time */
  long PJ;
  long  ti, tj;                                  /* sender time stamp */
  long  ai;                                      /* receiver time stamp */
  long initialoffset;                            /* Lets index = 0, for first packet */
  int index;                                     /* Circular buffer index */
  int late_count;                                /* Number of late arrivals */
  int Just_Started;                              /* Indicate if First received packet */
  unsigned short int current_sequence;
  unsigned short int last_seq;                   /* Last packet sequence number */
  unsigned char Marked;                          /* First packet in a talk spurt */
  
  pthread_mutex_t mutex;                         /* Protects the circular Buffre*/ 
  
  double var;		                                 /* a variable for spike delay detection */  
  
  int algorithm;
  int mode;
  
};

#endif
