#include <stdlib.h>
#include <iostream>
#include "audioDevice.h"

extern "C" {
  void g_seven22_coder(short *data, int NUM_INDATA, 
    unsigned short *outdata, int NUM_OUTDATA); 
  void g_seven22_decoder(unsigned short *decdata, int NUM_ADPCM, 
    short *pcmout,int NUM_PCM);
}



audioDevice::audioDevice() {
  format=FORMAT;
  stereo=CH_MODE;
  speed=SPEED;
  
  audio_samples=AUDIO_SAMPLES;
  buf = (short *)calloc(audio_samples, sizeof(short));
}

audioDevice::~audioDevice(){
  if ( buf != NULL ) {
    free(buf);
  }
}

int audioDevice::Open_audioDeviceice() {
  
  printf("Opening Audio Device\n");
  if ((audio_fd = open(DEVICE_NAME, OPEN_MODE, 0))==-1) {
    perror("Error in opening device");
    return -1;
  }
  printf("Opened Audio Device\n");
  Set_Audio_Parameters();
  printf("Set the params for Audio Device\n");
  return audio_fd;
} 

void audioDevice::Set_Audio_Parameters(){ 
  int parameter;  
  parameter=format;
  if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format)==-1) 
    perror("Error in setting format!");
  if (parameter!=format) perror("Requested Audio Format Not Supported!");
  
  parameter=stereo;
  if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo)==-1) 
    perror("Error in setting stereo!");
  if (parameter!=stereo) perror("Requested Audio Channel Mode Not Supported!");
  
  parameter=speed;
  if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &speed)==-1)
    perror("Error in setting speed!");
  if (parameter!=speed) perror("Requested Audio Speed Not Supported!");
  
}

void audioDevice::Close_audioDeviceice(){
  if (close(audio_fd)==-1) perror("Error in closing device!");
}

void audioDevice::Record_From_Mic() {
  int totallen = 0;
  int len;
  while (totallen < audio_samples*2)
  {
    len = read(audio_fd, (AUDIO_BYTE_BUF*)&buf[totallen], 
      (audio_samples*2) - totallen);
    if (len <= 0) {
      perror("read from audio device");
      break;
    }
    totallen += len;
  }
  if (totallen != audio_samples*2) {
    printf("Could not read %d bytes from audio device: %d\n", audio_samples*2,
	     totallen);
  }
}

int audioDevice::Playback_To_Speaker(AUDIO_SAMPLE_BUF* S16sample, int num_of_S16_samples) {
  return write(audio_fd, (AUDIO_BYTE_BUF*) S16sample, num_of_S16_samples*2);
}

int audioDevice::Convert_From_S16_To_G722( AUDIO_SAMPLE_BUF* S16sample,
                                          int num_of_S16_samples, 
                                          AUDIO_COMPRESSION_BUF* G722sample, 
                                          int num_of_g722_samples) {
  g_seven22_coder(S16sample, num_of_S16_samples, 
    G722sample, num_of_g722_samples);
  
  return 0;
}

int audioDevice::Convert_From_G722_To_S16( AUDIO_COMPRESSION_BUF* G722sample,
                                          int num_of_g722_samples, 
                                          AUDIO_SAMPLE_BUF* S16sample, 
                                          int num_of_s16_samples) {
  
  g_seven22_decoder(G722sample, num_of_g722_samples,
    S16sample, num_of_s16_samples);
  return 0;
}

double audioDevice::Get_Energy(  AUDIO_SAMPLE_BUF* S16sample, int num_of_S16_samples ) {
  //FILE *fp;
  // fp=fopen("Audio_data","wt");
  
  double sum=0.0;
  for (int i=0; i<num_of_S16_samples; i++)
  {
    sum+=( double)S16sample[i]*( double)S16sample[i];
    //fprintf( fp, "%d\n", S16Sample[i] );
    //printf( "s16Sample = %d\n", S16sample[i] );
	 }
  //fclose(fp);
  
  return sum/( double)num_of_S16_samples;
}
