#ifndef AUDIO_DEVICE
#define AUDIO_DEVICE

#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <stdio.h>          
#include <sys/ioctl.h>      

#include <new.h>            
#include <math.h>  

// CONSTANT DEFINITION SECTION 
// AUDIO Related

#define DEVICE_NAME  "/dev/audio"   /* options: audio, dsp, dspw.           */
#define OPEN_MODE    O_RDWR         /* options: O_RDWR, O_RDONLY, O_WRONLY  */
#define FORMAT       AFMT_S16_LE
#define CH_MODE      0              /* 0 for Mono, 1 for Stereo             */
#define SPEED        16000          /* 16Khz                                */
#define AUDIO_SAMPLES 320           /* 20ms for 16Khz Mono                  */

//  Audio data type define
#define AUDIO_SAMPLE_BUF        short     // AFMT_S16_LE: 2byte
#define AUDIO_COMPRESSION_BUF   unsigned short
#define AUDIO_BYTE_BUF  unsigned char   // audio device buffer type:1byte

class audioDevice {
  
public:
  /// Default Constructor
  audioDevice();
  
  /// Destructor
  ~audioDevice();
  
  int Open_audioDeviceice( );
  void Set_Audio_Parameters();
  void Close_audioDeviceice();
  void Record_From_Mic();
  int Playback_To_Speaker(   AUDIO_SAMPLE_BUF * S16sample, int num_of_S16_samples );
  int Convert_From_S16_To_G722( AUDIO_SAMPLE_BUF* S16sample,
    int num_of_S16_samples,
    AUDIO_COMPRESSION_BUF* G722sample, 
				int num_of_g722_samples );
  int Convert_From_G722_To_S16( AUDIO_COMPRESSION_BUF* G722sample,
    int num_of_g722_samples,
    AUDIO_SAMPLE_BUF* S16sample,
				int num_of_S16_samples);
  double Get_Energy(  AUDIO_SAMPLE_BUF* S16sample, int num_of_S16_samples );
  int Get_Number_Of_Samples() { return audio_samples; };
  AUDIO_SAMPLE_BUF* Get_Buf() { return buf; };
  
  //private:
  int audio_fd;
  int format;
  int stereo;
  int speed;
  int audio_samples;
  
  AUDIO_SAMPLE_BUF *buf;
  
}; // audioDevice

//  End of File: audioDevice.h

#endif
