/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : command.h
#
#  Module       : Main file Command interpreter.
#
#  Author(s)    : Kundan Singh (kns10@cs.columbia.edu)
#
#  Created      : 2 Aug 2000
#
#  Description  : 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#-----------------------------------------------------------------------
*/

#ifndef MY_COMMAND_H
#define MY_COMMAND_H    1

#include <iostream.h>
#include <iptelapi.h>
#include <ads.h>

/**
 * CommandObject is a command interpreter for any
 * text based user interface.
 * It is derived from GenericHeaders. All
 * the parameters, arguments of the command are
 * stored in the list.
 * e.g., 
 *  start  p1=v1 p2=v2 arg1 arg2
 * will have 
 * list[0] = list["command"] = "start"
 * list[1] = list["1"]  = arg1
 * list[2] = list["2"]  = arg2
 * list[3] = list["p1"] = "v1"
 * list[4] = list["p2"] = "v2"
 */

class CommandObject : public GenericHeaders
{
public:
  /**
   * Parse a command 
   */
  CommandObject(const char *cmd, GenericHeaders & globals);

  /*
   * Substitute the variables from another list.
   */
  char * Normalize(const char *p, GenericHeaders & g);

  /**
   * retrieve the command name.
   */
  operator const char * ();

  /**
   * compare the command name 
   */
  bool operator==(const char *);

  /**
   * Get the number of arguments
   */
  int GetArgsCount();

  /**
   * Compare the ith argument with the parameter passwd.
   */
  bool IsParameter(int i, const char *p);

  /**
   * Get teh string till the end
   * from this index argument.
   * This method is not re-entrant.
   */
  const char * TillEnd(int index);

  /**
   * print the syntax for the command, or any error.
   */
  friend ostream & operator<<(ostream & o, CommandObject & addr);

private:
  /* Argument count */
  int argc;
  /* parameter count */
  int paramc;
};

#endif /* MY_COMMAND_H */
