#include <stdlib.h>
#include <string.h>
#include "rtp_api.h"
#include "rtp_highlevel.h"
#include "audioBuffer.h"
#include "audioDevice.h"

extern "C" {
#include "error.h"
}

#define PACKET_LIST_COUNT 64

#define MASK(f,i)  ((((i==0) ? ((f)->mask) : ((f)->mask >> i)) & 0x01) != 0)
#define A(x) ((A_buffer[(x) / 32] & (1 << (31-((x) % 32)))) != 0)
#define A_set(x,val)  (val==1 ? (A_buffer[(x) / 32] |= (1 << (31 - (x) % 32))) \
: (A_buffer[(x) / 32] &= ~(1 << (31 - ((x) % 32)))  ))


typedef struct packet_s {
  
  int fec;                   /* FEC (1) or media (0) */
  
  int sn;                    /* SN of the packet, for media only */
  
  int mask;                  /* Mask, FEC only. Only LSB 24 bits used */
  int snbase;                /* SN Base, FEC only */
  
  rtp_packet *rtp;
  struct packet_s *next;
} packet;

unsigned long A_buffer[2048];
rtp_packet *packet_list[PACKET_LIST_COUNT];
packet *pending_list = NULL;

/*
* InsertPacketInList:
* Stores the rtp media packets for later use in recovery process.
*/
int InsertPacketInList(rtp_packet *r2, int sn)
{
  if (r2 == NULL || r2->RTP_header == NULL || r2->payload == NULL) {
    debug("InsertPacketInList", "invalid param\n");
    return 0;
}
  
  
  /*
  *copy the rtp packet structure locally
  *fill : RTP_header, RTP_extension, payload, payload_len
  */
  rtp_packet *rtp;
  
  rtp = (rtp_packet *)calloc(1, sizeof(rtp_packet));
  memcpy(rtp, r2, sizeof(rtp_packet));
  
  rtp->RTP_header = (rtp_hdr_t *) calloc(1,sizeof(rtp_hdr_t));
  memcpy(rtp->RTP_header, r2->RTP_header, sizeof(rtp_hdr_t));
  
  rtp->RTP_extension = NULL;
  
  rtp->payload = (char *)calloc(r2->payload_len, sizeof(char));
  memcpy(rtp->payload, r2->payload, r2->payload_len);
  rtp->payload_len = r2->payload_len;
  
  
  /*
  * Insert the copied packet into the list
  * if the same packet is there dont do anything
  * if an old packet is there, replace it.
  */
  int index = (sn % PACKET_LIST_COUNT);
  if ( packet_list[index] != NULL) {
    rtp_packet *p = packet_list[index];
    if (p->RTP_header->seq == rtp->RTP_header->seq){
      return 0;
    } else {
      if (p->RTP_header) {
        free(p->RTP_header);
        p->RTP_header = NULL;
      }
      if (p->RTP_extension) {
        // printf("ERROR: fatal error, extension is present\n");
        free(p->RTP_extension);
        p->RTP_extension = NULL;
      }
      if (p->payload) {
        free(p->payload);
        p->payload = NULL;
      }
      packet_list[index] = NULL;
    }
  }
  
  packet_list[index] = rtp;
  return 1;
}

/*
* packetfree:
* free a packet structure from memory
* mainly frees its rtp structure variable
*/
void packetfree(packet *p)
{
  if(p) {
    if (p->rtp) {
      if (p->rtp->RTP_header) {
        free(p->rtp->RTP_header);
        p->rtp->RTP_header = NULL;
      }
      if (p->rtp->RTP_extension) {
        // printf("ERROR: fatal error, extension is present\n");
        free(p->rtp->RTP_extension);
        p->rtp->RTP_extension = NULL;
      }
      if (p->rtp->payload) {
        free(p->rtp->payload);
        p->rtp->payload = NULL;
      }	
      p->rtp = NULL;
    }
    p = NULL;
  }
}
/* 
* add_fec_to_pending_list:
* stores the fec packet in the list 
* for later use in recovery process.
*/
void add_fec_to_pending_list(packet *p)
{
  p->next = pending_list;
  pending_list = p;  
}
/*
* remove_fec_from_pending_list:
* removes the fec from the list when 
* it is no longer needed
*/
void remove_fec_from_pending_list(packet *p)
{
  packet **i = &pending_list; 
  for(;(*i) != NULL && ((*i) != p); i = &((*i)->next)) {
    /* do nothing */; 
  } 
  if (*i != NULL) {
    (*i) = (*i)->next;
  }
  if (p != NULL) {
    packetfree(p);
  }
}
/*
* recover_packet:
* returns a packet structure whose rtp packet variable
* is the newly recoverd rtp media packet.
*/
packet *recover_packet(int pkt_to_recover, packet *fec_pkt, rtp_packet *p1[3])
{
  unsigned char pt1, pt2, pt3;
  unsigned long ts1, ts2, ts3;
  
  packet *p = (packet *)calloc(1,sizeof(packet));
  rtp_packet *rtp = (rtp_packet *)calloc(1,sizeof(rtp_packet));
  
  // recover the payload length from protection operation
  int length = p1[0]->payload_len ^ p1[1]->payload_len ^ ((unsigned short *)fec_pkt->rtp->payload)[1];
  if (length > 1000) {
    // printf("Fatal error, length is %d\n", length);
    return NULL;
  }
  
  // fill the packet structure variables
  // for media packets: fec, sn and the rtp structure. 
  p->fec = 0;
  p->sn  = pkt_to_recover;
  
  if (length > 0) {
    p->rtp = rtp;
    
    /*
    * fill the rtp structure after performing the protection operation
    * on the payload and other header fiels.
    */
    
    // payload_len
    rtp->payload_len = length;
    
    // payload
    rtp->payload = (char *) calloc(length, sizeof(unsigned char));
    for (int ind = 0; ind < length; ind++) {
      rtp->payload[ind] = 0;
      if (ind < p1[0]->payload_len) {
        rtp->payload[ind] ^= p1[0]->payload[ind];
      }
      if (ind < p1[1]->payload_len) {
        rtp->payload[ind] ^= p1[1]->payload[ind];
      }
      if (ind < (fec_pkt->rtp->payload_len - 12)) {
        rtp->payload[ind] ^= fec_pkt->rtp->payload[12+ind];
      }
    }
    
    // RTP_extension
    rtp->RTP_extension = NULL;
    
    // RTP_header
    rtp->RTP_header = (rtp_hdr_t *)calloc(1,sizeof(rtp_hdr_t));
    SET_RTP_VERSION(*(rtp->RTP_header), 2);
    SET_RTP_P(*(rtp->RTP_header), 0); 
    SET_RTP_X(*(rtp->RTP_header), 0);
    SET_RTP_CC(*(rtp->RTP_header), 0);
    SET_RTP_M(*(rtp->RTP_header), 0);
    pt1 = (p1[0]->RTP_header->mpt & 0x7f);
    pt2 = (p1[1]->RTP_header->mpt & 0x7f);
    pt3 = (fec_pkt->rtp->payload[4] & 0x7f);
    SET_RTP_PT(*(rtp->RTP_header), pt1 ^ pt2 ^ pt3);
    
    rtp->RTP_header->seq = pkt_to_recover; 
    
    ts1 = p1[0]->RTP_header->ts;
    ts2 = p1[1]->RTP_header->ts;
    ts3 = ((unsigned long *)fec_pkt->rtp->payload)[2];
    rtp->RTP_header->ts = ts1 ^ ts2 ^ ts3;
    
    rtp->RTP_header->ssrc = p1[0]->RTP_header->ssrc;
  }
  else {
    free(rtp);
  }
  
  return p;
}

/* 
* recover_with_fec:
* checks if all needed packets for recovery are present.
* if yes recovery procees is launched
* if there is no need to recover (all packets are there)
* then remove the fec from pending lis
* if not all packets needed are there don t do anything yet.
*
*/

packet *recover_with_fec(packet *fec_pkt) {
  
  packet *data_pkt;
  int pkts_present,       /* number of packets from the mask that are
    present */
    pkts_needed,          /* number of packets needed is the number 
													  of ones in the mask minus 1 */
    pkt_to_recover = 0,   /* sn of the packet we are recovering */
    i;
  rtp_packet *p1[3];
  int p1_count = 0;
  memset(p1, 0, sizeof(p1));
  
  pkts_present = 0;
  
  /* 
  * The number of packets needed is the number of ones in the mask
  * minus 1.  The code below increments pkts_needed by the number
  * of ones in the mask, so we initialize this to -1 so that the
  * final count is correct 
  */
  
  pkts_needed = -1;
  
  /* 
  * Go through all 24 bits in the mask, and check if we have
  * all but one of the media packets 
  */
  
  for(i = 0; i < 3; i++) {
    
    /* If the packet is here and in the mask, increment counter */
    
    /* printf("if A(i[%d]+snbase[%d])=%d && MASK(%d)=%d\n",
				 	i, fec_pkt->snbase, A(i+fec_pkt->snbase) , i, MASK(fec_pkt,i)); */
    
    if(A(i+fec_pkt->snbase) && MASK(fec_pkt,i)) {
      pkts_present++;
      p1[p1_count++] = packet_list[(i+fec_pkt->snbase) % PACKET_LIST_COUNT];
    }
    
    /* Count the number of packets needed as well */
    if(MASK(fec_pkt,i)) {
      pkts_needed++;
    }
    
    /* The packet to recover is the one with a bit in the
    mask that's not here yet */
    if(!A(i+fec_pkt->snbase) && MASK(fec_pkt,i)) {
      pkt_to_recover = i+fec_pkt->snbase;
    }
  }
  
  /* If we can recover, do so. Otherwise, return NULL */
  
  if(pkts_present == pkts_needed) {
    data_pkt = recover_packet(pkt_to_recover, fec_pkt, p1);
  } else if (pkts_present > pkts_needed) {
     remove_fec_from_pending_list(fec_pkt);
     data_pkt = NULL;
  }
  else {
     data_pkt = NULL;
  }
  return(data_pkt);
}

/*
* fec_recovery : the main function
* performs the logic for: 
* -storing media and fec packets appropriately,
* -recovering lost media packets as soon as possible:
*  when an fec packe arrives or when another necessary media packet arrives
*
*/
void fec_recovery(rtp_packet *rtp, int fec, unsigned long timestamp, audioBuffer *audio_buf, audioDevice *audio_dev) 
{
  
  packet  *p,       /* packet received */
    *fecp,    /* fec packet from pending list */
    *pnew;    /* new packets recovered */
  char *uncompressed_payload;
  if (rtp == NULL || rtp->RTP_header == NULL || rtp->payload == NULL) {
    debug("fec_recovery", "Invalid parameter in fec_recovery\n");
  }
  /* 
  * create a new packet structure
  * fill fec
  * if fec, fill mask and snbase
  * if media, fill sn
  * fill rtp packet
  *
  */
  p = (packet *)calloc(1, sizeof(packet));
  p->fec = fec;
  if (fec == 0) {
    p->sn = rtp->RTP_header->seq;
  }
  if (fec == 1) {
    p->mask   = ((rtp->payload[5] << 16) & 0x0ff0000 | 
      (rtp->payload[6] << 8 ) & 0x000ff00 | 
      (rtp->payload[7]      ) & 0x00000ff);
    
    p->snbase = ((rtp->payload[0] << 8 ) & 0x0ff00 | 
      (rtp->payload[1]      ) & 0x000ff);
  } 
  p->rtp = (rtp_packet *)calloc(1, sizeof(rtp_packet));
  memcpy(p->rtp, rtp, sizeof(rtp_packet));
  
  p->rtp->RTP_extension = NULL;
  
  p->rtp->RTP_header = (rtp_hdr_t *) calloc(1,sizeof(rtp_hdr_t));
  memcpy(p->rtp->RTP_header, rtp->RTP_header, sizeof(rtp_hdr_t));
  
  p->rtp->payload = (char *)calloc(rtp->payload_len, sizeof(char));
  memcpy(p->rtp->payload, rtp->payload, rtp->payload_len);
  p->rtp->payload_len = rtp->payload_len;
  
  uncompressed_payload = (char *)calloc(rtp->payload_len*4, sizeof(char));
  
  /* if it's an FEC packet, try to recover with it. If we can't,
  * store it for later potential use. If we can recover, act as
  * if the recovered packet is received and try to recover some
  * more.  Otherwise, if it's a data packet, mark it as received,
  * and check if we can now recover a data packet with the list
  * of pending FEC packets 
  */
  
  if(p->fec == 1 /* TRUE */) {
    pnew = recover_with_fec(p);
    if(pnew) {
      A_set(pnew->sn,1);
      InsertPacketInList(pnew->rtp, pnew->sn);
      audio_dev->Convert_From_G722_To_S16((unsigned short *)pnew->rtp->payload,
        pnew->rtp->payload_len/2,
        (short *)uncompressed_payload,
        pnew->rtp->payload_len*2);
      audio_buf->Store_Packet(pnew->rtp->RTP_header, 
        uncompressed_payload, 
        pnew->rtp->payload_len*4,
        timestamp);
      packetfree(p);
    }
    else {
      add_fec_to_pending_list(p);
    }
    
    /* We assign pnew to p since the while 
		 * loop will continue to recover based 
		 * on p not being NULL 
		 */
    p = pnew;
  } else {
    /* Mark this data packet as here */
    A_set(p->sn,1);
    if (InsertPacketInList(p->rtp, p->sn)){
      /* if packet wasnt already there */
      audio_dev->Convert_From_G722_To_S16((unsigned short *)p->rtp->payload,
        p->rtp->payload_len/2,
        (short *)uncompressed_payload,
        p->rtp->payload_len*2);
      audio_buf->Store_Packet(p->rtp->RTP_header,
        uncompressed_payload,
        p->rtp->payload_len * 4, 
        timestamp);
      
      /*  free all internal elements of p. */
      packetfree(p);
      p = NULL;
      
      /* Go through pending list. Try and recover a packet using
			 * each FEC. If we are successful, add the data packet to 
			 * the list of received packets, remove the FEC packet from 
			 * the pending list, since we've used it, and then try to
			 * recover some more 
			 */
      
      for(fecp = pending_list; fecp != NULL; fecp = fecp->next) {
        pnew = recover_with_fec(fecp);
        if(pnew) {
          /* The packet is now here, as we've recovered it */
          A_set(pnew->sn,1);
          InsertPacketInList(pnew->rtp, pnew->sn);
          audio_dev->Convert_From_G722_To_S16((unsigned short *)pnew->rtp->payload,
            pnew->rtp->payload_len/2,
            (short *)uncompressed_payload,
            pnew->rtp->payload_len*2);
          audio_buf->Store_Packet(pnew->rtp->RTP_header, 
            uncompressed_payload, 
            pnew->rtp->payload_len*4,
            timestamp);
          
          /* One FEC packet can only be used once to recover,
					 * so remove it from the pending list 
					 */
          remove_fec_from_pending_list(fecp);
          packetfree(p);
          
          p = pnew;
          
          break;
        } 
      } /*for*/
    } else { 
			/* if packet was already there. */
      packetfree(p);
      p = NULL;
    }
    
  } /*p->fec was false */
  if (uncompressed_payload != NULL) {
    free(uncompressed_payload);
  }
}



