/***************************** FILE HEADER *****************************
#               Copyright (c) 1999 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
*/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iptelapi.h"
#include "sipapi.h"
//#include "radio.h"  /* Audio tool API */
#include "command.h"

#ifdef __cplusplus
extern "C" {
#endif
  
#ifndef WIN32 /* For unix */
#include "unistd.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include "netinet/in.h"
#include "arpa/inet.h"
#else
#include "winsock2.h" /* for sockets */
  extern int getopt (int argc, char *const *argv, const char *shortopts);
  extern char *optarg;
#endif
  
  /* RTP headers */
#include "pthread-util.h"
#include "rtp_api.h"
#include "rtp_highlevel.h"
#include "sched.h"
#include "timespec.h"
#include "error.h" /* debug */
  
#ifdef __cplusplus
}
#endif

#include "audioBuffer.h"
#include "audioDevice.h"

#define G722_BUF_SIZE    320/4
#define MAX_RTP_BUF_SIZE 320
#define DEFAULT_THRESHOLD  2500000

double EnergyLevel (unsigned char *, int ); 
void Initialize();
int CopyTo(char *, int);
int CopyFrom(unsigned, char *, int);
int PutPacket(rtp_packet *, long);
int GetPacket(char *, int);
void fec_recovery(rtp_packet* rtp, int fec, unsigned long ts, audioBuffer *audio_buf, audioDevice *a);


void computeFEC(int size, int count, short _rtpPack[3][MAX_RTP_BUF_SIZE/4], short * _fecPack) {
  int ind;
  int c;
  for (ind = 0; ind < size; ind++) {
    _fecPack[ind] = 0;
    for (c=0; c<count; c++) {
      _fecPack[ind] ^= _rtpPack[c][ind];
    }
  }
}

void CalculateFecHeader(unsigned char *fec_buf, unsigned short snbase,
                        int size_fec [3],int E, int pt_fec[3], 
                        int mask, unsigned long rtp_ts_fec[3])
{
  unsigned short lengthrec = (unsigned short) (size_fec [0] ^ size_fec [1] ^ size_fec [2]);
  int pt = pt_fec[0] ^ pt_fec[1] ^ pt_fec[2];
  unsigned long tsrec = rtp_ts_fec[0] ^ rtp_ts_fec[1] ^ rtp_ts_fec[2];
  
  
  fec_buf[0]  = ((snbase >> 8) & 0x0ff);
  fec_buf[1]  = ((snbase) & 0x0ff);
  fec_buf[2]  = ((lengthrec >> 8) & 0x0ff);
  fec_buf[3]  = (lengthrec & 0x0ff);
  fec_buf[4]  = (((E & 0x01) << 7) | (pt & 0x07f));
  fec_buf[5]  = ((mask >> 16) & 0x0ff);
  fec_buf[6]  = ((mask >> 8) & 0x0ff);
  fec_buf[7]  = ((mask) & 0x0ff);
  fec_buf[8]  = ((tsrec >> 24) & 0x0ff);
  fec_buf[9]  = ((tsrec >> 16) & 0x0ff);
  fec_buf[10] = ((tsrec >> 8) & 0x0ff);
  fec_buf[11] = ((tsrec) & 0x0ff);
}




/*
void CalculateFecHeader(unsigned char *fec_buf, unsigned short snbase,
unsigned short lengthrec,
int E, int pt, int mask, unsigned long tsrec)
{
fec_buf[0]  = ((snbase << 8) & 0x0ff);
fec_buf[1]  = ((snbase) & 0x0ff);
fec_buf[2]  = ((lengthrec << 8) & 0x0ff);
fec_buf[3]  = (lengthrec & 0x0ff);
fec_buf[4]  = (((E & 0x01) << 7) | (pt & 0x07f));
fec_buf[5]  = ((mask << 16) & 0x0ff);
fec_buf[6]  = ((mask << 8) & 0x0ff);
fec_buf[7]  = ((mask) & 0x0ff);
fec_buf[8]  = ((tsrec << 24) & 0x0ff);
fec_buf[9]  = ((tsrec << 16) & 0x0ff);
fec_buf[10] = ((tsrec << 8) & 0x0ff);
fec_buf[11] = ((tsrec) & 0x0ff);
}
*/

/**
* MySIPUserAgent and MySIPCall
*/
class MySIPUserAgent : public SIPEndpoint
{
public:
  MySIPUserAgent(bool IsServer = true,bool IsRegistrar=false);
  MySIPUserAgent(const IptelIPAddress & ip,bool IsRegistrar=false);
  virtual IptelCall *AllocateNewCallObject(const char *callid);
};

/**
* Context to store the RTP and audio information
*/
class rtp_info_t {
public:
  rtp_info_t() {
    cid = -1;
    cid_fec = -1;
    oTerminateSend = 0; 
    oTerminateReceive = 0;
    //srand(time(NULL));
    //timestamp = abs(rand() % 100000);
    timestamp = 320;
  }
  ~rtp_info_t() {};
  
  /* RTP context */
  context cid;
  context cid_fec;
  
  /* mutex for synchronization */
  pthread_mutex_t mutex_rtp;
  pthread_cond_t cond_rtp;
  
  /* Send thread */
  int oTerminateSend;
  pthread_t thread_send;
  
  /* Receive thread */
  int oTerminateReceive;
  pthread_t thread_receive;
  
  /* Circular buffer, size, read and write pointers */
  audioDevice audio_dev;
  audioBuffer audio_buf;
  long timestamp;
  
  int o_silence;    /* silence suppression flag */
  double threshold; /* SS thread hold */
};


class MySIPCall : public SIPCall
{
public:
  rtp_info_t *rtp;
  
public:
  MySIPCall(const char *callid=NULL) : SIPCall(callid) { rtp=NULL; };
  MySIPCall(SIPEndpoint & ep, const char *callid = NULL) : SIPCall(ep,callid) { rtp=NULL; };
  virtual ~MySIPCall() { cout << "~MySIPCall()" << endl;
  if (rtp != NULL) delete rtp; /* default destructor */ };
  
  virtual void   OnNewIncomingCall(IptelEndpoint & ep);
  virtual int    OnHangup();
  virtual void   OnCallEstablished();
  virtual int    OnCallRejected();
  virtual void   OnNewMessageBody(const SIPMessageBody & body);
  virtual int    OnRemoteSessionChanged(const SessionDescription & sdp);
};

MySIPCall *call = NULL;

bool      oBusy = false;
bool      oLicensed = false; /* to limit features in unlicensed
version */

int8      oTtl = 255;

bool       measure_latency = false;

GenericHeaders globals;

/*
* L_rtp is the mutex for protecting RTP functions
*/
pthread_mutex_t L_rtp = PTHREAD_MUTEX_INITIALIZER;

/* Prototypes */
void * RtpSendThread(void *);
void * RtpReceiveThread(void *);


//////////////// Method defn for SIP ///////////////////////

MySIPUserAgent::MySIPUserAgent(bool IsServer,bool IsRegistrar)
: SIPEndpoint(IsServer,IsRegistrar)
{
  /* Do nothing here */
}

MySIPUserAgent::MySIPUserAgent(const IptelIPAddress & ip,bool IsRegistrar)
: SIPEndpoint(ip,IsRegistrar)
{
  /* Do nothing here */
}

IptelCall *MySIPUserAgent::AllocateNewCallObject(const char *callid)
{
  /* Allocate a new SIP call */
  return new MySIPCall(*this, callid);
}

void MySIPCall::OnNewIncomingCall(IptelEndpoint & ep)
{
  IptelAddress dest    = GetSelfAddress();
  IptelAddress src     = GetRemoteAddress();
  
  //cout << "On incoming call from " << dest << " for " << src << endl;
  
  if ( oBusy ) {
    Reject(486, "Busy here");
    /* How do we delete the call object here XXX */
    return;
  }
  
#if 0
  if ( !Authenticate("basic", "./passwd") ) {
    cerr << "Can not be authenticated, rejecting" << endl;
    Reject(); /* Error code are already set */
    return;
  }
  else {
    cout << "Call is authenticated" << endl;
  }
#endif
  
  /*
  * Set the global variable for call.
  * Accept or reject is called in UI thread 
  */
  
  oBusy = true;
  
  call = this;
  
  cout << "Incoming call from "  << src << " to " << dest << endl;
  const char *subj = GetSubject();
  if ( subj != NULL ) {
    cout << "Subject is " << subj << endl;
  }
  cout << "accept/reject/noresponse ? " << flush;
}

int   MySIPCall::OnHangup()
{
  cout << "Call closed by remote" << endl;
  oBusy = false;
  
  if (call != NULL && call->rtp != NULL) {
    void *val;
    
    pthread_mutex_lock(&rtp->mutex_rtp);
    rtp->oTerminateReceive = 1;
    rtp->oTerminateSend = 1;
    pthread_cond_broadcast(&rtp->cond_rtp);
    pthread_mutex_unlock(&rtp->mutex_rtp);
    
    /* the thread is terminated */
    pthread_join(rtp->thread_receive, &val);
    pthread_join(rtp->thread_send, &val);
    
    RTPCloseConnection(call->rtp->cid, "Call hungup");
    RTPCloseConnection(call->rtp->cid_fec, "Call hungup");
    RTPDestroy(call->rtp->cid);
    RTPDestroy(call->rtp->cid_fec);
    
    pthread_mutex_destroy(&rtp->mutex_rtp);
    pthread_cond_destroy(&rtp->cond_rtp);
    call->rtp->audio_dev.Close_audioDeviceice();
    delete call->rtp;
    call->rtp = NULL;
  }
  
  call = NULL;
  
  
  SIPCall::OnHangup();
  return 0;
}

void   MySIPCall::OnCallEstablished()
{
  cout << "Call Established" << endl;
  SIPCall::OnCallEstablished();
  rtperror theerr;
  
  if ( IsActive() ) {
    
    cout << "Accepted by remote" << endl;
    
    SessionDescription *session;
    MediaInfo media;
    
    /* Get remote IP and port */
    session = GetRemoteSessionDescription();
    
    cout << "Remote Session " << *session << endl;
    
    MediaInfo rem_media;
    
    session->FindSession(rem_media, MediaInfo::Receive, MediaInfo::Audio);
    
    IptelIPAddress rem_addr = rem_media.GetRxAddress();
    
    if ( rtp != NULL ) { 
      struct sockaddr_storage addr;
      rem_addr.GetIpAddress(addr);
      AFISetPort(&addr, rem_media.GetRxPort());
      struct sockaddr_storage addr_fec;
      rem_addr.GetIpAddress(addr_fec);
      AFISetPort(&addr_fec, rem_media.GetRxPort() + 2);
      
      theerr = 
      		RTPSessionAddSendAddr(rtp->cid, AFIGetDottedAddress((const
          struct sockaddr *) &addr),AFIGetPort(&addr), oTtl);
      
      if (theerr != RTP_OK) {
        cerr << "RTPSessionAddSendAddr(OnCallEstablished) failed" << endl;
      }
      
      theerr = 
      		RTPSessionAddSendAddr(rtp->cid_fec, AFIGetDottedAddress((const
          struct sockaddr *) &addr_fec),AFIGetPort(&addr_fec), oTtl);
      
      if (theerr != RTP_OK) {
        cerr << "RTPSessionAddSendAddr(OnCallEstablished) fec failed" << endl;
      }
      
      
      /* create the threads */
      
      pthread_mutex_init(&rtp->mutex_rtp, NULL);
      pthread_cond_init(&rtp->cond_rtp, NULL);
      if ( pthread_create(&rtp->thread_receive, NULL, 
        RtpReceiveThread, 
        (void *) rtp) != 0 ) {
        cerr << "pthread_create failed for receive thread" << endl;
      }
      
      debug("OnCallEstablished", "%s:%d\n", __FILE__, __LINE__);
      
      if ( pthread_create(&rtp->thread_send, NULL, 
        RtpSendThread, 
        (void *) rtp) != 0 ) {
        cerr << "pthread_create failed for send thread" << endl;
      }
      debug("OnCallEstablished", "%s:%d\b", __FILE__, __LINE__);
      
    }
    else {
      cerr << "No audio tool" << endl;
    }
  }  
  debug("OnCallEstablished", "%s:%d\b", __FILE__, __LINE__);
}

int   MySIPCall::OnCallRejected()
{
  cout << "Call is Rejected by remote" << endl;
  cout << GetStatus() << " " << GetStatusString() << endl;
  
  oBusy = false;
  
  
  int s = GetStatus();
  
  if ( s == 401 ) {
    const char *auth = GetIncomingResponseHeader("WWW-Authenticate");
    if ( auth != NULL ) {
      cout << "WWW-Authenticate: " << auth << endl;
    }
    cout << "Set your password and try again." << endl;
  }
  else if ( ( s >= 300) && ( s < 400)) {
    const char *contact = GetIncomingResponseHeader("Contact");
    if ( contact != NULL ) {
      cout << "Contact: " << contact << endl;
    }
    cout << "Try the new address given above." << endl;
  }
  
  call->rtp->audio_dev.Close_audioDeviceice();
  call = NULL;
  SIPCall::OnCallRejected();
  return 0;
}

void  MySIPCall::OnNewMessageBody(const SIPMessageBody &body)
{
  cout << "----- New Message Body -----\n"
    << body.GetContentType() 
    << "[" << body.GetContentDisposition() << "]"
    << "[" << body.GetContentHandling() << "]" << "\n"
    << body.GetContent() << "\n"
    << "----------------------------" << endl;
}

int   MySIPCall::OnRemoteSessionChanged(const SessionDescription & sdp)
{
/*
* We always accept change in SDP
  */
  cout << "Remote SDP is changed, trying to accept" << endl;
  return 0;
}

/* Main routine */

int main(int argc, char *argv[])
{
  int c;
  char cmd[200];
  unsigned short listening_port = 5060;
  char *command_file = NULL;
  bool interactive = true;
  unsigned short mediaport = 0;
  rtperror theerr;
  
  static char *mycommands = 
    " General format for any command is\n"
    " <commandname>  <list-of-options> <list-of-arguments>\n"
    " Different commands are invite, accept, reject, authenticate\n"
    " redirect, register, unregister, help, exit, quit, log, set,\n"
    " unset, and ?\n"
    " Options are of the form param=value. Every command defines\n"
    " a set of supported options. Additional options are ignored\n"
    " Typical options are\n"
    "  transport with values udp, tcp or multicast\n"
    "  expires with the expiry timeout in seconds\n"
    "  user with values ip or phone\n"
    "  sdp with the sdp identifier\n"
    "  body with the body identifier\n"
    "  cgi or cpl with the corresponding file name\n"
    " \n"
    " For example, \n"
    " you can place an outgoing call as\n"
    " invite transport=tcp sip:kns10@cs sip:ind.cs.columbia.edu\n"
    "\n"
    "Visit http://www.cs.columbia.edu/~kns10/software/siprtpua for\n"
    "a complete list of commands and user manual\n"
    "\n"
    ;
  
  // Others to be implemented
  // ,verbose,save filename, dump sip filename, dump h323 filename
  
  const char *servername = "SIPRTPUA-Columbia-University";
  
  cout << "SIPRTPUA, (c) 2000, Columbia University" << endl;
  cout << "Visit http://www.cs.columbia.edu/~kns10/software/siprtpua for more information." << endl;
  
  const char *usage = "[options]\n"
    " -v          : Print version and exit\n"
    " -d          : verbose mode\n"
    " -o tracefile: put debug trace in this file.\n"
    " -p port     : listening SIP port, default is 5060.\n"
    " -f file     : command file to be executed.\n"
    " -i          : non interactive mode, forbackground.\n"
    " -d          : use dotted decimal for self address.\n"
    " -t          : print timeing (latency) information.\n"
    " -P port     : media self port number.Default is 10000\n"
    ;
  
  /* Parse command line options. */
  while ((c = getopt(argc, argv, "hvdo:a:p:f:nP:it")) != EOF) {
    switch (c) {
    case 'v' :
#ifdef VERSION
      printf("siprtpua version %s\n", VERSION);
#else
      printf("siprtpua version unknown\n");
#endif 
      exit(0);
      break;
    case 'd':
      SIPLibrary::DebugOn();
      break;
    case 'o':
      SIPLibrary::DebugOn(optarg);
      break;
    case 'p':
      listening_port = atoi(optarg);
      break;
    case 'f' :
      command_file = strdup(optarg);
      break;
    case 'n' :
      interactive = false;
      break;
    case 'P' :
      mediaport = atoi(optarg);
      break;
    case 'i' :
      SIPLibrary::oUseNumericAddress = true;
      break;
    case 't' :
      measure_latency = true;
      break;
    case '?':
      /* no break */
    case 'h':
      cerr << "Usage: " << argv[0] << " "  << usage << endl;
      exit(1);
    } 
  }
  
  /* Create the listening servers */
  IptelIPAddress listeningAddress;
  listeningAddress.SetPort(listening_port);
  
  MySIPUserAgent sip(listeningAddress,
    false);
  IptelAddress localname = SIPLibrary::HostName4;
  localname.SetUser("siprtpua");
  sip.SetLocalName(localname);
  
  int e =
    SIPLibrary::CheckLicense("file://siprtpua.license", NULL);
  
  if ( e == 0 ) {
    oLicensed = true;
  }
  else {
    cerr << "Unlicensed copy. Contact hgs@cs.columbia.edu for a license." << endl;
  }
  
  /* Initialize debug options */
  SIPLibrary::SetName(servername);
  SIPLibrary::ErrorOn("stderr");
  
  cout << "SIP " << sip << endl;
  
  pthread_t rtp_schedule_thread; 
  SIPLibrary::DebugOn("stdout");
  
  if ( pthread_create_detached(&rtp_schedule_thread, NULL,
    RTPScheduleThread,
    NULL) != 0 ) {
    cout << "RTPMAddCallLeg" << 
      "pthread_create_detached for RTPMScheduleThread failed" << endl;
    return -1;
  }
  
  /* Allocate the audio tool */
  
  cout << "Waiting for connections..." << endl;
  
  cout << "Type help to get list of commands" << endl;
  
  bool oExitFlag = false;
  FILE *command_fp = NULL;
  while ( !oExitFlag ) {
    
    if ( command_file ) {
      if ( command_fp == NULL ) {
        command_fp = fopen(command_file, "r");
        if ( command_fp == NULL ) {
          free(command_file);
          command_file = NULL;
          continue;
        }
      }
      
      if ( fgets(cmd, sizeof(cmd), command_fp) == NULL ) {
        free(command_file);
        command_file = NULL;
        continue;
      }
      if ( strlen(cmd) >= 1 ) {
        cmd[strlen(cmd)-1] = 0;
      }
    }
    else {
      if ( interactive ) {
        cout << "siprtpua> " << flush;     /* prompt for command */
        strcpy(cmd,"");
        fscanf(stdin,"%[^\n]",cmd); /* read command */
        getchar();                      /* remove \n from stdin */
      }
      else {
        /* Do nothing in non-interactive mode. wait for ctrl-C */
        while ( 1 ) {
#ifdef WIN32
          Sleep(10000);
#else
          sleep(10);
#endif
        }
      }
    }
    
    CommandObject command(cmd, globals); /* Create the command object */
    
    if ( command == NULL )
      continue;
    
    if ( command == "exit" || command == "quit" ) {
      oExitFlag = true;
    }
    else if ( command == "help" ) {
      cout << "Commands supported are" << endl << mycommands << flush;
    }
    else if ( command ==  "?") {
      if ( command.GetArgsCount() <= 0 ) {
        fprintf(stderr,"%s\n", mycommands);
        continue;
      }
      if ( command.IsParameter(1,"call") ) {
        if ( call != NULL ) {
          cout << *call << endl;
        }
        else {
          cout << "No call present" << endl;
        }
      }
      else if ( command.IsParameter(1,"ep") ) {
        cout << sip << endl;
      }
      else if ( command.IsParameter(1,"calls")) {
        cout << "Numbers of calls: " << SIPLibrary::CallCount << endl;
      }
      else if ( command.IsParameter(1,"eps")) {
        cout << "Number of endpoints: " << SIPLibrary::EndpointCount
          << endl;
      }
      else {
        fprintf(stderr,"%s\n", mycommands);
      }
    }
    /* ---- LOGging mechanism --- */
    else if ( command == "log" ) {
      if ( command.GetArgsCount() <= 0 ) {
        fprintf(stderr,"%s\n", mycommands);
        continue;
      }
      if ( command.IsParameter(1,"msgflow") ) {
        if ( ((const char *) command[2]) == "off" ) {
          SIPLibrary::MessageFlowOn(NULL);
        }
        else {
          if ( command.GetArgsCount() >= 2 ) {
            if ( command.IsParameter(2, "stdout") ) {
              SIPLibrary::MessageFlowOn(stdout);
            }
            else {
              SIPLibrary::MessageFlowOn(stderr);
            }
          }
          else {
            SIPLibrary::MessageFlowOn(stderr);
          }
        }
      }
      else if ( command.IsParameter(1,"debug") ) {
        if ( command[2] == "off" ) {
          SIPLibrary::DebugOff();
        }
        else {
          if ( command.GetArgsCount() >= 2 ) {
            SIPLibrary::DebugOn((const char *) command[2]);
          }
          else {
            SIPLibrary::DebugOn();
          }
        }
      }
      else if ( command.IsParameter(1, "error") ) {
        fprintf(stderr,"errors log can not be modified\n");
      }
      else {
        fprintf(stderr,"%s\n", mycommands);
      }
      
    }
    /* ---- SIP Invite ---- */
    else if ( command == "invite") {
      if ( command.GetArgsCount() <= 0) {
        fprintf(stderr,"usage: invite [params...] sipurl_of_remote [request_uri]\n");
        continue;
      }
      
      if ( oBusy ) {
        cerr << "Already busy in a call" << endl;
        continue;
      }
      
      oBusy = true;
      
      /* Formate self URL */
      char *logname = getenv("LOGNAME");
      logname = (logname == NULL) ? getenv("USERNAME") : logname ;
      IptelAddress src = SIPLibrary::HostName4;
      src.SetScheme("sip");
      src.SetUser(logname == NULL ? "unknown" : logname);
      
      if (((const char *) command["from"]) != NULL ) {
        /* Set the from address to this */
        src = (const char *) command["from"];
        src.SetScheme("sip");
        cout << "Local address is " << src << endl;
      }
      
      /* Get the destination URL */
      IptelAddress dest = (const char *) command[1];
      dest.SetScheme("sip");
      
      MySIPCall *mycall;
      mycall = new MySIPCall(sip);
      call = mycall;
      
      if ( command.GetArgsCount() >= 2 ) {
        /* Read the request URI */
        IptelAddress requesturi = (const char *) command[2];
        requesturi.SetScheme("sip");
        mycall->SetRequestURI(requesturi);
      }
      
      mycall->rtp = new rtp_info_t;
      
      theerr = RTPCreateWithInitialValues(&mycall->rtp->cid, 1, 320);
      if (theerr != RTP_OK) {
        cerr << "RTPCreate failed" << endl;
      }
      
      theerr = RTPCreateWithInitialValues(&mycall->rtp->cid_fec, 1, 320);
      if (theerr != RTP_OK) {
        cerr << "RTPCreate failed fec" << endl;
      }
      
      theerr = RTPSessionSetReceiveAddr(mycall->rtp->cid,
        NULL, 0);
      if (theerr != RTP_OK) {
        cerr << "RTPSessionSetReceiveAddr(NULL,0) failed" << endl;
      }
      
      theerr = RTPOpenConnection(mycall->rtp->cid);
      if (theerr != RTP_OK) {
        cerr << "RTPOpenConnection failed" << endl;
      }
      
      char receive_addr[1024];
      unsigned short port;
      unsigned short port_fec;
      
      theerr = RTPSessionGetReceiveAddr(mycall->rtp->cid,
        receive_addr, 
        &port);
      if (theerr != RTP_OK) {
        cerr << "RTPSessionGetReceiveAddr failed" << endl;
      }
      
      port_fec = port + 2;
      
      theerr = RTPSessionSetReceiveAddr(mycall->rtp->cid_fec,
        NULL, port_fec);
      if (theerr != RTP_OK) {
        cerr << "RTPSessionSetReceiveAddr(NULL,0) failed fec" << endl;
      }
      
      theerr = RTPOpenConnection(mycall->rtp->cid_fec);
      if (theerr != RTP_OK) {
        cerr << "RTPOpenConnection failed" << endl;
      }
      
      SessionDescription *session;
      MediaInfo media;
      
      /* Allocate self session info */
      session = new SessionDescription;
      
      media.SetDirection(MediaInfo::Receive);
      media.SetDataType(MediaInfo::Audio);
      
      //media.SetRxAddress(receive_addr);
      media.SetRxAddress(IptelIPAddress(SIPLibrary::HostInAddr4));
      media.SetRxPort(port);
      
      FormatInfo g722;
      g722.SetPayloadType(96);
      g722.SetFormatName("g722");
      g722.SetClockRate(16000);
      media.AddFormat(g722); /* G722 */
      
      FormatInfo fec;
      fec.SetPayloadType(97);
      fec.SetFormatName("parityfec");
      fec.SetClockRate(16000);
      media.AddFormat(fec); /* FEC */
      
      session->AddSession(media);
      cout << "Self Session " << *session << endl;
      mycall->SetSelfSessionDescription(session);
      
      if ( ((const char *) globals["Password"]) != NULL ) {
        char *u;
        if (((const char *) globals["UserId"]) != NULL ) {
          u = strdup( ((const char *) globals["UserId"]));
        }
        else {
          u = src.GetUser();
        }
        
        mycall->SetAuthorization(u, (const char *) globals["Password"]); 
        /* Set basic authenticate */
        if ( u ) {
          free(u);
        }
      }
      if ( ((const char *) globals["Subject"]) != NULL ) {
        mycall->SetSubject(globals["Subject"]);
      }
      
      cout << "Calling " << dest << " from " << src  << endl;
      
      /* Set the outbound proxy address */
      if ( ((const char *) globals["proxy"]) != NULL ) {
        mycall->SetProxyAddress((const char *) globals["proxy"]);
      }
      
      if ( ((const char *) command["transport"]) != NULL ) {
        const char *transport = (const char *) command["transport"];
        if ( !strcmp(transport, "tcp") ) {
          mycall->SetTransportType(SOCK_STREAM);
        }
        else {
          mycall->SetTransportType(SOCK_DGRAM);
        }
      }
      
      debug("OnCallEstablished", "%s:%d\n", __FILE__, __LINE__);
      
      if (mycall->rtp->audio_dev.Open_audioDeviceice()== -1) {
        //exit (0) ;
      }
      
      debug("OnCallEstablished", "%s:%d\n", __FILE__, __LINE__);
      
      mycall->Initiate(dest,src);
      
    }
    else if ( command == "reinvite" ) {
      if ( call ) {
        call->Reinvite();
      }
      else {
        cout << "Not in a call" << endl;
        continue;
      }
    }
    else if ( command == "bye") {
      
      if ( !oBusy  ) {
        cerr << "Not in a call" << endl;
        continue;
      }
      
      if ( call != NULL ) {
        if (call->rtp != NULL) {
          
          void *val;
          
          pthread_mutex_lock(&call->rtp->mutex_rtp);
          call->rtp->oTerminateReceive = 1;
          call->rtp->oTerminateSend = 1;
          pthread_cond_broadcast(&call->rtp->cond_rtp);
          pthread_mutex_unlock(&call->rtp->mutex_rtp);
          
          /* the thread is terminated */
          pthread_join(call->rtp->thread_receive, &val);
          pthread_join(call->rtp->thread_send, &val);
          
          RTPCloseConnection(call->rtp->cid, "Call hungup");
          RTPDestroy(call->rtp->cid);
          
          RTPCloseConnection(call->rtp->cid_fec, "Call hungup");
          RTPDestroy(call->rtp->cid_fec);
          
          pthread_mutex_destroy(&call->rtp->mutex_rtp);	  
          pthread_cond_destroy(&call->rtp->cond_rtp);
          call->rtp->audio_dev.Close_audioDeviceice();
          delete call->rtp;
          call->rtp = NULL;
        }
        call->Hangup();
        delete call;
        call = NULL;
        oBusy = false;
      }
      else {
        cerr << "ERROR: call pointer is NULL" << endl;
      }
    }
    else if ( command == "set" || command == "unset") {
      if ( command.GetArgsCount() <= 0 ) {
        fprintf(stderr,"mycommands\n");
        continue;
      }
      if ( command == "set" ) {
        if ( command.GetArgsCount() < 2 ) {
          const char *value = globals[(const char *) command[1]];
          if (value) {
            printf("%s is %s\n", (const char *) command[1], value);
          }
          else {
            printf("%s is not set\n", (const char *) command[1]);
          }
        }
        else {
          globals -= (const char *) command[1];
          globals += GenericHeaderValue((const char *) command[1], command.TillEnd(2));
        }
      }
      else {
        globals -= (const char *) command[1];
      }
    }
    else if (command == "get-ogrqh" || command == "get-ogrsh" 
      ||  command == "get-inrqh" || command == "get-inrsh" ) {
      if ( command.GetArgsCount() <= 0 ) {
        fprintf(stderr, "get-??? header-name\n");
        continue;
      }
      
      if ( call == NULL ) {
        fprintf(stderr, "No active call\n");
      }
      else {
        const char * v = NULL;
        if ( command == "get-ogrqh" ) {
          v = call->GetOutgoingRequestHeader((const char *)command[1]);
        }
        else if ( command == "get-ogrsh" ) {
          v = call->GetOutgoingResponseHeader((const char *)command[1]);
        }
        else if ( command == "get-inrqh" ) {
          v = call->GetIncomingRequestHeader((const char *)command[1]);
        }
        else if ( command == "get-inrsh" ) {
          v = call->GetIncomingResponseHeader((const char *)command[1]);
        }
        printf("%s: %s\n", (const char *) command[1], ((v != NULL)? v : "<absent>"));
      }
    }
    else if ( command == "register" || command == "unregister"
      || command == "register-get" ) {
      if ( command.GetArgsCount() <= 0) {
        fprintf(stderr, 
          "usage: register  [transport=tcp|udp] [proxy=wqr] [content=filename] "
          "[content-type=mimetype] [from=local] [contact=abc@xyz] [expires=n] "
          "url [registrar]\n"
          "       unregister [transport=tcp|udp] [proxy=wqr] [from=local] [contact=*|abc@xyz] "
          "url [registrar]\n"
          "       register-get [transport=tcp|udp] [proxy=wqr] [from=local] url [registrar]\n");
        continue;
      }
      IptelAddress sipurl = (const char *) command[1];
      
      /* Register with SIP server */
      SIPRegistrationInfo reginfo;
      reginfo.localName = sipurl;
      IptelAddress registrar = sipurl;
      
      /* Override the registrar address */
      if ( command.GetArgsCount() >= 2) {
        registrar = (const char *) command[2];
      }
      
      registrar.SetUser(NULL); /* No user name is registrar address */
      registrar.SetScheme("sip");
      reginfo.registrar = registrar;
      
      if ( !(command == "register-get") ) {
        if ( ((const char *) command["contact"]) == NULL ) {
          reginfo.contactAddress = SIPLibrary::HostName4;
          reginfo.contactAddress.SetScheme("sip");
          char *logname = getenv("LOGNAME"); /* Try LOGNAME and USERNAME
          env variables */
          logname = (logname == NULL) ? getenv("USERNAME") : logname ;
          if ( logname != NULL) {
            reginfo.contactAddress.SetUser(logname);
          }
          
          IptelIPAddress listening_addr;
          listening_addr = sip.GetServerAddress();
          int listening_port = listening_addr.GetPort();
          if ( listening_port != 0 && listening_port != 5060) {
            reginfo.contactAddress.SetPort(listening_port);
          }
        }
        else {
          reginfo.contactAddress = (const char *) command["contact"];
        }
      }
      
      if (((const char *) command["from"]) != NULL ) {
        /* Set the from address to this */
        const char *from = (const char *) command["from"];
        if ( !strcmp(from, "local" ) ) {
          reginfo.fromAddress = SIPLibrary::HostName4;
          reginfo.fromAddress.SetScheme("sip");
          char *logname = getenv("LOGNAME"); /* Try LOGNAME and USERNAME
          env variables */
          logname = (logname == NULL) ? getenv("USERNAME") : logname ;
          if ( logname != NULL) {
            reginfo.fromAddress.SetUser(logname);
          }
        }
        else {
          reginfo.fromAddress = from;
          reginfo.fromAddress.SetScheme("sip");
        }
      }
      
      /* Set the outbound proxy */
      if ( globals["proxy"] != NULL ) {
        reginfo.outbound_proxy = (const char *) globals["proxy"];
      }
      if ( ((const char *) command["proxy"]) != NULL ) {
        /* Set the proxy to this address */
        reginfo.outbound_proxy = (const char *) command["proxy"];
      }
      
      /* Set the expires header */
      if ( ((const char *) command["expires"]) != NULL) {
        reginfo.expiry = (long)(atoi((const char *) command["expires"]));
      }
      else if ( command == "register" ) {
        reginfo.expiry = (long) 3600;
      }
      
      /* Check if there is any body */
      const char *content = NULL;
      const char *content_type = NULL;
      if ( ((const char *) command["content"]) != NULL ) {
        content = (const char *) command["content"];
        content_type = "text/x-perl";
      }
      if ( ((const char *) command["content-type"]) != NULL ) {
        content_type = (const char *) command["content-type"];
      }
      
      if ( ((const char *) globals["Password"]) != NULL ) {
        char *u;
        if (((const char *) globals["UserId"]) != NULL ) {
          u = strdup( ((const char *) globals["UserId"]));
        }
        else {
          u = reginfo.fromAddress.GetUser();
          
          if ( !u ) {
            u = reginfo.localName.GetUser();
          }
        }
        
        if ( u ) {
          sip.SetAuthorization(u, (const char *) globals["Password"]); 
        }
        
        if ( u ) {
          free(u);
        }
      }
      
      if ( ((const char *) command["transport"]) != NULL ) {
        const char *transport = (const char *) command["transport"];
        if ( !strcmp(transport, "tcp") ) {
          reginfo.type = SOCK_STREAM;
        }
        else {
          reginfo.type = SOCK_DGRAM;
        }
      }
      
      cout << "Registering to server as " << reginfo.localName << " at " <<
        reginfo.registrar << " with contact " <<
        reginfo.contactAddress << endl;
      if ( command == "register" ||
        command == "register-get") {
        if ( content == NULL ) {
          sip.Register(reginfo);
        }
        else {
          sip.Register(reginfo, content, content_type);
        }
      }
      else {
        sip.Unregister(reginfo);
      }
    }
    else if ( command == "options") {
      if ( command.GetArgsCount() <= 0 ) {
        fprintf(stderr, "options url [from]\n");
        continue;
      }
      IptelAddress sipurl = (const char *) command[1];
      IptelAddress from;
      
      if ( command.GetArgsCount() >= 2) {
        from = (const char *) command[2];
      }
      else {
        from = SIPLibrary::HostName4;
        from.SetScheme("sip");
        char *logname = getenv("LOGNAME"); /* Try LOGNAME and USERNAME
        env variables */
        logname = (logname == NULL) ? getenv("USERNAME") : logname ;
        if ( logname != NULL) {
          from.SetUser(logname);
        }
      }
      
      sip.GetRemoteSessionDescription(sipurl, from);
    }
    else if ( command == "authenticate" ) {
      if ( !oBusy ) {
        cerr << "No pending incoming call" << endl;
        continue;
      }
      
      MySIPCall *mycall;
      mycall = call;
      if ( mycall == NULL ) {
        cerr << "ERROR: Call pointer is NULL" << endl;
        oBusy = false;
        continue;
      }
      
      const char *method = "basic";
      const char *file = "./passwd";
      if (((const char *) command["method"]) != NULL ) {
        method = (const char *) command["method"];
      }
      if (((const char *) command["file"]) != NULL ) {
        file = (const char *) command["file"];
      }
      
      if ( !mycall->Authenticate(method, file) ) {
        cerr << "Can not be authenticated, rejecting" << endl;
        mycall->Reject(); /* error code is already set */
        delete mycall;
        call = NULL; /* XXX free ?? */
        oBusy = false;
      }
      else {
        cout << "Call is authenticated\n you can accept the call (accept/reject)?" << endl;
      }
    }
    else if (    command == "accept" 
      || command == "reject"
      || command == "noresponse" 
      || command == "redirect") {
      
      if ( !oBusy ) {
        cerr << "No pending incoming call" << endl;
        continue;
      }
      if ( command == "noresponse" ) {
        /* pretend as if not received any incoming call */
        oBusy = false;
        call = NULL;
        continue;
      }
      
      MySIPCall *mycall;
      mycall = call;
      if ( mycall == NULL ) {
        cerr << "ERROR: Call pointer is NULL" << endl;
        oBusy = false;
        continue;
      }
      
      if ( ((const char *) command["content"]) != NULL ) {
        SIPMessageBody body;
        FILE *fp;
        char text[1500]; /* note this limitation XXX */
        strcpy(text, "");
        
        if ((fp = fopen((const char *) command["content"], "rb")) != NULL) {
          int length = fread(text, 1, sizeof(text) - 1, fp);
          text[length] = 0;
          if ( length > 0 ) {
            body.SetContent(text);
            body.SetContentLength(length);
          }
          else {
            body.SetContent((const char *) command["content"]);
          }
        }
        else {
        /* File is empty use whatever is given on the command line.
          */
          body.SetContent((const char *) command["content"]);
        }
        
        if ( ((const char *) command["content-type"]) != NULL ) {
          body.SetContentType((const char *) command["content-type"]);
        }
        if ( ((const char *) command["content-disposition"]) != NULL ) {
          body.SetContentDisposition((const char *) command["content-disposition"]);
        }
        if ( ((const char *) command["content-handling"]) != NULL ) {
          body.SetContentHandling((const char *) command["content-handling"]);
        }
        mycall->SetMessageBody(body);
      }
      
      if ( command == "reject" ) {
        int status = 0;
        const char *status_str = (const char *) command["status"];
        const char *reason_str = NULL;
        
        if ( status_str && *status_str) {
          status = atoi(status_str);
        }
        if ( command.GetArgsCount() >= 1 ) {
          reason_str = command.TillEnd(1);
        }
        
        cout << "Call rejecting" << endl;
        if ( status < 400 || status >= 700) {
          /* Do not send back incorrect status */
          if ( reason_str && *reason_str ) {
            status = 603;
          }
          else {
            status = 0;
          }
        }
        if ( status == 0 ) {
          mycall->Reject();
        }
        else {
          mycall->Reject(status, (const char *)(reason_str &&
            *reason_str 
            ? reason_str
            : NULL) );
        }
        
        delete mycall;
        call = NULL; /* XXX free ?? */
        oBusy = false;
        continue;
      }
      else if ( command ==  "redirect") {          
        if ( command.GetArgsCount() < 1) {
          fprintf(stderr,"usage: redirect uri\n");
        }
        else {
          cout << "Call redirecting to " << ((const char *) command[1]) << endl;
          const char *reason_str = NULL;
          
          if ( command.GetArgsCount() >= 2 ) {
            reason_str = command.TillEnd(2);
          }
          
          mycall->Redirect((const char *) command[1], 
            reason_str && *reason_str ? reason_str : "Redirected by user");
          
          call = NULL;
          oBusy = false;
        }
        continue;
      }
      
      
      mycall->rtp = new rtp_info_t;
      
      theerr = RTPCreateWithInitialValues(&mycall->rtp->cid, 1, 320);
      if (theerr != RTP_OK) {
        cerr << "RTPCreate failed" << endl;
      }
      
      theerr = RTPCreateWithInitialValues(&mycall->rtp->cid_fec, 1, 320);
      if (theerr != RTP_OK) {
        cerr << "RTPCreate failed" << endl;
      }
      
      theerr = RTPSessionSetReceiveAddr(mycall->rtp->cid,
        NULL, 0);
      if (theerr != RTP_OK) {
        cerr << "RTPSessionSetReceiveAddr(NULL,0) failed" << endl;
      }
      
      theerr = RTPSessionSetReceiveAddr(mycall->rtp->cid_fec,
        NULL, 0);
      if (theerr != RTP_OK) {
        cerr << "RTPSessionSetReceiveAddr(NULL,0) failed" << endl;
      }
      
      IptelAddress dest    = mycall->GetSelfAddress();
      IptelAddress src     = mycall->GetRemoteAddress();
      
      /* --- get remote session --- */
      SessionDescription *session;
      session = mycall->GetRemoteSessionDescription();
      if ( session != NULL ) {
        cout << "Remote Session is " << *session << endl;
        MediaInfo rem_media;  
        if ( session->FindSession(rem_media, 
          MediaInfo::Receive, MediaInfo::Audio) ) {
          
          IptelIPAddress rem_addr;
          rem_addr = rem_media.GetRxAddress();
          struct sockaddr_storage addr;
          rem_addr.GetIpAddress(addr);
          AFISetPort(&addr, rem_media.GetRxPort());
          theerr = 
            RTPSessionAddSendAddr(mycall->rtp->cid, AFIGetDottedAddress((const
            struct sockaddr *) &addr),AFIGetPort(&addr), oTtl);
          
          if (theerr != RTP_OK) {
            cerr << "RTPSessionAddSendAddr failed" << endl;
          }
          
          theerr = 
            RTPSessionAddSendAddr(mycall->rtp->cid_fec, AFIGetDottedAddress((const
            struct sockaddr *) &addr),AFIGetPort(&addr) + 2, oTtl);
          
          if (theerr != RTP_OK) {
            cerr << "RTPSessionAddSendAddr failed" << endl;
          }
        }
      }
      else {
        cout << "No remote session in INVITE" << endl;
      }
      
      
      theerr = RTPOpenConnection(mycall->rtp->cid);
      if (theerr != RTP_OK) {
        cerr << "RTPOpenConnection failed" << endl;
      }
      
      theerr = RTPOpenConnection(mycall->rtp->cid_fec);
      if (theerr != RTP_OK) {
        cerr << "RTPOpenConnection failed fec" << endl;
      }
      
      /* --- Allocate self session info --- */
      session = new SessionDescription;
      
      MediaInfo media;
      media.SetDirection(MediaInfo::Receive);
      media.SetDataType(MediaInfo::Audio);
      
      char receive_addr[1024];
      unsigned short port;
      
      theerr = RTPSessionGetReceiveAddr(mycall->rtp->cid,
        receive_addr, 
        &port);
      if (theerr != RTP_OK) {
        cerr << "RTPSessionGetReceiveAddr failed" << endl;
      }
      
      /* self IP address */
      //media.SetRxAddress(IptelIPAddress(receive_addr));
      media.SetRxAddress(IptelIPAddress(SIPLibrary::HostInAddr4));
      media.SetRxPort(port);
      
      FormatInfo g722;
      g722.SetPayloadType(96);
      g722.SetFormatName("g722");
      g722.SetClockRate(16000);
      media.AddFormat(g722); /* G722  */
      
      FormatInfo fec;
      fec.SetPayloadType(97);
      fec.SetFormatName("parityfec");
      fec.SetClockRate(16000);
      media.AddFormat(fec); /* PCMU */
      
      session->AddSession(media);
      
      cout << "Self Session is " << *session << endl;
      
      mycall->SetSelfSessionDescription(session);
      
      cout << "Accepting the call..." << endl;
      mycall->Accept();
      
      
      /* create the threads */
      //mycall->rtp->audio_dev.Open_audioDeviceice();
      if (mycall->rtp->audio_dev.Open_audioDeviceice()== -1) {
        //exit (0) ;
      }
      pthread_mutex_init(&mycall->rtp->mutex_rtp, NULL);
      pthread_cond_init(&mycall->rtp->cond_rtp, NULL);
      
      if ( pthread_create(&mycall->rtp->thread_receive, NULL, 
        RtpReceiveThread, 
        (void *) mycall->rtp) != 0 ) {
        cerr << "pthread_create failed for receive thread" << endl;
      }
      if ( pthread_create(&mycall->rtp->thread_send, NULL, 
        RtpSendThread, 
        (void *) mycall->rtp) != 0 ) {
        cerr << "pthread_create failed for send thread" << endl;
      }
      
    }
    
    else if ( command ==  "sleep") {
      if ( command.GetArgsCount() < 1) {
        fprintf(stderr,"usage: sleep n\n");
      }
      else {
        cout << "Waiting for " << ((const char *) command[1]) <<" seconds"<< endl;
        
        int sec = atoi((const char *) command[1]);
        if ( sec <= 0 ) {
          sec = 0;
        }
#ifdef WIN32
        Sleep(sec * 1000);
#else
        sleep(sec);
#endif
        cout << "Wait existing" << endl;
        
      }
      
    }
    
    else {
      fprintf(stderr,"Unknown command: %s\n", cmd);
    }
  }
  
  cout << "Exiting" << endl;
  
  return 0; /* Not required */
}


void * RtpSendThread(void *p)
{
  rtperror the_err;
  rtp_info_t *rtp;
  struct timespec next;
  struct timeval initialtime;
  long default_duration;
  double total_duration = 0.0, duration_interval;
  unsigned short rtp_seq_no = 1;
  unsigned long  rtp_ts = 320;
  int tsinc = 0;
  int tsincfec = 0;
  int marker=0;
  int pt;
  int size;
  int pack_so_far = 0;
  int fec_send =0;
  
  AUDIO_SAMPLE_BUF *buf;
  AUDIO_SAMPLE_BUF fec_buf[MAX_RTP_BUF_SIZE / 4 + 6]; /* 12 bytes = 6 short of RTP header */
  AUDIO_COMPRESSION_BUF g722_buf[MAX_RTP_BUF_SIZE / 4];
  
  /* arrays for fec computation */
  AUDIO_SAMPLE_BUF _rtpPack[3][MAX_RTP_BUF_SIZE / 4];
  int size_fec [3];
  int pt_fec [3];
  unsigned long  rtp_ts_fec [3];
  
  int o_silence     = 0;
  int talk_spurt    = 1;
  double o_energy   = 0;
  
  if ( p == NULL ) {
    debug("RTPSendThread", "parameter is NULL\n");
    return NULL;
  }
  rtp = (rtp_info_t *) p;
  
  rtp->o_silence = 1;
  rtp->threshold = DEFAULT_THRESHOLD;
  
  pthread_mutex_lock(&rtp->mutex_rtp);
  debug("RTPSendThread", "got the lock\n");
  
  /* Current time */
  gettimeofday(&initialtime, NULL);
  total_duration = 0.0;
  default_duration = 20;
  duration_interval = (double) default_duration/1000.0;
  debug("RTPSendThread", "Duration Interval is %g\n",
    duration_interval);
  
  /* While thread is not terminated */
  
  while ( !rtp->oTerminateSend ) {
  /* Set the threshold  
	 * an attempt to alter the threshold dynamically
   if ( ((const char *) globals["threshold"]) != NULL ) {
   rtp->threshold = atoi((const char *) globals["threshold"]);
			if (rtp->threshold < 0 )
      rtp->threshold = DEFAULT_THRESHOLD; 
		} */
    /* Wait for default_duration */
    next = timespec_delay(initialtime, total_duration);
    total_duration += duration_interval;
    
    /* XXX check if the duration is negative or in past */
    pthread_cond_timedwait(&rtp->cond_rtp, &rtp->mutex_rtp, &next);
    if ( rtp->oTerminateSend ) {
      break; /* from while loop */
    }
    
    /* 
    * Take size bytes from audio device in buf. 
    */
    
    rtp->audio_dev.Record_From_Mic();
    size = rtp->audio_dev.Get_Number_Of_Samples();
    tsinc += size ;    
    tsincfec += size;
    buf = rtp->audio_dev.Get_Buf();
    if ( buf == NULL ) {
      continue;
    }
    pt = 96; /* S16 */
    /* set Marker depending on Energy audio_dev */
    o_energy = rtp->audio_dev.Get_Energy(buf, size);
    o_silence = 0;
    if ( rtp->o_silence ) {      
      if ( o_energy <  rtp->threshold ) {
        o_silence = 1; /* Do not transmit  */
        talk_spurt = 1;
        fec_send =0;
      }
    }
    
    /* call the encoder for G.722 here and
    	* call the FEC calculator on the encoded data 
    	*/
    
    
    rtp->timestamp += size;
    if ( !o_silence ) {
      marker = (talk_spurt ? 1 : 0 );
      /* Send a packet to remote  */
      rtp->audio_dev.Convert_From_S16_To_G722(buf, size, g722_buf, size/4);
      the_err = RTPSend(rtp->cid, 
        (int32) tsinc,
        (int8) marker,
        (int16) pt, 
        (int8 *) g722_buf,
        (int) size / 2);
      debug("RTPSendThread", "Sending Audio marker = %d\n", marker);
      debug("RTPSendThread", "Energy = %f\n", o_energy);
      
      rtp_ts     += tsinc;
      rtp_seq_no += 1;
      
      marker         = 0;        
      talk_spurt     = 0;   /* no longer beguinning of talk spurt */
      tsinc          = 0;
      
      if (pack_so_far < 3) {
				/* check if it is time to send the FEC packet */
        printf("pack_so_far is %d; size = %d\n", pack_so_far, size);
        for(int k=0; k<size/4; k++) {
          _rtpPack[pack_so_far][k] = g722_buf[k];
        }
        /* do the same for ts */
        size_fec[pack_so_far]= size / 2;
        pt_fec[pack_so_far] = pt;
        rtp_ts_fec[pack_so_far] = rtp_ts;
        
        pack_so_far++;
      }
      else {     
        CalculateFecHeader((unsigned char *)fec_buf, rtp_seq_no - 3, 
          size_fec,0, pt_fec, 0x007,rtp_ts_fec);
        computeFEC(size/4, 3, _rtpPack, &fec_buf[6] /* skip FEC header */);
        
        pack_so_far = 0;
        fec_send =1;
      }
    }
    if (fec_send) {
      the_err = RTPSend(rtp->cid_fec, 
        (int32) tsincfec,
        (int8) 0,
        (int16) 97, 
        (int8 *) fec_buf,
        (int) size/2 + 12);
      tsincfec = 0;
      fec_send =0;
      
      debug("RTPSendThread", "Sending FEC \n");
    }
    /*
    * Receive size bytes from the circular buffer and
    * put it to the audio device
    */
    rtp->audio_buf.Retrieve_Packet((unsigned short *)buf, size); 
    rtp->audio_dev.Playback_To_Speaker( buf, size);
  }
  
  pthread_mutex_unlock(&rtp->mutex_rtp);
  
  return NULL;
}

void * RtpReceiveThread(void *p)
{
  rtp_info_t *rtp = (rtp_info_t *) p;
  int sock_rtp = 0, sock_rtcp= 0, sock_rtp_fec = 0, sock_rtcp_fec = 0;
  rtperror the_err;
  fd_set selection;
  int maxfd = 0;
  char theload[8192];
  int loaduse;
  rtp_packet rtppkt;
  struct timeval initialtime, socktv;
  struct timespec next;
  
  if ( rtp == NULL ) {
    debug("MyReceiveThread", "parameter is null\n");
    return NULL;
  }
  
  debug("RTPReceiveThread", "started\n");
  
  pthread_mutex_lock(&rtp->mutex_rtp);
  
  debug("RTPReceiveThread", "got the lock\n");
  
  /* Get RTP and RTCP sockets */
  
  the_err = RTPSessionGetRTPSocket(rtp->cid, 
    &sock_rtp);
  
  if ( the_err != RTP_OK ) {
    error("RTPReceiveThread", "RTPSessionGetRtpSocket %s\n", 
      RTPDebugStr());
    rtp->oTerminateReceive = 1;
    pthread_mutex_unlock(&rtp->mutex_rtp);
    return NULL;
  }
  else {
    debug("RTPReceiveThread", "RTP socket is %d\n", sock_rtp);
  }
  
  if ( maxfd < sock_rtp ) {
    maxfd = sock_rtp;
  }
  
  the_err = RTPSessionGetRTCPSocket(rtp->cid,
    &sock_rtcp);
  
  if ( the_err != RTP_OK ) {
    error("RTPReceiveThread", "RTPSessionGetRtcpSocket %s\n", 
      RTPDebugStr());
    rtp->oTerminateReceive = 1;
    pthread_mutex_unlock(&rtp->mutex_rtp);
    return NULL;
  }
  else {
    debug("RTPReceiveThread", "RTCP socket is %d\n", sock_rtcp);
  }
  
  if ( maxfd < sock_rtcp ) {
    maxfd = sock_rtcp;
  }
  
  the_err = RTPSessionGetRTPSocket(rtp->cid_fec, 
    &sock_rtp_fec);
  
  if ( the_err != RTP_OK ) {
    error("RTPReceiveThread", "RTPSessionGetRtpSocket %s\n", 
      RTPDebugStr());
    rtp->oTerminateReceive = 1;
    pthread_mutex_unlock(&rtp->mutex_rtp);
    return NULL;
  }
  else {
    debug("RTPReceiveThread", "RTP socket is %d\n", sock_rtp_fec);
  }
  
  if ( maxfd < sock_rtp_fec ) {
    maxfd = sock_rtp_fec;
  }
  
  the_err = RTPSessionGetRTCPSocket(rtp->cid_fec,
    &sock_rtcp);
  
  if ( the_err != RTP_OK ) {
    error("RTPReceiveThread", "RTPSessionGetRtcpSocket %s\n", 
      RTPDebugStr());
    rtp->oTerminateReceive = 1;
    pthread_mutex_unlock(&rtp->mutex_rtp);
    return NULL;
  }
  else {
    debug("RTPReceiveThread", "RTCP socket is %d\n", sock_rtcp_fec);
  }
  
  if ( maxfd < sock_rtcp_fec ) {
    maxfd = sock_rtcp_fec;
  }
  
  gettimeofday(&initialtime, NULL);
  
  /* While call not closed */
  
  while ( !rtp->oTerminateReceive ) {
    
    next = timespec_delay(initialtime, 15/1000.0 ) ; /* 10 millisec */
    /* releases mutex before wait */
    pthread_cond_timedwait(&rtp->cond_rtp, &rtp->mutex_rtp, &next);
    if ( rtp->oTerminateReceive ) {
      break; /* from while loop */
    }
    
    gettimeofday(&initialtime, NULL);
    
    FD_ZERO(&selection);
    FD_SET(sock_rtp, &selection);
    FD_SET(sock_rtcp, &selection);
    FD_SET(sock_rtp_fec, &selection);
    FD_SET(sock_rtcp_fec, &selection);
    
    socktv.tv_sec = 0;
    socktv.tv_usec = 0; /* 0 millisec */
    
    /* Recv UDP packets from RTP/RTCP socket */
    while ( select(maxfd + 1, &selection, NULL, NULL, &socktv) > 0 ) {
      if ( FD_ISSET (sock_rtp, &selection)) {
        
        loaduse = sizeof(theload);
        
        the_err = RTPReceive(rtp->cid, 
          sock_rtp, 
          theload, 
          &loaduse);
        
        /* make sure that the packet is not sent by me */
        if ( the_err == RTP_PACKET_LOOPBACK ) {
          debug("RTPReceiveThread", "Got my own RTP packet\n");
        }
        else if ( the_err != RTP_OK) {
          debug("RTPReceiveThread", "RTPReceive %s\n", RTPDebugStr());
        }
        else {
          
          rtppkt = RTPGetRTPPacket(theload, loaduse);
          
          /* First, some details about the packet */
          
          /* debug("RTPReceive",
            "RTP len=%d, m&pt=0x%x, ts=%lu, seq=%lu\n",
            (int)loaduse,
            (unsigned long) rtppkt.RTP_header->mpt,
            (unsigned long) rtppkt.RTP_header->ts,
            (unsigned long) rtppkt.RTP_header->seq);
          */
          
            /*
            * put the received buffer into the audio
            * circular buffer.
          */
          
          if ( ((int)(rtppkt.RTP_header->mpt & 0x07f)) == 96) {
	    fec_recovery(&rtppkt, 0, rtp->timestamp, & rtp->audio_buf, & rtp->audio_dev);
          }
        }
      }
      
      if ( FD_ISSET(sock_rtcp, &selection)) {
        loaduse = sizeof(theload);
        
        the_err = RTPReceive(rtp->cid, 
          sock_rtcp, 
          theload, 
          &loaduse);
      }
      
      if ( FD_ISSET (sock_rtp_fec, &selection)) {
        
        loaduse = sizeof(theload);
        
        the_err = RTPReceive(rtp->cid_fec, 
          sock_rtp_fec, 
          theload, 
          &loaduse);
        
        /* make sure that the packet is not sent by me */
        if ( the_err == RTP_PACKET_LOOPBACK ) {
          debug("RTPReceiveThread", "Got my own RTP packet\n");
        }
        else if ( the_err != RTP_OK) {
          debug("RTPReceiveThread", "RTPReceive %s\n", RTPDebugStr());
        }
        else {
          rtppkt = RTPGetRTPPacket(theload, loaduse);
          
          /* First, some details about the packet */
          
          /* debug("RTPReceive",
            "RTP FEC len=%d, m&pt=0x%x, ts=%lu, seq=%lu\n",
            (int)loaduse,
            (unsigned long) rtppkt.RTP_header->mpt,
            (unsigned long) rtppkt.RTP_header->ts,
            (unsigned long) rtppkt.RTP_header->seq);
          */
          /*
            * Use the fec packet to recover data if necessary
          */
          
          if ( ((int)(rtppkt.RTP_header->mpt & 0x07f)) == 97) { 
            fec_recovery(&rtppkt, 1, rtp->timestamp, & rtp->audio_buf,
              & rtp->audio_dev);
          }
        }
      }
      
      if ( FD_ISSET(sock_rtcp_fec, &selection)) {
        loaduse = sizeof(theload);
        
        the_err = RTPReceive(rtp->cid_fec, 
          sock_rtcp_fec, 
          theload, 
          &loaduse);
      }
      
      FD_ZERO(&selection);
      FD_SET(sock_rtp, &selection);
      FD_SET(sock_rtcp, &selection);
      FD_SET(sock_rtp_fec, &selection);
      FD_SET(sock_rtcp_fec, &selection);
    } /* while select() succeeds */
    
  } /* While the thread exists */
  
  pthread_mutex_unlock(&rtp->mutex_rtp);
  return NULL;
}

/************************** END OF FILE ******************************** */

