# Columbia SIP makefile fragment for siprtpua
# Copyright 1999-2000 by Columbia University; all rights reserved 

siprtpua_dir := siprtpua

# SIPRTPUA_MODULES := docs

# Include the description for each sub-module
# include $(patsubst %,$(topsrcdir)/$(siprtpua_dir)/%/module.mk,$(SIPRTPUA_MODULES))


SIPRTPUA_BIN := \
  $(siprtpua_dir)/siprtpua

SIPRTPUA_SRC := \
  $(siprtpua_dir)/main.cpp \
  $(siprtpua_dir)/command.cpp \
  $(siprtpua_dir)/audioBuffer.cpp \
  $(siprtpua_dir)/audioDevice.cpp \
  $(siprtpua_dir)/adpcm64_coder.c \
  $(siprtpua_dir)/adpcm64_decoder.c \
  $(siprtpua_dir)/fec.cpp \
  $(siprtpua_dir)/sched.c

SIPRTPUA_HEADERS := \
  $(siprtpua_dir)/audioBuffer.h \
  $(siprtpua_dir)/audioDevice.h \
  $(siprtpua_dir)/rtp.h \
  $(siprtpua_dir)/pcmu_l.h \
  $(siprtpua_dir)/types.h \
  $(siprtpua_dir)/command.h \
  $(siprtpua_dir)/sched.h \
  $(siprtpua_dir)/stdtypes.h


SIPRTPUA_MISC := \
  $(siprtpua_dir)/module.mk \
  $(siprtpua_dir)/siprtpua.dsp

SIPRTPUA_MISCDIST := \
  $(siprtpua_dir)/README \
  $(siprtpua_dir)/siprtpua.license


SIPRTPUA_NEEDED_LIBRARIES_LIST := \
	$(siprtpua_dir)/siprtpua-needed-shared-libraries

SIPRTPUA_NEEDED_LIBRARIES_SOURCES := \
	$(siprtpua_dir)/siprtpua-needed-shared-libraries-sources

SIPRTPUA_OBJ_CPP := \
	$(patsubst %.cpp,%.o,$(filter %.cpp,$(SIPRTPUA_SRC)))

SIPRTPUA_OBJ_C := \
	$(patsubst %.c,%.o,$(filter %.c,$(SIPRTPUA_SRC)))

SIPRTPUA_OBJ := $(SIPRTPUA_OBJ_CPP) $(SIPRTPUA_OBJ_C)

SIPRTPUA_DEP := \
        $(SIPRTPUA_OBJ:.o=.d)

SIPRTPUA_MISCCLEAN := \
	$(SIPRTPUA_NEEDED_LIBRARIES_LIST)

# Include directories required locally.
$(siprtpua_dir)-CFLAGS := $(SIPAPI_INC) $(LIBSIP_INC) $(LIBCINE_INC) \
                          $(LIBRTP_INC)

SIPRTPUA_LIBS := $(SIPAPI_LIB) $(LIBSIP_LIB) $(DBAPI_LIB) $(LIBCINE_LIB) \
	$(LIBRESPARSE) $(LIBDICT_LIB) $(LIBRTP_LIB) \
	$(LDAPLIBS) $(DBAPI_LIB) \
        $(MYSQLLIBS) $(ZLIBLIBS) $(RESOLVLIBS) $(NISLIBS) $(SOCKLIBS)
SIPRTPUA_DEPENDENCIES := $(SIPAPI) $(LIBRTP)
SIPRTPUA_REQS :=


$(SIPRTPUA_NEEDED_LIBRARIES_SOURCES): $(siprtpua_dir)/siprtpua
	@echo "Building list of non-system shared libraries..."
	rm -f $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES)
	$(LDD) $(siprtpua_dir)/siprtpua | \
		sed -n -e '/not found/ s/^/XXX SET LD_LIBRARY_PATH: /p;' \
			-e '/=>/ s|^.* \([^ ]*/[^ ]*\).*$$|\1|p' | \
			egrep -v '^(/usr)?/lib/' \
			> $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES); \
			true
	if grep XXX $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES) > /dev/null; \
	then \
		echo "Missing shared library; can't continue"; \
		cat $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES); \
		rm -f $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES); \
		exit 1; \
	fi

$(SIPRTPUA_NEEDED_LIBRARIES_LIST): $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES)
	@echo "Building local names of non-system shared libraries..."
	rm -f $(SIPRTPUA_NEEDED_LIBRARIES_LIST)
	@siprtpua_needed_libraries=`cat $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES)`; \
	for library in $$siprtpua_needed_libraries; \
	do\
		echo $(siprtpua_dir)/`basename $$library`; \
	done > $(SIPRTPUA_NEEDED_LIBRARIES_LIST)

# Do not add this --- all: $(SIPRTPUA)
all: siprtpua
.PHONY: siprtpua
siprtpua: $(SIPRTPUA_REQS) $(SIPRTPUA_DEP) $(SIPRTPUA_BIN)

# SIPRTPUA uses libsip++ == SIPAPI

$(siprtpua_dir)/siprtpua: $(SIPRTPUA_OBJ) $(SIPRTPUA_DEPENDENCIES)
	@echo "creating application $@..."
	$(CXX) -o $@ -w $(SIPRTPUA_INCS) $(SIPRTPUA_OBJ) \
	$(LDFLAGS) $(SIPRTPUA_LIBS) $(LIBS)

BIN	  += $(SIPRTPUA_BIN)
SRC	  += $(SIPRTPUA_SRC)
MISC      += $(SIPRTPUA_MISC)
MISCDIST  += $(SIPRTPUA_MISCDIST)
MISCCLEAN += $(SIPRTPUA_OTHER_MISCCLEAN)

SIPRTPUA_DIST := $(SIPRTPUA_SRC) $(SIPRTPUA_HEADERS) $(SIPRTPUA_MISC) $(SIPRTPUA_MISCDIST)

SIPRTPUA_BINDIST := $(SIPRTPUA_MISCDIST) $(SIPRTPUA_BIN)

SIPRTPUA_MISC_DLLS := 

SIPRTPUA_BIN_WIN32 := \
  $(siprtpua_dir)/Debug/siprtpua.exe \
  $(siprtpua_dir)/Release/siprtpua.exe

SIPRTPUA_BINDIST_WIN32 := $(SIPRTPUA_BIN_WIN32) $(SIPRTPUA_MISCDIST) \
		      $(SIPRTPUA_MISC_DLLS)

.PHONY: siprtpua-dist siprtpua-bindist siprtpua-bindist-win32 copy-siprtpua-shared-libraries

copy-siprtpua-shared-libraries: $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES)
	@echo "Copying non-system shared libraries..."
	@siprtpua_needed_libraries=`cat $(SIPRTPUA_NEEDED_LIBRARIES_SOURCES)`; \
	if test "x$$siprtpua_needed_libraries" != "x"; \
	then\
		echo cp $$siprtpua_needed_libraries $(siprtpua_dir); \
		cp $$siprtpua_needed_libraries $(siprtpua_dir); \
	fi


SIPRTPUA_DIST_NAME := siprtpua-$(VERSION)

SIPRTPUA_DIST_FILES := \
	$(TOP_DIST) \
	$(LIBCINE_DIST) $(LIBSIP_DIST) $(LIBRESPARSE_DIST) \
	$(LIBDICT_DIST) \
	$(DBAPI_DIST) \
	$(WIN32_DIST) \
	$(SIPAPI_DIST) \
	$(SIPRTPUA_DIST) \
	$(SIPRTPUA_DOC_DIST) \
	$(DOC_DIST) \
	$(EXTRA_DIST_FILES)

siprtpua-dist: win32-dist
	$(MKDIST) $(topsrcdir) $(SIPRTPUA_DIST_NAME) $(SIPRTPUA_DIST_FILES)

SIPRTPUA_BINDIST_NAME := siprtpua-$(VERSION)-$(SYSNAME)

SIPRTPUA_BINDIST_FILES := \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		\
		$(SIPRTPUA_BINDIST) \
		$(SIPRTPUA_DOCS_BINDIST) \
		$(DOC_BINDIST) \
		`cat $(SIPRTPUA_NEEDED_LIBRARIES_LIST)` \
		$(EXTRA_BINDIST_FILES)

siprtpua-bindist:: $(SIPRTPUA)  copy-siprtpua-shared-libraries \
			$(SIPRTPUA_NEEDED_LIBRARIES_LIST)
	$(MKDIST) $(topsrcdir) $(SIPRTPUA_BINDIST_NAME) $(SIPRTPUA_BINDIST_FILES)


SIPRTPUA_BINDIST_WIN32 := $(SIPRTPUA_MISCDIST) \
		$(siprtpua_dir)/Debug/siprtpua.exe \
                $(siprtpua_dir)/Release/siprtpua.exe


SIPRTPUA_BINDIST_WIN32_NAME := siprtpua-$(VERSION)-Win32

SIPRTPUA_BINDIST_WIN32_FILES := \
		$(TOP_BINDIST) \
		$(LIBCINE_BINDIST) \
		$(LIBDICT_BINDIST) \
		\
		$(SIPRTPUA_BINDIST_WIN32) \
		$(SIPRTPUA_DOCS_BINDIST) \
		$(DOC_BINDIST) \
		$(EXTRA_BINDIST_FILES)

siprtpua-bindist-win32::
	$(MKDIST_WIN32) $(topsrcdir) $(SIPRTPUA_BINDIST_WIN32_NAME) \
	$(SIPRTPUA_BINDIST_WIN32_FILES)

siprtpua-help: 
	@echo "SIPRTPUA (Test User Agent) Make Targets"
	@echo " make siprtpua                Make siprtpua executable on Unix."
	@echo " make siprtpua-dist           Make siprtpua source distribution."
	@echo " make siprtpua-bindist        Make siprtpua binary distribution for Unix."
	@echo " make siprtpua-bindist-win32  Make siprtpua binary distribution for Win32."

help: siprtpua-help
